/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog;

import java.io.File;
import java.io.FilenameFilter;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.catalog.internal.ui.Policy;
import net.refractions.udig.catalog.ui.UDIGImportPage;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.PlatformUI;

/**
 * A Wizard for importing files
 * @author jeichar
 * @since 0.9.0
 */
public class FileChooserWizardPage extends WizardPage implements KeyListener,UDIGImportPage {

	private static final String LASTDIRECTORY = "LastDirectory"; //$NON-NLS-1$
	Combo root;
	CheckboxTableViewer fileViewer;
	private boolean canFlip=false;

	public String getId() {
		// TODO Auto-generated method stub
		return null;
	}
	
	public boolean canProcess(Object object) {
        URL url = CatalogPlugin.locateURL(object);
        try {
            new File(url.toURI());
            return true;
        } catch (URISyntaxException e) {
            return false;
        }
	}
	
	/**
	 * Construct <code>FileChooserWizardPage</code>.
	 */
	public  FileChooserWizardPage() {
		super(Policy.bind("FileChooserWizardPage.title")); //$NON-NLS-1$
	}

	/**
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent,SWT.NULL);
        composite.setLayout(new GridLayout(3, false));
        
        // add url
        createDirectoryInput(composite);
        
        Button button = new Button(composite, SWT.PUSH);
        button.setLayoutData(new GridData());

        button.setText(Policy.bind("FileChooserWizardPage.browseButton.text")); //$NON-NLS-1$
		
		Composite fileComposite=new Composite(composite, SWT.NULL);
		GridData data=new GridData(SWT.FILL, SWT.FILL,true, true);
		data.horizontalSpan=2;
		fileComposite.setLayoutData(data);
//		fileComposite.setLayout(new GridLayout(2, true));
		fileComposite.setLayout(new GridLayout(1, true));
//		createDirectoryTree(fileComposite);
		createFileViewer(fileComposite);
		
        addButtonListener(button);
        
        setControl(composite);
        setPageComplete(true);
	}

	/**
	 * TODO summary sentence for createFileViewer ...
	 * 
	 * @param fileComposite
	 */
	private void createFileViewer(Composite fileComposite) {
		Table table=new Table(fileComposite, SWT.CHECK|SWT.BORDER);
		fileViewer=new CheckboxTableViewer(table);
		fileViewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL,true, true));
		fileViewer.setContentProvider(new ArrayContentProvider());
		fileViewer.setLabelProvider(new LabelProvider());
		fileViewer.addSelectionChangedListener(new ISelectionChangedListener(){

			public void selectionChanged(SelectionChangedEvent event) {
				setPageComplete(isPageComplete());
			}

		});
		setFileViewerInput();
	}

	/**
	 * TODO summary sentence for addButtonListener ...
	 * 
	 * @param button
	 */
	private void addButtonListener(Button button) {
		button.addSelectionListener(new SelectionAdapter(){
            public void widgetSelected( SelectionEvent e ) {
                Display display = Display.getCurrent();
                if (display == null) { // not on the ui thread?
                    display = Display.getDefault();
                }
                
                DirectoryDialog dialog = new DirectoryDialog(display.getActiveShell());
                dialog.setText( Policy.bind("FileChooserWizardPage.directoryDialog.text") ); //$NON-NLS-1$
               
                String path = dialog.open();
                if(path == null)
                    path = ""; //$NON-NLS-1$
                root.setText( path );

				PlatformUI.getWorkbench().getPreferenceStore().putValue(LASTDIRECTORY, path);
				setFileViewerInput();
                setPageComplete(isPageComplete());
            }
        });
	}

	private void createDirectoryInput(Composite composite) {
		Label label = new Label( composite, SWT.NONE );
		label.setText(Policy.bind("FileChooserWizardPage.rootDirectory.label.text") ); //$NON-NLS-1$
		label.setToolTipText( Policy.bind("FileChooserWizardPage.rootDirectory.label.tooltip") ); //$NON-NLS-1$
		label.setLayoutData( new GridData(SWT.END, SWT.DEFAULT, false, false ) );
		Combo combo = new Combo( composite, SWT.BORDER );
		combo.setLayoutData( new GridData(GridData.FILL_HORIZONTAL) );
		String last=PlatformUI.getWorkbench().getPreferenceStore().getString(LASTDIRECTORY);
		if( last.length()==0)
			last=System.getProperty("user.dir"); //$NON-NLS-1$
		combo.setText( last );
		root = combo;
		combo.addKeyListener( this );
	}

	void setFileViewerInput() {
		File file=new File(root.getText());
		fileViewer.setInput(file.list(new FilenameFilter(){

			public boolean accept(File arg0, String arg1) {
				if( arg1.startsWith(".") || !new File(arg0,arg1).isFile()) //$NON-NLS-1$
					return false;
				if( arg1.endsWith(".shp") || arg1.endsWith(".udig") ) //$NON-NLS-1$ //$NON-NLS-2$
					return true;
				return false;
			}
			
		}));
	}
	
	@Override
	public boolean isPageComplete() {
		return !fileViewer.getSelection().isEmpty();
	}
    /**
     * @see org.eclipse.swt.events.KeyListener#keyReleased(org.eclipse.swt.events.KeyEvent)
     */
    public void keyReleased( KeyEvent e ) {
		setFileViewerInput();
    }
	
    public void keyPressed( KeyEvent e ) {
		//do nothing
	}

    /*
     * @see net.refractions.udig.catalog.ui.UDIGImportPage#getResources(org.eclipse.core.runtime.IProgressMonitor)
     */
    public List<IResolve> getResources( IProgressMonitor monitor ) throws Exception {
        List<IResolve> resources=new ArrayList<IResolve>();
        Object[] checked = fileViewer.getCheckedElements();
        monitor.beginTask( Policy.bind("FileChooserWizardPage.taskname"), checked.length ); //$NON-NLS-1$
        int worked=0;
		for (Object object: checked ) {        
            if( monitor.isCanceled() ) return null;
            
			String string=(String) object;
            try {
				String rootText=this.root.getText();
				if( !rootText.startsWith("file") ) //$NON-NLS-1$
					rootText="file://"+rootText; //$NON-NLS-1$
				if( !rootText.endsWith("/") && !string.startsWith("/") ) //$NON-NLS-1$ //$NON-NLS-2$
					rootText+="/"; //$NON-NLS-1$
                
                URL url = new URL(rootText+string);
                
                monitor.subTask( Policy.bind("FileChooserWizardPage.subtask.name") + string );                 //$NON-NLS-1$
                List<IService> aquire = CatalogPlugin.getDefault().getServiceFactory().aquire( url );
                if( aquire.size() == 1 ){
                    IService service = aquire.get(0);
                    List<? extends IGeoResource> list = service.members( monitor );
                    if( list.size() == 1 ){
                        resources.addAll( list );
                    }
                    else {
                        resources.add( service );
                    }
                }
                else {
                    resources.addAll( aquire );
                }
			} catch (MalformedURLException e) {                
				// do nothing.
			}
            monitor.worked( worked++ );            
		}
		return resources;
	}

	public void setCanFlipToNextPage(boolean value) {
		canFlip=value;
	}
	
	@Override
	public boolean canFlipToNextPage() {
		return canFlip;
	}

    
}
