/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.google;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.GZIPInputStream;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.ICatalog;
import net.refractions.udig.catalog.ICatalogInfo;
import net.refractions.udig.catalog.IResolve;
import net.refractions.udig.catalog.IResolveChangeEvent;
import net.refractions.udig.catalog.IResolveChangeListener;
import net.refractions.udig.catalog.IService;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.util.ListenerList;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import com.vividsolutions.jts.geom.Envelope;

public class GoogleCatalog extends ICatalog {

    private Throwable msg;
    URL url = null;
    
    /**
     * Construct <code>GoogleCatalog</code>.
     *
     */
    public GoogleCatalog() {
        catalogListeners = new ListenerList();
        try {
            url = new URL("http://udig.refractions.net/search/google-xml.php?"); //$NON-NLS-1$
        } catch (MalformedURLException e) {
            msg = e;
            GooglePlugin.log(null,e);
        }
    }
    private ListenerList catalogListeners;
    
    /*
     * @see net.refractions.udig.catalog.ICatalog#add(net.refractions.udig.catalog.IService)
     */
    public void add( IService service ) throws UnsupportedOperationException {
        throw new UnsupportedOperationException();
    }

    /*
     * @see net.refractions.udig.catalog.ICatalog#remove(net.refractions.udig.catalog.IService)
     */
    public void remove( IService service ) throws UnsupportedOperationException {
        throw new UnsupportedOperationException();
    }

    /*
     * @see net.refractions.udig.catalog.ICatalog#replace(java.net.URL, net.refractions.udig.catalog.IService)
     */
    public void replace( URL id, IService service ) throws UnsupportedOperationException {
        throw new UnsupportedOperationException();
    }

    /*
     * Required adaptions:
     * <ul>
     * <li>ICatalogInfo.class
     * <li>List.class <IService>
     * </ul>
     * @see net.reurl.fractions.udig.catalog.IResolve#resolve(java.lang.Class, org.eclipse.core.runtime.IProgressMonitor)
     */
    public <T> T resolve( Class<T> adaptee, IProgressMonitor monitor ) throws IOException{
        if(adaptee == null)
            return null;
        if(adaptee.isAssignableFrom(ICatalogInfo.class)){
            return adaptee.cast(getInfo(monitor));
        }
        if(adaptee.isAssignableFrom(List.class)){
            return adaptee.cast(members(monitor));
        }
        return null;
    }

    private ICatalogInfo info = null;
    /*
     * @see net.refractions.udig.catalog.ICatalog#getInfo(org.eclipse.core.runtime.IProgressMonitor)
     */
    public ICatalogInfo getInfo(IProgressMonitor monitor){
        if(info != null){
            info = new GoogleICatalogInfo();
        }
        return info;
    }

    private class GoogleICatalogInfo extends ICatalogInfo{
        GoogleICatalogInfo(){
            this.title = Policy.bind("GoogleCatalog.title"); //$NON-NLS-1$
            this.description = Policy.bind("GoogleCatalog.description"); //$NON-NLS-1$
            this.source = url;
            this.keywords = new String[]{"Catalog","Google","Refractions Research","Search"}; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
        }
    }
    
    /*
     * @see net.refractions.udig.catalog.ICatalog#find(java.net.URL)
     */
    public List<IResolve> find( URL id, IProgressMonitor monitor ) {
        return new LinkedList<IResolve>();
    }


	@Override
	public List<IService> findService(URL query) {
		return new ArrayList<IService>();
	}

    /*
     * hits the server using soap ...
     * 
     * @see net.refractions.udig.catalog.ICatalog#search(java.lang.String, com.vividsolutions.jts.geom.Envelope, org.eclipse.core.runtime.IProgressMonitor)
     */
    public List<IResolve> search( String pattern, Envelope bbox, IProgressMonitor monitor ) throws IOException{
        
        monitor.beginTask(Policy.bind("GoogleCatalog.searchMessage")+pattern, IProgressMonitor.UNKNOWN); //$NON-NLS-1$
        
        List<IResolve> results = new ArrayList<IResolve>();
        
        if (bbox == null || bbox.isNull()) {
            bbox = new Envelope(-180, 180, -90, 90);
        }

        double xmin = bbox.getMinX();
        double xmax = bbox.getMaxX();
        
        double ymin = bbox.getMinY();
        double ymax = bbox.getMaxY();
        
        //keywords=bird&xmin=-180&ymin=-90&xmax=180&ymax=90
        String urlString = url.toExternalForm();
        
        urlString = urlString.concat("keywords="+URLEncoder.encode( pattern, "UTF-8")); //$NON-NLS-1$ //$NON-NLS-2$
        urlString = urlString.concat("&xmin="+xmin+"&ymin="+ymin+"&xmax="+xmax+"&ymax="+ymax); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
        
        if (monitor.isCanceled()) {
            return results;
        }
        
        URL finalURL = new URL(urlString);
        
        URLConnection connection = finalURL.openConnection();
        
        connection.addRequestProperty("Accept-Encoding", "gzip"); //$NON-NLS-1$ //$NON-NLS-2$

        InputStream inputStream = connection.getInputStream();
        
        if (connection.getContentEncoding() != null && connection.getContentEncoding().indexOf("gzip") != -1) { //$NON-NLS-1$
            inputStream = new GZIPInputStream(inputStream);
        }
                
        SAXBuilder builder = new SAXBuilder(false);
        Document document = null;
        
        if (monitor.isCanceled()) {
            return results;
        }
        
        try {
            document = builder.build(inputStream);
        } catch (JDOMException e) {
            throw (IOException) new IOException(Policy.bind("GoogleCatalog.parseError")).initCause(e); //$NON-NLS-1$
        }
        
        if (monitor.isCanceled()) {
            return results;
        }
        
        for (Object object : document.getRootElement().getChildren("r")) { //$NON-NLS-1$
            
            if (monitor.isCanceled()) {
                return results;
            }
            
            Element rElement = (Element) object;
            try{
            String name = rElement.getChildText("name"); //$NON-NLS-1$
            String title = rElement.getChildText("title"); //$NON-NLS-1$
            String description = rElement.getChildText("description"); //$NON-NLS-1$
            URL onlineResource = new URL(rElement.getChildText("onlineresource")); //$NON-NLS-1$
            String serverType = rElement.getChildText("servertype"); //$NON-NLS-1$
            String serverVersion = rElement.getChildText("serverversion"); //$NON-NLS-1$
            URL id = new URL(rElement.getChildText("id")); //$NON-NLS-1$
            
            OGCLayer layer = new OGCLayer(name, title, description, onlineResource, serverType, serverVersion, id);
            
            results.add(GoogleResource.getResource(layer));
            }catch(Exception e){
            	GooglePlugin.log("eror creating OGCLayer: "+rElement.getChildText("name"), e);
            }
        }
       
        return results;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#canResolve(java.lang.Class)
     */
    public <T> boolean canResolve( Class<T> adaptee ) {
        return adaptee!=null && (adaptee.isAssignableFrom(ICatalogInfo.class) || adaptee.isAssignableFrom(List.class));
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#members(org.eclipse.core.runtime.IProgressMonitor)
     */
    public List< ? extends IResolve> members( IProgressMonitor monitor ) throws IOException{
        return search("",new Envelope(),monitor); //$NON-NLS-1$
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getStatus()
     */
    public Status getStatus() {
        return msg == null?Status.CONNECTED:Status.BROKEN;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getMessage()
     */
    public Throwable getMessage() {
        return msg;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getIdentifier()
     */
    public URL getIdentifier() {
        return url;
    }
    void fire( IResolveChangeEvent event ) {
        Object[] listeners = catalogListeners.getListeners();        
        if( listeners.length == 0 ) return;
        
        for (int i = 0; i < listeners.length; ++i) {
            try {
                ((IResolveChangeListener) listeners[i]).changed( event );
            }
            catch( Throwable die) {
                CatalogPlugin.log( null, new Exception(die));
            }
        }
    }
    
    /**
     * 
     * @see net.refractions.udig.catalog.ICatalog#addCatalogListener(net.refractions.udig.catalog.ICatalog.ICatalogListener)
     * @param listener
     */
    public void addCatalogListener( IResolveChangeListener listener ) {
        catalogListeners.add(listener);
    }

    /**
     * 
     * @see net.refractions.udig.catalog.ICatalog#removeCatalogListener(net.refractions.udig.catalog.ICatalog.ICatalogListener)
     * @param listener
     */
    public void removeCatalogListener( IResolveChangeListener listener ) {
        catalogListeners.remove(listener);
    }

}
