/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.internal;

import java.io.Serializable;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.IServiceFactory;
import net.refractions.udig.catalog.ServiceExtension;
import net.refractions.udig.core.internal.ExtensionPointProcessor;
import net.refractions.udig.core.internal.ExtensionPointUtil;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;

/**
 * Provides ...TODO summary sentence
 * <p>
 * TODO Description
 * </p>
 * @author David Zwiers, Refractions Research
 * @since 0.6
 */
public class ServiceFactoryImpl implements IServiceFactory{

    /**
     * TODO summary sentence for aquire ...
     * 
     * @see net.refractions.udig.catalog.IServiceFactory#aquire(java.net.URI, java.util.Map)
     * @param id
     * @param params
     * @return
     */
    public List<IService> aquire(final URL id, final Map<String,Serializable> params ) {
        final List<IService> services = new LinkedList<IService>();

        // load services
        ExtensionPointUtil.process( CatalogPlugin.getDefault(), "net.refractions.udig.catalog.ServiceExtension",new ExtensionPointProcessor(){ //$NON-NLS-1$
            /**
             * Attempt to construct a service, and add to the list if available.
             * <p>
             * Note: ExtentionPointUtil will log exceptions against provided plugin.
             * </p>
             * @see net.refractions.udig.core.internal.ExtensionPointProcessor#process(org.eclipse.core.runtime.IExtension, org.eclipse.core.runtime.IConfigurationElement)
             * @param extension
             * @param element
             * @return
             */
            public void process( IExtension extension, IConfigurationElement element ) throws Exception {                
                ServiceExtension se = (ServiceExtension)element.createExecutableExtension("class"); //$NON-NLS-1$                   
                IService service = se.createService(id,params);
                if(service!=null)
                    services.add(service);                
            }
        });
        
        return services;
    }
    /**
     * TODO summary sentence for aquire ...
     * 
     * @see net.refractions.udig.catalog.IServiceFactory#aquire(java.util.Map)
     * @param params
     * @return
     */
    public List<IService> aquire( Map<String,Serializable> params ) {
        return aquire(null,params);
    }
    /**
     * Aquire IService handles generated by all ServiceExtentions that
     * think they can handle the provided target url.
     * <p>
     * Note: Just because a target is created does *NOT* mean it will actually
     * work. You can check the handles in the usual manner (ask for their info)
     * after you get back this list.
     * </p> 
     * 
     * @see net.refractions.udig.catalog.IServiceFactory#aquire(java.net.URL)
     * @param target
     * @return
     */
    public List<IService> aquire(final URL target ) {
        final List<Map<String,Serializable>> maps = new LinkedList<Map<String,Serializable>>();
        //load maps
        ExtensionPointUtil.process( CatalogPlugin.getDefault(), "net.refractions.udig.catalog.ServiceExtension",new ExtensionPointProcessor(){ //$NON-NLS-1$
            /**
             * TODO summary sentence for process ...
             * 
             * @see net.refractions.udig.core.internal.ExtensionPointProcessor#process(org.eclipse.core.runtime.IExtension, org.eclipse.core.runtime.IConfigurationElement)
             * @param extension
             * @param element
             * @return
             */
            public void process( IExtension extension, IConfigurationElement element ) throws Exception {
                ServiceExtension se = (ServiceExtension)element.createExecutableExtension("class"); //$NON-NLS-1$
                Map<String,Serializable> m = se.createParams(target);
                if(m!=null)
                    maps.add(m);
            }
        });
        
        List<IService> r = new LinkedList<IService>();
        Iterator<Map<String,Serializable>> i = maps.iterator();
        while(i.hasNext()){
            List<IService> o = aquire(i.next());
            if(o!=null && !o.isEmpty())
                r.addAll(o);
        }
        return r;
    }
}
