package net.refractions.udig.catalog.internal.ui;

import java.io.IOException;
import java.util.List;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.ICatalog;
import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IResolve;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.ui.CatalogUIPlugin;
import net.refractions.udig.catalog.ui.DataSourceSelectionPage;
import net.refractions.udig.catalog.ui.IDataWizard;
import net.refractions.udig.catalog.ui.UDIGImportPage;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

public class CatalogImportWizard extends IDataWizard implements IImportWizard{

    @Override
    public void init( IWorkbench workbench, IStructuredSelection selection ) {
        super.init(workbench, selection);
        
        setWindowTitle( Policy.bind( "CatalogImportWizard.title" ) );  //$NON-NLS-1$
        setDefaultPageImageDescriptor( Images.getDescriptor( ImageConstants.DATA_WIZBAN ));
    }
    
    
	@Override
	public boolean canFinish() {
		IWizardPage page = getContainer().getCurrentPage();
		return page instanceof UDIGImportPage;
	}
	
	@Override
	public boolean performFinish() {
		UDIGImportPage page = (UDIGImportPage) getContainer().getCurrentPage();
		
		//if import page returns a new page, then an error occured
		IWizardPage next = page.getNextPage(); 
		if (next != null) {
			//error, 
			getContainer().showPage(next);
			return false;
		}
		
		new CatalogImport(page).schedule();
		return true;
	}
	
	@Override
	protected WizardPage[] getPrimaryPages() {
		return new WizardPage[]{new DataSourceSelectionPage()};
	}
	
	class CatalogImport extends Job {

		UDIGImportPage page;
		
		CatalogImport(UDIGImportPage page) {
			super(Policy.bind("CatalogImportWizard.importJobName"));  //$NON-NLS-1$
			this.page = page;
		}
		
		@Override
		protected IStatus run(IProgressMonitor monitor) {
			//resolve each resource to an IService and add to catalog
			ICatalog catalog = CatalogPlugin.getDefault().getLocalCatalog();
			IService first = null;
			
			List<IResolve> resources = null;
			try {
				resources = page.getResources(monitor);
				if (resources == null || resources.isEmpty())
					return Status.CANCEL_STATUS;
			} 
			catch (Exception e1) {
				String msg = e1.getLocalizedMessage();
				CatalogUIPlugin.log(msg, e1);
				return Status.CANCEL_STATUS;
			}
			
			try {
				for ( int i = 0; i < resources.size(); i++) {
					IResolve resolvee = resources.get(i);
					IService service = null;
					try {
						if (resolvee instanceof IService) {
							service = (IService)resolvee;
						}
						else if (resolvee instanceof IGeoResource) {
							service = resolvee.resolve(IService.class, monitor);
						}
					
						if (service != null) {
							catalog.add(service);
							if (first == null) first = service;
						}
					}
					catch(IOException e) {
						String msg = 
							Policy.bind("CatalogImportWizard.serviceErrorMessage"); //$NON-NLS-1$
						CatalogUIPlugin.log(msg, e);			
					}
				}
				
				final IService ffirst = first;
				Display.getDefault().asyncExec(
					new Runnable() {
						public void run() {
							try {
								//set the selection in the catalog view
								IWorkbenchPage page  = PlatformUI.getWorkbench()
									.getActiveWorkbenchWindow().getActivePage();
				                CatalogView catalogView;
								catalogView = (CatalogView) page
										.showView(CatalogView.VIEW_ID);
								
				                if (catalogView != null) {
				                	catalogView.getTreeviewer().setSelection(
				                		new StructuredSelection(ffirst)	
				                	);
				                }
							} 
							catch (PartInitException e) {
								String msg = Policy.bind(
									"CatalogImportWizard.selectionErrorMessage" //$NON-NLS-1$
								); 
								CatalogUIPlugin.log(msg, e);
							}
						}
					}
				);
			} 
			catch (Exception e) {
				Plugin plugin = CatalogUIPlugin.getDefault();
				String msg = 
					Policy.bind("DataImportWizard.resourceErrorMessage"); //$NON-NLS-1$
				CatalogUIPlugin.log(msg, e);
			}
					 
			return Status.OK_STATUS;
		}
	}

}

