package net.refractions.udig.catalog.memory.internal;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IGeoResourceInfo;
import net.refractions.udig.catalog.IResolveChangeEvent;
import net.refractions.udig.catalog.IResolveDelta;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.internal.CatalogImpl;
import net.refractions.udig.catalog.internal.ResolveChangeEvent;
import net.refractions.udig.catalog.internal.ResolveDelta;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureSource;
import org.geotools.data.FeatureStore;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureType;
import org.geotools.geometry.JTS;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import com.vividsolutions.jts.geom.Envelope;

public class MemoryGeoResourceImpl extends IGeoResource {

	/** parent service * */
	MemoryServiceImpl parent;

	/** feature type name * */
	String type;

	/** info object * */
	ScratchResourceInfo info;

	public MemoryGeoResourceImpl(String type, MemoryServiceImpl parent) {
		this.type = type;
		this.parent = parent;
	}

	@Override
	public <T> T resolve(Class<T> adaptee, IProgressMonitor monitor)
			throws IOException {
		if (adaptee == null)
			return null;
		if (adaptee.isAssignableFrom(IService.class))
			return adaptee.cast(parent);
		if (adaptee.isAssignableFrom(IGeoResource.class))
			return adaptee.cast(this);
		if (adaptee.isAssignableFrom(IGeoResourceInfo.class))
			return adaptee.cast(getInfo(monitor));
		if (adaptee.isAssignableFrom(FeatureStore.class))
			return adaptee.cast(parent.getDS().getFeatureSource(type));
		if (adaptee.isAssignableFrom(FeatureSource.class))
			return adaptee.cast(parent.getDS().getFeatureSource(type));

		return null;
	}

	public <T> boolean canResolve(Class<T> adaptee) {
		if (adaptee == null)
			return false;

		return adaptee.isAssignableFrom(IGeoResourceInfo.class)
				|| adaptee.isAssignableFrom(FeatureStore.class)
				|| adaptee.isAssignableFrom(FeatureSource.class)
				|| adaptee.isAssignableFrom(IService.class);
	}

	public Status getStatus() {
		return parent.getStatus();
	}

	public Throwable getMessage() {
		return parent.getMessage();
	}

	public URL getIdentifier() {
		try {
			return new URL(parent.getIdentifier().toString() + "#" + type); //$NON-NLS-1$
		} catch (MalformedURLException e) {
			return parent.getIdentifier();
		}
	}

	@Override
	public IGeoResourceInfo getInfo(IProgressMonitor monitor)
			throws IOException {
		if (info == null) {
			synchronized (parent.getDS()) {
				if (info == null) {
					info = new ScratchResourceInfo();
				}
			}
			IResolveDelta delta = new ResolveDelta(this,
					IResolveDelta.Kind.CHANGED);
			((CatalogImpl) CatalogPlugin.getDefault().getLocalCatalog())
					.fire(new ResolveChangeEvent(this,
							IResolveChangeEvent.Type.POST_CHANGE, delta));
		}
		return info;

	}

	class ScratchResourceInfo extends IGeoResourceInfo {
		FeatureType ft = null;

		ScratchResourceInfo() throws IOException {
			try {
				FeatureSource source = parent.getDS().getFeatureSource(type);
				ft = source.getSchema();
				Envelope e = source.getBounds();
				if (e == null) {
					e = new Envelope();
					for (FeatureReader iter = source.getFeatures().reader(); iter.hasNext();) {
						Feature element = iter.next();
						if (e.isNull()) e.init(element.getBounds());
						else e.expandToInclude(element.getBounds());
					}
				}
				bounds = new JTS.ReferencedEnvelope(e,getCRS());
					
			} catch (Exception e) {
				org.eclipse.core.runtime.Status status = new org.eclipse.core.runtime.Status(
					IStatus.WARNING,"net.refractions.udig.catalog", //$NON-NLS-1$
					0,Policy.bind("catalog.memory.bounds.error"), e //$NON-NLS-1$
				);
					
				CatalogPlugin.getDefault().getLog().log(status); //$NON-NLS-1$ //$NON-NLS-2$
				bounds = new JTS.ReferencedEnvelope(new Envelope(), getCRS());
			}

			keywords = new String[] { type, ft.getNamespace().toString() };
		}

		public CoordinateReferenceSystem getCRS() {
			return ft.getDefaultGeometry().getCoordinateSystem();
		}

		public String getName() {
			return ft.getTypeName();
		}

		public URI getSchema() {
			return ft.getNamespace();
		}

		public String getTitle() {
			return ft.getTypeName();
		}
	}
}