/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.catalog.ui;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IResolve;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.internal.ui.HeadlessWizardDialog;
import net.refractions.udig.catalog.internal.ui.Policy;
import net.refractions.udig.core.internal.ExtensionPointProcessor;
import net.refractions.udig.core.internal.ExtensionPointUtil;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.PlatformUI;

/**
 * A wizard page that opens a file dialog and closes the wizard when dialog is closed.
 * @author jeichar
 * @since 0.9.0
 */
public class OpenFilePage extends WizardPage implements UDIGImportPage {

    private boolean canFlipToNextPage=false;
    private ArrayList<URL> list;
    private Composite comp;
    private ArrayList<String> extensionList;

    public String getId() {
        return "net.refractions.udig.catalog.ui.openFilePage"; //$NON-NLS-1$
    }
    
    public boolean canProcess(Object object) {
        return (getFileURL(object) != null);
    }
    
    /** Check that any trailing #layer is removed from the url */
    static public URL checkedURL( URL url ){
        String check = url.toExternalForm();
        int hash = check.indexOf('#');
        if ( hash == -1 ){
            return url;            
        }
        try {
            return new URL( check.substring(0, hash ));
        } catch (MalformedURLException e) {
            return null;
        }
    }
    public URL getFileURL(Object thing) {
        URL url = CatalogPlugin.locateURL(thing);
        
//        if(thing instanceof String) {
//        	String string = (String)thing;
//        	int index = string.indexOf('\n');
//            if (index > -1) {
//            	string = string.substring(0, index);	
//            }
//            
//        	try {
//                blah = new URL(string);
//            } 
//        	catch(MalformedURLException ex) {
//                // try turning it into a file
//            	File file = new File(string);
//            	try {
//					blah = file.toURL();
//				} 
//            	catch (MalformedURLException e) {
//					//do nothing wierd string
//				}
//            	
//            }
//        } else if(thing instanceof URL) {
//            blah = (URL)thing;
//        } else if(thing instanceof File) {
//            try {
//            blah = ((File)thing).toURL();
//            } catch(MalformedURLException ex) {
//                // nothing needed
//            }
//        }
        if (url == null)
        	return url;
        
        url = checkedURL( url );
        String fileExt = url.getFile().substring(url.getFile().lastIndexOf('.') + 1);
        for (String goodExt : getExtensionList()) {
            if(fileExt.equals(goodExt.substring(goodExt.lastIndexOf('.') + 1))) {
                //actually do a test
            	File f = new File(url.getFile());
            	if (f.exists())
            		return url;
            }
        }
        return null;
    }
    /**
     * Construct <code>OpenFilePage</code>.
     *
     */
    public OpenFilePage() {
        super(Policy.bind("OpenFilePage.pageTitle")); //$NON-NLS-1$
    }
    /*
     * @see net.refractions.udig.catalog.ui.UDIGImportPage#getResources(org.eclipse.core.runtime.IProgressMonitor)
     */
    public List<IResolve> getResources( IProgressMonitor monitor ) throws Exception {
        if( list==null ){
            if( openFileDialog(comp) )
                finishWizard();
            else
                cancelWizard();
        }
        return process( list, monitor );
    }

    List<IResolve> process( List<URL> urls, IProgressMonitor monitor ){
        List<IResolve> resources=new ArrayList<IResolve>();
        monitor.beginTask( Policy.bind("OpenFilePage.1"), list.size() ); //$NON-NLS-1$
        int worked=0;
        for (URL url : urls ) {        
            if( monitor.isCanceled() ) return null;            
            try {
                monitor.subTask( url.toExternalForm() );                
                List<IService> aquire = CatalogPlugin.getDefault().getServiceFactory().aquire( url );
                resources.addAll( aquire );
            } catch (Exception e) {                
                // do nothing.
            }
            monitor.worked( worked++ );            
        }
        return resources;
    }
    private void cancelWizard() {
        try {
            getShell().getDisplay().asyncExec( new Runnable(){
                public void run() {
                    findButton(getShell().getChildren(), IDialogConstants.BACK_ID).notifyListeners(SWT.Selection, new Event());
                }
                
            });
        } catch (Exception e) {
            //do nothing
        }
    }

    Button findButton(Control[] children, int id) {
        if ( ((Integer)getShell().getDefaultButton().getData()).intValue()==id) //$NON-NLS-1$
            return getShell().getDefaultButton();
        
        for(Control child:children){
            if (child instanceof Button) {
                Button button = (Button) child;
                if( ((Integer)button.getData()).intValue()==id) 
                    return button;
            }                   
            if (child instanceof Composite) {
                Composite composite = (Composite) child;
                Button button=findButton(composite.getChildren(), id);
                if( button!=null )
                    return button;
            }
        }
        return null;
    }

    
    private void finishWizard() {
        try {
        	//one good hack deserves another
        	if (getContainer() instanceof HeadlessWizardDialog) {
        		//do nothing, the wizard will close the windows 
        	}
        	else {
        		//this is a hack to get the wizard to close
        		getShell().getDisplay().asyncExec( new Runnable(){

                    public void run() {
                    	IWizardPage page = getContainer().getCurrentPage();
                        IWizardPage next = getWizard().getNextPage(page);
                        
                        
                        if (next != null) {
                            getContainer().showPage(next);  
                        }
                        else {
                            //finish the wizard
                            findButton(getShell().getChildren(), IDialogConstants.FINISH_ID).notifyListeners(SWT.Selection, new Event());
                        }
                        
                      
                        
//                      IWizardPage[] pages = getWizard().getPages();
//                      getContainer().showPage(pages[pages.length-1]);
//                      getContainer().updateButtons();
//                      
                    }
                    
                });
        	}
        } catch (Exception e) {
            //do nothing
        }
    }

    /**
     * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
     */
    public boolean canFlipToNextPage() {
        return (list!=null && list.size()>1) || canFlipToNextPage;
    }
    
    /**
     * @see net.refractions.udig.ui.UDIGImportPage#setCanFlipToNextPage(boolean)
     */
    public void setCanFlipToNextPage(boolean value) {
        canFlipToNextPage=true;
    }

    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        comp=new Composite(parent, SWT.NULL);
//      comp.setLayout(new GridLayout(1,true));
//      viewer=new ListViewer(comp);
//      viewer.getControl().setLayoutData(new GridData(SWT.FILL,SWT.FILL,true, true));
//      viewer.setContentProvider(new ArrayContentProvider());
//      viewer.setLabelProvider(new LabelProvider());       
        setControl(comp);

        boolean open = openFileDialog(comp);
        if( open ) {
            finishWizard();
        }
        else {
            cancelWizard();
        }
    }
    
    private boolean checkDND(FileDialog fileDialog) {
    	try {
    		 IStructuredSelection selection = ((IDataWizard) getWizard()).getSelection();
    	        if( selection == null ) return false;
    	        ArrayList<URL> urlList = new ArrayList<URL>();
    	        for( Iterator itr = selection.iterator(); itr.hasNext(); ) {
    	            URL url = getFileURL(itr.next());
    	            if (url != null) {
    	                urlList.add(url);
    	            }
    	        }
    	        if(urlList.size() != 0) {
    	            list = urlList;
    	            String file = urlList.get(0).getFile();
    	            String ext = file.substring(file.lastIndexOf('.'));
    	            String dir = new File(file).getParent();
    	            
    	            file = file.substring(file.lastIndexOf(File.separator)+1);
    	            fileDialog.setFilterPath(dir);
    	            fileDialog.setFileName(file);
    	            
    	            String[] filters = fileDialog.getFilterExtensions();
    	            if (filters == null || filters.length == 0) {
    	            	//no filters set, set em up
    	            	fileDialog.setFilterExtensions(new String[]{"*"+ext,"*.*"}); //$NON-NLS-1$ //$NON-NLS-2$	
    	            }
    	            else {
    	            	//we have some filters, look for the one in question 
    	            	// in the list
    	            	int i = 0;
    	            	for (; i < filters.length; i++) {
    	            		if (("*"+ext).equals(filters[i])) break; //$NON-NLS-1$
    	            	}
    	            	
    	            	if (i < filters.length) {
    	            		//we found it, reorganize the array so that 
    	            		// it is first
    	            		String[] nfilters = new String[filters.length];
    	            		nfilters[0] = filters[i];
    	            		System.arraycopy(filters,0,nfilters,1,i);
    	            		System.arraycopy(filters,i+1,nfilters,i+1,filters.length-i-1);
    	            		fileDialog.setFilterExtensions(nfilters);
    	            	}
    	            	else {
    	            		// no dice, add the filter
    	            		String[] nfilters = new String[filters.length+1];
    	            		nfilters[0] = "*"+ext; //$NON-NLS-1$
    	            		System.arraycopy(filters,0,nfilters,1,filters.length);
    	            		fileDialog.setFilterExtensions(nfilters);
    	            	}
    	            }
    	            
    	            return true;
    	        }
    	}
    	catch(Exception e) {
    		CatalogUIPlugin.log(e.getLocalizedMessage(), e);
    	}
    	
    	return false;
       
    }
    
    private List<String> getExtensionList() {
        if(extensionList == null) {
            extensionList = new ArrayList<String>();
            ExtensionPointUtil.process( CatalogUIPlugin.getDefault(), "net.refractions.udig.catalog.ui.fileFormat",new ExtensionPointProcessor(){ //$NON-NLS-1$
                public void process( IExtension extension, IConfigurationElement element ) throws Exception {
                    String ext = element.getAttribute("fileExtension"); //$NON-NLS-1$
                    extensionList.add(ext);
                }
            });
        }
        return (List<String>)extensionList.clone();
    }
    
    private boolean openFileDialog(Composite parent) {
        String lastOpenedDirectory = PlatformUI.getPreferenceStore().getString(CatalogUIPlugin.PREF_OPEN_DIALOG_DIRECTORY);
        FileDialog fileDialog = new FileDialog(parent.getShell(), SWT.MULTI);
        
        //load maps

        List<String> fileTypes = getExtensionList();
        StringBuffer all = new StringBuffer();
        for( Iterator<String> i = fileTypes.iterator(); i.hasNext(); ) {
            all.append( i.next() );
            if( i.hasNext()) all.append( File.pathSeparator );
        }
        fileTypes.add(0,all.toString() );
        fileTypes.add("*.*"); //$NON-NLS-1$
        
        fileDialog.setFilterExtensions(fileTypes.toArray(new String[fileTypes.size()])); //$NON-NLS-1$ //$NON-NLS-2$

        if (lastOpenedDirectory != null) {
            fileDialog.setFilterPath(lastOpenedDirectory);
        }
        
        boolean dnd = checkDND(fileDialog);
        
        //this is a HACK to check for headless execution
        if (getContainer() instanceof HeadlessWizardDialog) {
        	if (dnd)
        		return true; //there was a workbench selection that allowed us select a file
        }
        
        String result = fileDialog.open();
        if (result == null) {
            return false;
        }
        String path = fileDialog.getFilterPath();
        PlatformUI.getPreferenceStore().setValue(CatalogUIPlugin.PREF_OPEN_DIALOG_DIRECTORY, path);
        String[] filenames = fileDialog.getFileNames();
        list = new ArrayList<URL>();
        for (int i = 0; i < filenames.length; i++) {
            try {
                URL url = new File(path+System.getProperty("file.separator")+filenames[i]).toURL(); //$NON-NLS-1$
                list.add( url );                
            } catch (Throwable e) {
                e.printStackTrace();
            }
        }
        return true;
//      viewer.setInput(urls);
    }

}
