package net.refractions.udig.core.internal;

import org.geotools.data.DataUtilities;
import org.geotools.feature.AttributeType;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureType;
import org.geotools.referencing.CRS;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;
import org.opengis.util.CodeList;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
/**
 * A utility class for playing with features.
 * 
 * @author jeichar
 */
public class FeatureUtils {
	/**
	 * @see net.refractions.udig.project.command.MapCommand#run()
	 */
	public static Feature createFeature(CoordinateReferenceSystem coordCRS, CoordinateReferenceSystem destinationCRS,FeatureType type, Coordinate[] coordinates, Class geomType) throws Exception {

		transform(coordCRS, destinationCRS,coordinates);
		Object[] attrs = new Object[type.getAttributeCount()];
		for (int i = 0; i < attrs.length; i++) {
			attrs[i] = setDefaultValue(type.getAttributeType(i));
		}
		final Feature newFeature = type.create(attrs);
//		Class geomType = type.getDefaultGeometry().getType();

		Geometry geom = GeometryBuilder.create().safeCreateGeometry(geomType,
				coordinates);
		newFeature.setDefaultGeometry(geom);

		return newFeature;
	}

	/**
	 * @param object
	 * @param object2
	 */
	private static Object setDefaultValue(AttributeType type) {
		if (type.createDefaultValue() != null)
			return type.createDefaultValue();
		if (Boolean.class.isAssignableFrom(type.getType())
				|| boolean.class.isAssignableFrom(type.getType()))
			return new Boolean(false);
		if (String.class.isAssignableFrom(type.getType()))
			return ""; //$NON-NLS-1$
		if (Integer.class.isAssignableFrom(type.getType()))
			return new Integer(0);
		if (Double.class.isAssignableFrom(type.getType()))
			return new Double(0);
		if (Float.class.isAssignableFrom(type.getType()))
			return new Float(0);
		if (CodeList.class.isAssignableFrom(type.getType())) {
			return type.createDefaultValue();
		}
		return null;
	}

	/**
	 * Transforms coordinates into the layer CRS if nessecary
	 * 
	 * @throws Exception
	 */
	private static void transform(CoordinateReferenceSystem coordCRS, CoordinateReferenceSystem destinationCRS, Coordinate[] coordinates) throws Exception {
		MathTransform mt = CRS.transform(coordCRS,
				destinationCRS, true);
		if (mt == null || mt.isIdentity())
			return;
		double[] coords = new double[coordinates.length * 2];
		for (int i = 0; i < coordinates.length; i++) {
			coords[i * 2] = coordinates[i].x;
			coords[i * 2 + 1] = coordinates[i].y;
		}
		mt.transform(coords, 0, coords, 0, coordinates.length);
		for (int i = 0; i < coordinates.length; i++) {
			coordinates[i].x = coords[i * 2];
			coordinates[i].y = coords[i * 2 + 1];
		}
	}

    /**
     * Returns 0 if the features are the same.  1 if the attributes are the same but have different featureIDs and -1 if attributes are
     * different or are of different featureTypes.
     *
     * @param feature1
     * @param feature2
     * @return
     */
    public static int same( Feature feature1, Feature feature2 ) {
        if( DataUtilities.compare(feature1.getFeatureType(), feature2.getFeatureType())!=0 ){
            return -1;
        }
        for( int i=0; i<feature1.getNumberOfAttributes(); i++){
            if( feature1.getAttribute(i)==null ){
                if (feature2.getAttribute(i)!=null )
                    return -1;
                else
                    continue;
            }
            if (feature1.getAttribute(i) instanceof Geometry) {
                Geometry geom1 = (Geometry) feature1.getAttribute(i);
                if (feature2.getAttribute(i) instanceof Geometry) {
                    Geometry geom2 = (Geometry) feature2.getAttribute(i);
                    if( geom1.equalsExact(geom2) )
                        continue;
                    else
                        return -1;
                }else
                    return -1;
            }
            if( !feature1.getAttribute(i).equals(feature2.getAttribute(i)) )
                return -1;
        }
        
        return feature1.getID().equals(feature2.getID())?0:1;
    }

}
