package net.refractions.udig.internal.ui;

import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.core.internal.ExtensionPointProcessor;
import net.refractions.udig.core.internal.ExtensionPointUtil;
import net.refractions.udig.ui.IDropAction;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

public class UDIGDropProcessor {
	
	public static List<IDropAction> process(Object data, UDIGDropHandler handler) {
		if (data == null || handler == null || handler.getDestination() == null)
			return new ArrayList<IDropAction>();
		
		//process to see if anyone cares
		Processor d = new Processor(data, handler);
		ExtensionPointUtil.process(UiPlugin.getDefault(), IDropAction.XPID, d);
		
		return d.actions;
	}
	
	static class Processor implements ExtensionPointProcessor {

		Object data;
		UDIGDropHandler handler;
		List<IDropAction> actions;
		
		Processor(Object data, UDIGDropHandler handler) {
			this.data = data;
			this.handler = handler;
			actions = new ArrayList<IDropAction>();
		}
		
		public void process(IExtension extension, IConfigurationElement element) 
			throws Exception {
			
			ClassLoader sloader = data.getClass().getClassLoader();
			if (sloader == null) {
				//probably boot class loader
				sloader = ClassLoader.getSystemClassLoader();
			}
			ClassLoader dloader = handler.getDestination().getClass()
				.getClassLoader();
			
			//first find a matching destination
			IConfigurationElement[] targets = element.getChildren("destination"); //$NON-NLS-1$
			Class c = null;
			for (int i = 0; i < targets.length && c == null; i++) {
				IConfigurationElement target = targets[i];
				
				try {
					String clazz = target.getAttribute("class");
					c = dloader.loadClass(clazz);
					
					if (c == null)
						continue;
					if (!c.isAssignableFrom(handler.getDestination().getClass()))
						c = null;
				}
				catch(ClassNotFoundException e) {
					//expected, do nothing
					continue;
				}
			}
	
			if (c == null) 
				return;
			
            //next find a matching type
			IConfigurationElement[] acceptedTypes = element.getChildren("acceptedType");
			c = null;

			for (int i = 0; i < acceptedTypes.length && c == null; i++) {
				IConfigurationElement acceptedType = acceptedTypes[i];

				
				try {

					String clazz = acceptedType.getAttribute("class");
					c = sloader.loadClass(clazz);
					
					if (c == null)
						continue;
					if (!c.isAssignableFrom(data.getClass()))
						c = null;
				}
				catch(ClassNotFoundException e) {
					//expected, do nothing
					continue;
				}
			}
			
			if (c != null) {
				//do the accept check
				try {
					IDropAction action = 
						(IDropAction)element.createExecutableExtension("class");
					if (action.accept(data, handler.getDestination())) {
						action.setElement(element);
						action.setDropHandler(handler);
						actions.add(action);
					}
				}
				catch(Throwable t) {
					String msg = "Error validating drop action";
					String ns = element.getNamespace();
					
					Status s = new Status(IStatus.WARNING,ns,0,msg,t);
					UiPlugin.getDefault().getLog().log(s);
				}
			}
		}
	}
	
}