package net.refractions.udig.internal.ui;

import java.net.Authenticator;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import net.refractions.udig.internal.ui.operations.OperationMenuFactory;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class UiPlugin extends AbstractUIPlugin {
    //The shared instance.
    private static UiPlugin plugin;

    //Resource bundle.
    private ResourceBundle resourceBundle;

    /** Icons path (value "icons/") */
    public final static String ICONS_PATH = "icons/";//$NON-NLS-1$
    
    public final static String ID = "net.refractions.udig.ui"; //$NON-NLS-1$
    
    public static final String DROP_ACTIONS_ID = ID+".dropActions"; //$NON-NLS-1$
    
    public static final String DROP_TRANSFERS_ID = ID+".dropTransfers"; //$NON-NLS-1$

    /** Managed Images instance */
    private Images images = new Images();

    private URL iconsUrl;

	private OperationMenuFactory operationMenuFactory;

    /**
     * The constructor.
     */
    public UiPlugin() {
        super();
        plugin = this;
    }

    /**
     * This method is called upon plug-in activation
     * @param context 
     * @throws Exception 
     */
    public void start( BundleContext context ) throws Exception {
        super.start(context);
        Policy.localize( "net.refractions.udig.ui.messages" ); //$NON-NLS-1$
        iconsUrl = context.getBundle().getEntry(ICONS_PATH);        
        
        Authenticator.setDefault(new UDIGAuthenticator());
        /*
         * TODO Further code can nuke the previously set authenticator. Proper
         * security access should be configured to prevent this.
         */
    }

    /**
     * Creates an image descriptor for later use.
     */
    synchronized ImageDescriptor create(String id) {
        URL url = null;
        try {
            url = new URL(iconsUrl, id);
        } catch (MalformedURLException e) {
//            System.out.println( "Could not locate "+id );
            return null;
        }
        ImageDescriptor image = ImageDescriptor.createFromURL(url);
        getImageRegistry().put(id, image );
        return image;
    }
    
    /**
     * Returns the shared instance.
     */
    public static UiPlugin getDefault() {
        return plugin;
    }

    /**
     * Returns the string from the plugin's resource bundle, or 'key' if not found.
     */
    public static String getResourceString( String key ) {
        ResourceBundle bundle = UiPlugin.getDefault().getResourceBundle();
        try {
            return (bundle != null) ? bundle.getString(key) : key;
        } catch (MissingResourceException e) {
            return key;
        }
    }

    /**
     * Returns the plugin's resource bundle,
     */
    public ResourceBundle getResourceBundle() {
        return resourceBundle;
    }
    /**
     * Images instance for use with ImageConstants.
     * 
     * @return Images for use with ImageConstants.
     */
    public Images getImages() {
        return images;
    }
    
    public OperationMenuFactory getOperationMenuFactory(){
    	if (operationMenuFactory == null) {
			operationMenuFactory = new OperationMenuFactory();
		}

		return operationMenuFactory;
    }
   

	public void stop(BundleContext context) throws Exception {
		Policy.cleanup();
		super.stop(context);
	}
    /**
     * Writes an info log in the plugin's log.
     * <p>
     * This should be used for user level messages.
     * </p>
     */
    public static void log( String message, Throwable e) {
    	if( message==null )
    		message="";
        getDefault().getLog().log(new Status(IStatus.INFO, ID, 0, message, e));
    }
    /**
     * Messages that only engage if getDefault().isDebugging()
     * <p>
     * It is much prefered to do this:<pre><code>
     * private static final String RENDERING = "net.refractions.udig.project/render/trace";
     * if( ProjectUIPlugin.getDefault().isDebugging() && "true".equalsIgnoreCase( RENDERING ) ){
     *      System.out.println( "your message here" );
     * }
     */
    public static void trace( String message, Throwable e) {
        if( getDefault().isDebugging() ) {
            if( message != null ) System.out.println( message );
            if( e != null ) e.printStackTrace();
        }
    }
    /**
     * Performs the Platform.getDebugOption true check on the provided trace
     * <p>
     * Note: ProjectUIPlugin.getDefault().isDebugging() must also be on.
     * <ul>
     * <li>Trace.RENDER - trace rendering progress
     * </ul>
     * </p> 
     * @param trace currently only RENDER is defined
     */
    public static boolean isDebugging( final String trace ){
        return getDefault().isDebugging() &&
            "true".equalsIgnoreCase(Platform.getDebugOption(trace)); //$NON-NLS-1$    
    }
}
