package net.refractions.udig.mapgraphic.internal;


import java.awt.Graphics2D;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.mapgraphic.MapGraphic;
import net.refractions.udig.mapgraphic.MapGraphicContext;
import net.refractions.udig.mapgraphic.MapGraphicPlugin;
import net.refractions.udig.project.internal.render.impl.RendererImpl;
import net.refractions.udig.project.render.ICompositeRenderContext;
import net.refractions.udig.project.render.IMultiLayerRenderer;
import net.refractions.udig.project.render.IRenderContext;
import net.refractions.udig.project.render.RenderException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import com.vividsolutions.jts.geom.Envelope;

public class MapGraphicRenderer extends RendererImpl implements IMultiLayerRenderer {


    /**
     * 
     * @see net.refractions.udig.project.render.impl.RendererImpl#render(java.awt.Graphics2D)
     * @param destination
     */
    public void render( Graphics2D destination, IProgressMonitor monitor ) {
        List<IOException> exceptions=new ArrayList<IOException>(); 
        for (IRenderContext l : getContext().getContexts()) {
            try{
                        MapGraphic mg = l.getGeoResource().resolve(MapGraphic.class, null);
                        MapGraphicContext mgContext = new MapGraphicContextImpl(
                            l
                        );
                        mg.draw(mgContext);
            }catch(IOException e){
                exceptions.add(e);
            }
        }
        if( !exceptions.isEmpty() ){
            RenderException exception=new RenderException(exceptions.size()+" exceptions we raised while drawing map graphics", exceptions.get(0));
            exception.fillInStackTrace();
        }
    }
    
    /**
     * @see net.refractions.udig.project.internal.render.impl.RendererImpl#getContext()
     */
    public ICompositeRenderContext getContext() {
        return (ICompositeRenderContext) super.getContext();
    }
    
 
    /**
     * TODO summary sentence for getInfo ...
     * 
     * @see net.refractions.udig.project.render.impl.RendererImpl#getInfo(java.awt.Point)
     * @param screenLocation
     * @return
     * @throws IOException
     *
    public InfoList getInfo( Point screenLocation ) throws IOException {
        return null;
    }*/

    /**
     * TODO summary sentence for stopRendering ...
     * 
     * @see net.refractions.udig.project.render.impl.RendererImpl#stopRendering()
     * 
     */
    public void stopRendering() {
        //do nothing
    }

    /**
     * TODO summary sentence for dispose ...
     * 
     * @see net.refractions.udig.project.render.impl.RendererImpl#dispose()
     * 
     */
    public void dispose() {
        //do nothing
    }

    /* (non-Javadoc)
     * @see net.refractions.udig.project.render.Renderer#render(com.vividsolutions.jts.geom.Envelope)
     */
    public void render( Envelope bounds, IProgressMonitor monitor ) {
        setState(RENDERING);
        render(getContext().getImage().createGraphics(), monitor);
        setState(DONE);
    }
    
    
    Job refreshJob=new Job("RefreshJob"){
        protected IStatus run( IProgressMonitor monitor ) {
            getContext().clearImage();            
            try {
                render((Envelope)null, monitor);
            } catch (Throwable e) {
                MapGraphicPlugin.log( null, e);
            }
            return Status.OK_STATUS;
        }
        
    };
    
    public void refreshImage() throws RenderException {
            refreshJob.schedule();
    }
        
    
}