package net.refractions.udig.mapgraphic.internal;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IGeoResourceInfo;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.mapgraphic.MapGraphic;
import net.refractions.udig.mapgraphic.MapGraphicFactory;
import net.refractions.udig.mapgraphic.Policy;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.geometry.JTS;
import org.geotools.geometry.JTS.ReferencedEnvelope;

import com.vividsolutions.jts.geom.Envelope;


public class MapGraphicResource extends IGeoResource {

    /** the service which contains all decorator resources **/
    MapGraphicService parent;
    
    /** the info for the map graphic resource **/ 
    MapGraphicResourceInfo info;
    
    /** the map graphic itself **/
    MapGraphic mapGraphic;
    
    /** the map graphic name **/
    String name;

    /** the map graphic id **/
    String id;
    
    MapGraphicResource(MapGraphicService service, IConfigurationElement element) {
        parent = service;
        this.name = element.getAttribute("name"); //$NON-NLS-1$
        this.id = element.getAttribute("id"); //$NON-NLS-1$
    }
    
    /*
     * @see net.refractions.udig.catalog.IGeoResource#resolve(java.lang.Class, org.eclipse.core.runtime.IProgressMonitor)
     */
    public <T> T resolve( Class<T> adaptee, IProgressMonitor monitor ) throws IOException {
        if (adaptee == null) return null;
        
        if (adaptee.isAssignableFrom(IService.class)) {
            return adaptee.cast(parent);
        }
        if (adaptee.isAssignableFrom(IGeoResourceInfo.class)) {
            if (info == null) {
                info = new MapGraphicResourceInfo();
            }
            return adaptee.cast(info);
        }
        //TODO: this is not safe, the caller of this method might not have
    	// the MapGraphicClasses on their classpath
        if (adaptee.isAssignableFrom(MapGraphic.class)) {
            if (mapGraphic == null) {
            	synchronized (MapGraphicFactory.getInstance()) {
            		if (mapGraphic == null)
            			mapGraphic = MapGraphicFactory.getInstance()
            				.createMapGraphic(id);
				}
            }
            return adaptee.cast(mapGraphic);
        }
        if (adaptee.isAssignableFrom(MapGraphicFactory.class)) {
            return adaptee.cast(MapGraphicFactory.getInstance());
        }
        
        return null;
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#canResolve(java.lang.Class)
     */
    public <T> boolean canResolve( Class<T> adaptee ) {
    	
        return adaptee != null && (
            adaptee.isAssignableFrom(IService.class) || 
            adaptee.isAssignableFrom(IGeoResource.class) ||
            //TODO: this is not safe, the caller of this method might not have
        	// the MapGraphicClasses on their classpath
            adaptee.isAssignableFrom(MapGraphic.class) ||
            adaptee.isAssignableFrom(MapGraphicFactory.class) ||
            (mapGraphic != null && adaptee.isAssignableFrom(mapGraphic.getClass()))
        ); 
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getStatus()
     */
    public Status getStatus() {
        return parent.getStatus();
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getMessage()
     */
    public Throwable getMessage() {
        return parent.getMessage();
    }

    /*
     * @see net.refractions.udig.catalog.IResolve#getIdentifier()
     */
    public URL getIdentifier() {
        try {
            return new URL(parent.getIdentifier().toString()+"#"+id); //$NON-NLS-1$
        } 
        catch (MalformedURLException e) {
            e.printStackTrace();
        } 
        
        return null;
    }
    
    class MapGraphicResourceInfo extends IGeoResourceInfo {
        
        /*
         * @see net.refractions.udig.catalog.IGeoResourceInfo#getName()
         */
        public String getName() {
            return name;
        }
        
        @Override
        public String getDescription() {
        	MessageFormat formatter = new MessageFormat(Policy.bind("MapGraphicResource.description")); //$NON-NLS-1$
            return formatter.format(name);
        }
        
        @Override
        public ReferencedEnvelope getBounds() {
            if (bounds == null) {
                Envelope e = new Envelope();
                e.setToNull();
                
                bounds = new JTS.ReferencedEnvelope(e,null);
            }
            return bounds;
        }
    }
}

