package net.refractions.udig.mapgraphic.internal.ui;

import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IResolve;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.IServiceFactory;
import net.refractions.udig.catalog.ui.UDIGImportPage;
import net.refractions.udig.mapgraphic.internal.MapGraphicService;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;

public class MapGraphicWizardPage extends WizardPage
	implements UDIGImportPage {

	public static final String ID = 
		"net.refractions.udig.mapgraphic.wizardPageId";
	
	public MapGraphicWizardPage() {
		super("Map Graphic");
	}

	public List<IResolve> getResources(IProgressMonitor monitor) throws Exception {
		IServiceFactory factory = CatalogPlugin.getDefault().getServiceFactory();
		List<IService> services = factory.aquire(MapGraphicService.SERVICE_URL);
		List<IResolve> resolves = new ArrayList<IResolve>();
		
		for (IService service : services) {
			resolves.add(service);
		}
		return resolves;
	}

	public boolean canProcess(Object object) {
		return false;
	}

	public String getId() {
		return ID;
	}

	public void createControl(Composite parent) {
		setControl(new Composite(parent, SWT.NONE));
		
		Display.getDefault().asyncExec(
				new Runnable() {
					public void run() {
						IWizardPage page = getContainer().getCurrentPage();
	                    IWizardPage next = getWizard().getNextPage(page);
	                    
	                    if (next != null) {
	                        getContainer().showPage(next);  
	                    }
	                    else {
	                        //finish the wizard
	                        findButton(getShell().getChildren(), IDialogConstants.FINISH_ID).notifyListeners(SWT.Selection, new Event());
	                    }
						
					}
				}
			);
			
		
	}
	
	Button findButton(Control[] children, int id) {
        if ( ((Integer)getShell().getDefaultButton().getData()).intValue()==id) //$NON-NLS-1$
            return getShell().getDefaultButton();
        
        for(Control child:children){
            if (child instanceof Button) {
                Button button = (Button) child;
                if( ((Integer)button.getData()).intValue()==id) 
                    return button;
            }                   
            if (child instanceof Composite) {
                Composite composite = (Composite) child;
                Button button=findButton(composite.getChildren(), id);
                if( button!=null )
                    return button;
            }
        }
        return null;
    }
	
}