/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.printing.model;

import java.awt.Graphics2D;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * A Box is a conceptual object that can appear on a page. It is an extension
 * of an <code>Element</code> and provides methods that allow a Box to draw
 * itself and support connections to other boxes. 
 * 
 * Implementors should directly implement this interface or one of its
 * implementations. DecoratorBox in particular.
 * 
 * @author Richard Gould
 * @since 0.3
 * @see Element
 * @model abstract="true"
 */
public interface Box extends Element{
    /**
     * This method will be called by Page when it is time to display itself on
     * a screen or when the actual printing is being performed.
     * 
     * The graphics object passed in will have its appropriate clipping set,
     * so the drawing code can only draw inside this clip.
     * 
     * @see Page
     * @param graphics A <code>Graphics2D</code> object to perform the drawing on
     * @param monitor 
     * @model
     */
    public void draw(Graphics2D graphics, IProgressMonitor monitor);
    
    /**
     * This is used by the GEF system to access the source connections on this Box
     * 
     * @return a List of Connections that use this Box as a source
     * @model resolveProxy="false" type="net.refractions.udig.printing.model.Connection"
     */
    public List getSourceConnections();
    
    /**
     * This is used by the GEF system to access the target connections on this Box
     * 
     * @return a List of Connectoins that use this Box as a target
     * @model resolveProxy="false" type="net.refractions.udig.printing.model.Connection"
     */
    public List getTargetConnections();
    
    /**
     * 
     * Adds a Connection to this Box
     * @see Connection
     * @param connection the connection to add to this Box
     */
    void add(Connection connection);
    
    /**
     * 
     * Removes a Connection to this Box 
     * @see Connection
     * @param connection the connection to be removed
     */
    void remove(Connection connection);
}
