/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.printing.ui.internal;

import net.refractions.udig.printing.model.LabelBox;
import net.refractions.udig.printing.model.MapBox;
import net.refractions.udig.printing.model.ModelFactory;
import net.refractions.udig.printing.model.Page;
import net.refractions.udig.printing.ui.Policy;
import net.refractions.udig.project.internal.Map;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;

/**
 * Implementation of a Template at its most basic. Contains a title bar
 * and a map.
 * 
 * @author Richard Gould
 */
public class BasicTemplate extends AbstractTemplate {

	/**
	 * Constructs the BasicTemplate and populates its two boxes with a title
	 * and a map.
	 */
	public BasicTemplate () {
		super();		
	}
	
	/**
	 * Populates the templates two boxes with a title and map
	 * @param page  * @param decorators a List of the Layers that are Decorators to be handled separate from the Map
	 * 
	 * @param map the Map to be drawn
	 * @param width
	 * @param height
	 */
	public void init(Page page, Map map, int width, int height) {
	    
	    LabelBox box = ModelFactory.eINSTANCE.createLabelBox();
	    box.setText(Policy.bind("BasicTemplate.label.defaultTitle")); //$NON-NLS-1$
	    box.setSize(new Dimension(150, 30));
	    box.setLocation(new Point(10, 10));
	    boxes.add(box);
	    
	    MapBox mapBox = ModelFactory.eINSTANCE.createMapBox();
	    mapBox.setMap(map);
	    mapBox.setSize(new Dimension(400, 400));
	    mapBox.setLocation(new Point(50,50));
	    boxes.add(mapBox);
        
        //This code should be added back in at some point
/*        for (int i = 0; i < decorators.size(); i++) {
            DecoratorBox decoratorBox = ModelFactory.eINSTANCE.createDecoratorBox();
            decoratorBox.setMap(map);
            decoratorBox.setLayer(decorators.get(i));
            decoratorBox.setSize(new Dimension(130, 75));
            decoratorBox.setLocation(new Point(100, 10));
    	    
    	    boxes.add(decoratorBox);
            
    	    Connection connection = ModelFactory.eINSTANCE.createConnection();
    	    connection.setSource(mapBox);
    	    connection.setTarget(decoratorBox);
    	    connection.reconnect();
        }
        */
	    
        page.setName(map.getName());
	}

    /**
     * @see net.refractions.udig.printing.ui.Template#getName()
     */
    public String getName() {
        return Policy.bind("BasicTemplate.name"); //$NON-NLS-1$
    }
}
