/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.printing.ui.internal.editor;

import java.util.EventObject;

import net.refractions.udig.printing.model.Page;
import net.refractions.udig.printing.ui.Policy;
import net.refractions.udig.printing.ui.actions.EditMapAction;
import net.refractions.udig.printing.ui.internal.PrintAction;
import net.refractions.udig.printing.ui.internal.editor.parts.PartFactory;
import net.refractions.udig.printing.ui.internal.editor.parts.TreePartFactory;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.dnd.TemplateTransferDragSourceListener;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.DirectEditAction;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.gef.ui.palette.PaletteViewerProvider;
import org.eclipse.gef.ui.palette.FlyoutPaletteComposite.FlyoutPreferences;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;


/**
 * Provides ...TODO summary sentence
 * <p>
 * TODO Description
 * </p>
 * <p>
 * Responsibilities:
 * <ul>
 * <li>
 * <li>
 * </ul>
 * </p>
 * <p>
 * Example Use:
 * 
 * <pre><code>
 * 
 *  
 *   
 *    
 *     
 *      PageEditor x = new PageEditor( ... );
 *      TODO code example
 *      
 *     
 *    
 *   
 *  
 * </code></pre>
 * 
 * </p>
 * 
 * @author Richard Gould
 * @since 0.3
 */
public class PageEditor extends GraphicalEditorWithFlyoutPalette {
    
    public final static String EDIT_MAP ="edit map"; //$NON-NLS-1$
    public final static String ID = "net.refractions.udig.printing.ui.internal.editor.pageEditor"; //$NON-NLS-1$
    
    private boolean savePreviouslyNeeded;
    private Page page;
    private PaletteRoot paletteRoot;
    private PageEditorOutlinePage outlinePage;

    public PageEditor() {
        setEditDomain(new DefaultEditDomain(this));
    }

    protected void configureGraphicalViewer() {
        super.configureGraphicalViewer();
               
        getGraphicalViewer().setRootEditPart(new ScalableFreeformRootEditPart());
        
        getGraphicalViewer().setEditPartFactory(new PartFactory());
        
        ContextMenuProvider provider = new PageContextMenuProvider(getGraphicalViewer(), getActionRegistry());
        getGraphicalViewer().setContextMenu(provider);
        getSite().registerContextMenu("net.refractions.udig.printing.editor.contextmenu", //$NON-NLS-1$
                provider, getGraphicalViewer());
    }

    public void commandStackChanged( EventObject event ) {
        if (isDirty()) {
            if (!this.savePreviouslyNeeded) {
                this.savePreviouslyNeeded = true;
                firePropertyChange(IEditorPart.PROP_DIRTY);
            }
        } else {
            savePreviouslyNeeded = false;
            firePropertyChange(IEditorPart.PROP_DIRTY);
        }
        super.commandStackChanged(event);
    }
    
    protected void setInput( IEditorInput input ) {
        super.setInput(input);
        
        page = (Page) ((PageEditorInput) input).getProjectElement();
        
        setPartName(page.getName());
    }
    
    protected void initializeGraphicalViewer() {
        getGraphicalViewer().setContents(this.page);
    }

    public void doSave( IProgressMonitor monitor ) {
        //TODO fix this at some point, currently doesn't work
//        Platform.run(new SafeRunnable(){
//
//            public void run() throws Exception {
//                page.getProject().eResource().save(null);
//            }
//            
//        });
//        setDirty(false);
    }

    public void doSaveAs() {
    }
    
    /* TODO Permit saving at some point */
    public boolean isDirty() {
        return false;
    }
    
    /* TODO Permit saving at some point */
    public boolean isSaveAsAllowed() {
        return true;
    }

    protected void createActions() {
        super.createActions();
    	ActionRegistry registry = getActionRegistry();
    	IAction action;
    	
    	action = new DirectEditAction((IWorkbenchPart) this);
    	registry.registerAction(action);
        getSelectionActions().add(action.getId());
        
        action = new EditMapAction(this);
        registry.registerAction(action);
        getSelectionActions().add(action.getId());
    }

    protected PaletteRoot getPaletteRoot() {
        
        if (paletteRoot == null) {
            paletteRoot = PageEditorPaletteFactory.createPalette();
        }
        
        return paletteRoot;
        
//        PaletteRoot root = new PaletteRoot();
//        
//        List catagories = new ArrayList();
//        
//        PaletteGroup controls = new PaletteGroup("Controls");
//        
//        ToolEntry tool = new SelectionToolEntry();
//        controls.add(tool);
//        root.setDefaultEntry(tool);
//        
//        controls.add(new MarqueeToolEntry());
//        
//        PaletteSeparator separator = new PaletteSeparator(
//                PrintingPlugin.PLUGIN_ID + ".palette.separator");
//        
//        separator.setUserModificationPermission(
//                PaletteEntry.PERMISSION_NO_MODIFICATION);
//        controls.add(separator);
//
//        root.addAll(catagories);
//        
//        return root;
    }
    
    protected FlyoutPreferences getPalettePreferences() {
        return PageEditorPaletteFactory.createPalettePreferences();
    }
    
    protected PaletteViewerProvider createPaletteViewerProvider() {
    	return new PaletteViewerProvider(getEditDomain()) {
    		protected void configurePaletteViewer(PaletteViewer viewer) {
    			super.configurePaletteViewer(viewer);
    			// create a drag source listener for this palette viewer
    			// together with an appropriate transfer drop target listener, this will enable
    			// model element creation by dragging a CombinatedTemplateCreationEntries 
    			// from the palette into the editor
    			// @see ShapesEditor#createTransferDropTargetListener()
    			viewer.addDragSourceListener(new TemplateTransferDragSourceListener(viewer));
    		}
    	};
    }
    
    public void createPartControl( Composite parent ) {
        super.createPartControl(parent);
        
//      enable printing
        IActionBars actionBars = getEditorSite().getActionBars();
        actionBars.setGlobalActionHandler(ActionFactory.PRINT.getId(), new PrintAction());
    }

    public Object getAdapter( Class type ) {
        if (type == IContentOutlinePage.class) {
            if (outlinePage == null) {
                outlinePage = new PageEditorOutlinePage(this, new TreeViewer());
            }
            return outlinePage;
        }
        return super.getAdapter(type);
    }
    
    private void disposeOutlinePage() {
        this.outlinePage = null;
    }
    
    private Page getModel() {
        return page;
    }
    
    public class PageEditorOutlinePage extends ContentOutlinePage {

        private PageEditor editor;
        private PageBook pagebook;

        /**
         * Construct <code>PageEditorOutlinePage</code>.
         *
         * @param viewer
         */
        public PageEditorOutlinePage( PageEditor editor, EditPartViewer viewer ) {
            super(viewer);
            
            if (editor == null) {
                throw new IllegalArgumentException(Policy.bind("PageEditor.error.nullEditor")); //$NON-NLS-1$
            }
            this.editor = editor;
        }
        
        /**
         * TODO summary sentence for createControl ...
         * 
         * @see org.eclipse.ui.part.IPage#createControl(org.eclipse.swt.widgets.Composite)
         * @param parent
         */
        public void createControl( Composite parent ) {
            pagebook = new PageBook(parent, SWT.NONE);
            Control outline = getViewer().createControl(pagebook);
            getViewer().setEditDomain(editor.getEditDomain());
            getViewer().setEditPartFactory(new TreePartFactory());
//            getViewer().setKeyHandler(editor.getCommonKeyHandler());
            
            ContextMenuProvider cmProvider = new PageContextMenuProvider(getViewer(),
                    editor.getActionRegistry());
            getViewer().setContextMenu(cmProvider);
            getSite().registerContextMenu("net.refractions.udig.printing.outline.contextmenu", //$NON-NLS-1$
                    cmProvider, getSite().getSelectionProvider());
            editor.getSelectionSynchronizer().addViewer(getViewer());
            getViewer().setContents(editor.getModel());
            pagebook.showPage(outline);
            
            
        }
        /**
         * TODO summary sentence for getControl ...
         * 
         * @see org.eclipse.ui.part.IPage#getControl()
         * @return
         */
        public Control getControl() {
            return pagebook;
        }
        /**
         * TODO summary sentence for dispose ...
         * 
         * @see org.eclipse.ui.part.IPage#dispose()
         * 
         */
        public void dispose() {
            editor.getSelectionSynchronizer().removeViewer(getViewer());
            super.dispose();
            editor.disposeOutlinePage();
        }
    }
}