/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.printing.ui.internal.editor.parts;

import net.refractions.udig.printing.model.MapBox;
import net.refractions.udig.printing.model.PropertyListener;
import net.refractions.udig.printing.ui.Policy;
import net.refractions.udig.printing.ui.internal.editor.figures.MapFigure;
import net.refractions.udig.printing.ui.internal.editor.policies.MapEditPolicy;
import net.refractions.udig.project.internal.Map;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.tools.DirectEditManager;

/**
 * Provides ...TODO summary sentence
 * <p>
 * TODO Description
 * </p><p>
 * Responsibilities:
 * <ul>
 * <li>
 * <li>
 * </ul>
 * </p><p>
 * Example Use:<pre><code>
 * NodePart x = new NodePart( ... );
 * TODO code example
 * </code></pre>
 * </p>
 * @author Richard Gould
 * @since 0.3
 */
public class MapPart extends BoxPart{
    
    protected DirectEditManager manager;
    private InternalPropertyListener listener = new InternalPropertyListener();
    
    public void activate() {
        if (isActive()) {
            return;
        }
        
        super.activate();
        ((MapBox) getModel()).eAdapters().add(listener);
    }
    
    public void deactivate() {
        if (!isActive()) {
            return;
        }
        super.deactivate();
        ((MapBox) getModel()).eAdapters().remove(listener);
    }
    
    protected void refreshVisuals() {
        MapBox mapBox = (MapBox) this.getModel();
        Point loc = mapBox.getLocation();
        Dimension size = mapBox.getSize();
        Rectangle rectangle = new Rectangle(loc, size);
        
        Map map = mapBox.getMap();
        String title = map.getName();
        if (title == null || title.length() == 0) {
            title = Policy.bind("MapPart.defaultMapTitle"); //$NON-NLS-1$
        }
        
        ((MapFigure) this.getFigure()).setName(title);
        
        ((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(), rectangle);
    }
    
    public void performRequest(Request request) {
        super.performRequest(request);
    }
    
    protected IFigure createFigure() {
        return new MapFigure();
    }
    
    protected void createEditPolicies() {
        super.createEditPolicies();
        installEditPolicy(EditPolicy.COMPONENT_ROLE, new MapEditPolicy());    
    }
        
    protected class InternalPropertyListener extends PropertyListener {
        
        protected void locationChanged() {
            refreshVisuals();
        }
        protected void sizeChanged() {
            refreshVisuals();
        }
        
        protected void mapChanged() {
            refreshVisuals();
        }
    }
}
