/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project;

import net.refractions.udig.project.internal.ProjectPackage;

/**
 * TODO Purpose of net.refractions.udig.project <p> </p>
 * @author   Jesse
 * @since   1.0.0
 */
public class EditManagerEvent {
    /** Indicates the current edit feature has changed. */
    public static final int EDIT_FEATURE=ProjectPackage.EDIT_MANAGER__EDIT_FEATURE;
    /** Indicates the current edit layer has changed. */
    public static final int EDIT_LAYER=ProjectPackage.EDIT_MANAGER__EDIT_LAYER_INTERNAL;
    
    private IEditManager source;
    private int type;
    private Object oldValue;
    private Object newValue;
    
    /**
     * Construct <code>EditManagerEvent</code>.
     *
     * @param source the object that raised the event.
     * @param type the type of event this object represents.
     * @param newValue the new value, if this applies.
     * @param oldValue the old value, if this applies.
     */
    public EditManagerEvent(IEditManager source, int type, Object newValue, Object oldValue) {
        this.newValue=newValue;
        this.oldValue=oldValue;
        this.type=type;
        this.source=source;
    }

    /**
     * Gets the new value, if it applies. 
     * @return   The new value, if it applies.
     * @uml.property   name="newValue"
     */
    public Object getNewValue() {
        return newValue;
    }
    /**
     * Gets the old value, if it applies. 
     * @return   The old value, if it applies.
     * @uml.property   name="oldValue"
     */
    public Object getOldValue() {
        return oldValue;
    }
    /**
     * Gets the source of the event. 
     * @return   the source of the event.
     * @uml.property   name="source"
     */
    public IEditManager getSource() {
        return source;
    }
    /**
     * Gets the type of the event. 
     * @return   the type of the event.
     * @see #EDIT_FEATURE
     * @see #EDIT_LAYER
     * @uml.property   name="type"
     */
    public int getType() {
        return type;
    }
}
