/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project;

import java.util.EventObject;

import net.refractions.udig.project.internal.ProjectPackage;

import org.eclipse.emf.common.notify.Notification;

/**
 * TODO Purpose of net.refractions.udig.project <p> </p>
 * @author   Jesse
 * @since   1.0.0
 */
public class LayerEvent extends EventObject {
	/** <code>serialVersionUID</code> field */
	private static final long serialVersionUID = 3762247556733614390L;

	/** Type of event, FILTER, RESOURCE, ALL or STYLE */
	private int type;

	/**
	 * Previous value of the changed field, may be null if unknown.
	 * <p>
	 * Remeber the event is issued after the change has taken place
	 * </p>
	 */
	private Object oldValue;

	/**
	 * Current value of changed field, may be null if unknown.
	 * <p>
	 * Remeber the event is issued after the change has taken place
	 * </p>
	 */
	private Object newValue;

	// API enum?
	/** Indicate a change has occured the the filter (defines the selection) */
	public static final int FILTER = ProjectPackage.LAYER__FILTER;

	/** Resource used by the layer has changed */
	public static final int RESOURCE = ProjectPackage.LAYER__GEO_RESOURCE;

	/** Indicates something somewhere is different - make no assumptions */
	public static final int ALL = Notification.NO_FEATURE_ID;

	/**
	 * The contents of the blackboard have changed.
	 */
	public static final int STYLE = ProjectPackage.LAYER__STYLE_BLACKBOARD;

	/**
	 * Creates a new event for the given source, indicating that all labels provided by the source
	 * are no longer valid and should be updated.
	 * 
	 * @param source the label provider
	 */
	public LayerEvent(ILayer layer) {
		this(layer, ALL);
	}

	/** Creates a specific kind of layer event, FILTER, RESOURCE, ALL or STYLE */
	public LayerEvent(ILayer layer, int type) {
		this(layer, type, null, null);
	}

	/** Creates a specific kind of layer event, FILTER, RESOURCE, ALL or STYLE */
	public LayerEvent(ILayer layer, int type, Object oldValue, Object newValue) {
		super(layer);
		this.type = type;
		this.oldValue = oldValue;
		this.newValue = newValue;
	}

	/**
	 * The layer being modified.
	 * 
	 * @return The modified layer
	 * @see ILayer
	 */
	public ILayer getSource() {
		return (ILayer) super.getSource();
	}

	/**
     * @return   the newValue.
     * @uml.property   name="newValue"
     */
	public Object getNewValue() {
		return newValue;
	}

	/**
     * @param newValue   The newValue to set.
     * @uml.property   name="newValue"
     */
	public void setNewValue(Object newValue) {
		this.newValue = newValue;
	}

	/**
     * @return   Returns the oldValue.
     * @uml.property   name="oldValue"
     */
	public Object getOldValue() {
		return oldValue;
	}

	/**
     * @param oldValue   The oldValue to set.
     * @uml.property   name="oldValue"
     */
	public void setOldValue(Object oldValue) {
		this.oldValue = oldValue;
	}

	/**
     * Type of event
     * @uml.property   name="type"
     */
	public int getType() {
		return type;
	}

	/**
     * Type of event, ALL, FILTER, RESOURCE, STYLE API include this? why? API I can re-targ events on the fly ... making infinite loops all too easily
     * @param type
     * @uml.property   name="type"
     */
	public void setType(int type) {
		this.type = type;
	}
}
