/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.command;

import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;

/**
 * A UndoableCommand composed of multiple UndoableCommands. Executes and rollsback as a atomic
 * command. See Composite Pattern.
 * 
 * @author jeichar
 * @since 0.3
 * @see CompositeCommand
 * @see UndoableCommand
 */
public class UndoableComposite extends CompositeCommand implements
		UndoableCommand {

	/**
	 * Creates a new instance of UndoableComposite
	 * 
	 * API List<UndoableCommand>
	 * 
	 * @param undoableCommands an ordered list of UndoableCommands
	 */
	public UndoableComposite(List<? extends MapCommand> undoableCommands) {
		super(undoableCommands);
	}

	/**
	 * @see net.refractions.udig.project.internal.command.UndoableCommand#rollback()
	 */
	public void rollback(IProgressMonitor monitor) throws Exception {
		for (int i = commands.size() - 1; i > -1; i--) {
			UndoableCommand command = (UndoableCommand) commands.get(i);
			command.rollback(monitor);
		}
	}

}
