/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal;

import java.awt.Point;
import java.awt.geom.AffineTransform;
import java.io.IOException;

import net.refractions.udig.project.IAbstractContext;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.render.RenderManager;
import net.refractions.udig.project.internal.render.ViewportModel;
import net.refractions.udig.project.render.displayAdapter.IMapDisplay;

import org.eclipse.emf.ecore.EObject;
import org.geotools.data.FeatureResults;
import org.geotools.geometry.JTS.ReferencedEnvelope;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;

/**
 * TODO Purpose of net.refractions.udig.project.internal <p> </p>
 * @author   Jesse
 * @since   1.0.0
 * @model  abstract="true"
 */
public interface AbstractContext extends EObject, Cloneable, IAbstractContext {
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	String copyright = "uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details."; //$NON-NLS-1$

	/**
	 * The Viewport model of the toolkit's map
	 * 
	 * @return The viewport model.
	 */
	ViewportModel getViewportModelInternal();

	/**
	 * The LayerManager of the toolkit's map
	 * 
	 * @return The LayerManager.
	 */
	EditManager getEditManagerInternal();

	/**
	 * The Viewport model of the toolkit's map
	 * 
	 * @return The viewport model.
	 * @model resolveProxies="false"
	 */
	public RenderManager getRenderManagerInternal();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.AbstractContext#getRenderManagerInternal <em>Render Manager Internal</em>}' reference.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Render Manager Internal</em>' reference.
	 * @see #getRenderManagerInternal()
	 * @generated
	 */
	void setRenderManagerInternal(RenderManager value);

	/**
	 * The display area of the Map.
	 * 
	 * @return The display area of the Map.
	 */
	public IMapDisplay getMapDisplay();

	/**
	 * The Map object.
	 * 
	 * @return The Map object
	 * @model resolveProxies="false" many="false"
	 */
	Map getMapInternal();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.AbstractContext#getMapInternal <em>Map Internal</em>}' reference.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Map Internal</em>' reference.
	 * @see #getMapInternal()
	 * @generated
	 */
	void setMapInternal(Map value);

	/**
	 * The map's containing project
	 * 
	 * @return The map's containing project
	 */
	public Project getProjectInternal();

	/**
	 * Gets up the affine transform that will transform from the world to screen. A convenience
	 * method.
	 * 
	 * @return a transform that maps from real world coordinates to the screen
	 */
	public AffineTransform worldToScreenTransform();

	/**
	 * Returns the pixel on the screen for a given coordinate in world space. A convenience method.
	 * 
	 * @param coord A coordinate in world space.
	 * @return The pixel on the screen that the world coordinate is drawn on.
	 */
	public Point worldToPixel(Coordinate coord);

	/**
	 * Converts a coordinate expressed on the device space back to real world coordinates A
	 * convenience method.
	 * 
	 * @param x horizontal coordinate on device space
	 * @param y vertical coordinate on device space
	 * @return The correspondent real world coordinate
	 */
	public Coordinate pixelToWorld(int x, int y);

	/**
	 * Returns the size of a pixel in world units. A convenience method.
	 * 
	 * @return the size of a pixel in world units.
	 */
	public Coordinate getPixelSize();

	/**
	 * Returns a world bounding box the size of a pixel at the location corresponding to the point
	 * on the screen. A convenience method.
	 * 
	 * @return the size of a pixel in world units.
	 */
	public Envelope getPixelBoundingBox(Point screenLocation);

	/**
	 * Returns a world bounding box the scalefactor * (size of a pixel) at the location
	 * corresponding to the point on the screen. A convenience method.
	 * 
	 * @return the size of a pixel in world units.
	 */
	public ReferencedEnvelope getBoundingBox(Point screenLocation,
			int scalefactor);

	/**
	 * Returns all the features that intersect with the bounding box.
	 * 
	 * @param source The featuresource to get features from.
	 * @param bbox The bounding box that acts as a filter
	 * @return all the features that intersect with the bounding box.
	 */
	public FeatureResults getFeaturesInBbox(ILayer layer, Envelope bbox)
			throws IOException;

	/**
	 * Returns a copy of the receiver.
	 */
	Object clone();
}