/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal;

import java.net.URL;
import java.util.List;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IResolveChangeListener;
import net.refractions.udig.core.IBlockingAdaptable;
import net.refractions.udig.project.IBlackboard;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.ui.palette.ColourScheme;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.resource.ImageDescriptor;
import org.geotools.data.FeatureEvent;
import org.geotools.data.Query;
import org.geotools.filter.Filter;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

/**
 * TODO Purpose of net.refractions.udig.project.internal <p> </p>
 * @author   Jesse
 * @since   1.0.0
 * @model
 */
public interface Layer extends EObject, ILayer, IAdaptable, IBlockingAdaptable,
		IResolveChangeListener {

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
	String copyright = "uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details."; //$NON-NLS-1$

	/**
	 * Returns the owning ContextModel object

	 * 
	 * @return the owning ContextModel object
	 * @model many="false" opposite="layers"
	 */
	public ContextModel getContextModel();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getContextModel <em>Context Model</em>}' container reference.
	 * <!-- begin-user-doc --> TODO: Remove Context Model (1 to 1 relationship does not added anything) <!-- end-user-doc -->
	 * @param value the new value of the '<em>Context Model</em>' container reference.
	 * @see #getContextModel()
	 * @generated
	 */
	void setContextModel(ContextModel value);

	/**
	 * Filter indicating the selected features. <p> In order for this value to be useful the layer should be selectable, often a single fid filter during user edit opperations. </p> <p> Note: Filter.ALL indicates no selected Features. (All features are filtered out) </p> <p> A tool may wish to record the previous Filter, before replacing (or adding to) this value. </p> XXX: Consider making use of the General Purpose Blackboard
	 * @return   Filter indicating the selected features. Filter.ALL indicates no selected Features.
	 * @uml.property   name="filter"
	 * @model  transient="true" dataType="org.geotools.filter.Filter"
	 */
	Filter getFilter();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getFilter <em>Filter</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Filter</em>' attribute.
	 * @see #getFilter()
	 * @generated
	 */
	void setFilter(Filter value);

	/**
	 * StyleBlackboard used to persist user supplied appearance settings.
	 * <p>
	 * This method forms part of the EMF model of a Layer, client code should use style(). The
	 * style() method allows access the StyleBlackboard without being troubled by all the model
	 * methods.
	 * </p>
	 * 
	 * @see style();
	 * @model many="false" containment="true"
	 */
	public StyleBlackboard getStyleBlackboard();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getStyleBlackboard <em>Style Blackboard</em>}' containment reference.
	 * <!-- begin-user-doc --> Note: The Rendering Process will be restarted as appearance information changes, this is usual limited to a single Layer. <!-- end-user-doc -->
	 * @param value the new value of the '<em>Style Blackboard</em>' containment reference.
	 * @see #getStyleBlackboard()
	 * @generated
	 */
	void setStyleBlackboard(StyleBlackboard value);

	/**
	 * Returns the ZOrder of the Layer. The ZOrder dictates the order in which the Layer is rendered Low z-orders are rendered first and higher z-orders overwrite low z-orders.
	 * @return   the ZOrder of the Layer.
	 * @uml.property   name="zorder"
	 * @model  volatile="true" transient="true"
	 */
	public int getZorder();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getZorder <em>Zorder</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Zorder</em>' attribute.
	 * @see #getZorder()
	 * @generated
	 */
	void setZorder(int value);

	/**
	 * Indication of Layer status. <p> This is used to provide feedback for a Layers rendering status. </p>
	 * @return
	 * @uml.property   name="status"
	 * @model  transient='true' default='0'
	 */
	public int getStatus();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getStatus <em>Status</em>}' attribute.
	 * <!-- begin-user-doc --> Indication of Layer status. <p> This is used to provide feedback for a Layers rendering status. </p>
	 * @see ILayer#DONE
	 * @see ILayer#ERROR
	 * @see ILayer#MISSING
	 * @see ILayer#UNCONFIGURED
	 * @see ILayer#WARNING
	 * @see ILayer#WORKING
	 * @param status   Status should be one of DONE, ERROR, MISSING, UNCONFIGURED, WARNING, WORKING  <!-- end-user-doc -->
	 * @param value the new value of the '<em>Status</em>' attribute.
	 * @see #getStatus()
	 * @generated
	 */
	void setStatus(int value);

	/**
	 * Sets the current rendering status message
	 * 
	 * @param message the status message
	 */
	void setStatusMessage(String message);

	/**
	 * Set toolset applicability.
	 * 
	 * @param toolCategoryId ID of toolset being considered
	 * @param isApplicable true if layer is to be used with indicated toolset
	 */
	public void setApplicable(String toolCategoryId, boolean isApplicable);

	/**
	 * Indicates this layer is capable of selectable. <p> A Selectable Layer can be used with the Utilities.getQuery opperation. The selection tool category will maintain a separate user interface concept of applicability. The selection tool will not be capabile of considering a non selectable layer applicable. </p> IMPORTANT FUTURE CHANGE NOTE: future design: Instead of isSelectable and isInfoable a single boolean isApplicable( String toolkitID ) will be used to determine whether the layer is applicable to the current tool cateogry. If the layer is not capable of Each tool category extension will declare a color, which will be an underlay for the layer decorator, and an optional validator class, to determine whether the capability for the layer can be set. Indicates this layer is selectable.
	 * @return   <code>true</code> if layer is selectable, <code>false</code> otherwise.
	 * @uml.property   name="selectable"
	 * @model  default="true"
	 */
	public boolean isSelectable();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#isSelectable <em>Selectable</em>}' attribute.
	 * <!-- begin-user-doc --> Used by the user to control which layers are selectable, may be ignored for GeoResources that do not support editing. <!-- end-user-doc -->
	 * @param value the new value of the '<em>Selectable</em>' attribute.
	 * @see #isSelectable()
	 * @generated
	 */
	void setSelectable(boolean value);

	/**
	 * Gets the name from the associated metadata.
	 * @return   the name from the associated metadata
	 * @uml.property   name="name"
	 * @model
	 */
	public String getName();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getName <em>Name</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Name</em>' attribute.
	 * @see #getName()
	 * @generated
	 */
	void setName(String value);

	/**
	 * Returns the layer reference information..
	 * @model type="net.refractions.udig.project.internal.CatalogRef"
	 */
	CatalogRef getCatalogRef();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getCatalogRef <em>Catalog Ref</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Catalog Ref</em>' attribute.
	 * @see #getCatalogRef()
	 * @generated
	 */
	void setCatalogRef(CatalogRef value);

	/**
	 * Gets the unique id, unique within a context model
	 * @return   the id of the layerRef
	 * @uml.property   name="iD"
	 * @model  id="true"
	 */
	public URL getID();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getID <em>ID</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>ID</em>' attribute.
	 * @see #getID()
	 * @generated
	 */
	void setID(URL value);

	/**
	 * Returns whether this layer is currently visible
	 * @return   whether this layer is currently visible
	 * @uml.property   name="visible"
	 * @model
	 */
	public boolean isVisible();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#isVisible <em>Visible</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Visible</em>' attribute.
	 * @see #isVisible()
	 * @generated
	 */
	void setVisible(boolean value);

	/**
	 * Returns the currently preferred.
	 * 
	 * @model transient="true" changeable="true"
	 */
	public IGeoResource getGeoResource();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getGeoResource <em>Geo Resource</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Geo Resource</em>' attribute.
	 * @see #getGeoResource()
	 * @generated
	 */
	void setGeoResource(IGeoResource value);

	/**
	 * Access to resources that hold data for this layer.
	 * 
	 * @see resources() for type safe access
	 * @return IGeoResources that can used to obtain layer data
	 * @model type="net.refractions.udig.catalog.IGeoResource" transient="true" changeable="false"
	 */
	public List<IGeoResource> getGeoResources();

	/**
	 * ImageDescriptor for this Layer. <p> Note we need to do the decorator exention on Layer to reflect status.
	 * @return   Custom glyph - or null if none available.
	 * @uml.property   name="glyph"
	 * @model  transient="true"
	 */
	public ImageDescriptor getGlyph();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getGlyph <em>Glyph</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Glyph</em>' attribute.
	 * @see #getGlyph()
	 * @generated
	 */
	void setGlyph(ImageDescriptor value);

	/**
	 * Query that selects all the features for the layer.
	 * <p>
	 * The selected flag is used with respect to {@link getFilter()}:
	 * <ul>
	 * <li><b>false </b>: Query for layers contents
	 * <li><b>true </b>: Query for the layer's selected features
	 * </ul>
	 * </p>
	 * 
	 * @param layer The layer the Query is associated with.
	 * @param selection true will return a query for the selected features.
	 * @return If selection if false then the features that are not selected are returned, otherwise
	 *         a query that selects all the selected features is returned.
	 * @model volatile="true"
	 */
	public Query getQuery(boolean selection);

	/**
	 * Returns a deep copy of the layer
	 * 
	 * @return a deep copy of the layer
	 */
	public Layer clone();

	/**
	 * Gets the CRS for the layer. NOTE: THIS METHOD MAY BLOCK!!!
	 * 
	 * @param monitor may be null.
	 * @return the CoordinateReferenceSystem of the layer or if the CRS cannot be determined. the
	 *         current map's CRS will be returned, or if this fails the CRS will be WGS84.
	 * @model
	 */
	CoordinateReferenceSystem getCRS(IProgressMonitor monitor);

	/**
	 * A convenience method for getCRS(null). Logs any exceptions with the Plugin log. <p> This method also allows the CRS to be viewed as an attribute by EMF so ui components and events can be raised. This method may block. </p>
	 * @return   the CoordinateReferenceSystem of the layer or if the CRS cannot be determined.
	 * @uml.property   name="cRS"
	 * @model  transient="true" changeable="true"
	 */
	CoordinateReferenceSystem getCRS();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getCRS <em>CRS</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>CRS</em>' attribute.
	 * @see #getCRS()
	 * @generated
	 */
	void setCRS(CoordinateReferenceSystem value);

	/**
	 * Temporary layer properties, used for lightweight collaboration.
	 * <p>
	 * Note these values are not persisted, this can act as a blackboard for plugin collabaration.
	 * These properties are not saved and are reset when a map is opened.
	 * </p>
	 * If you need long term collaboration we can set up a persistent blackboard in the same manner
	 * as StyleBlackbord.
	 * </p>
	 * <p>
	 * Note: Please don't use this to work around limitations of our object model, instead send
	 * email and we can set up a long term solution.
	 * </p>
	 * 
	 * @return Blackboard used for lightweight collaboration.
	 * @model changeable="false" transient="true" resolveProxies="false"
	 */
	IBlackboard getProperties();

	/**
	 * Gets Containing Map.
	 * 
	 * @return
	 */
	Map getMapInternal();

	/**
	 * 
	 * @return
	 * @model 
	 */
	ColourScheme getColourScheme();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getColourScheme <em>Colour Scheme</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Colour Scheme</em>' attribute.
	 * @see #getColourScheme()
	 * @generated
	 */
	void setColourScheme(ColourScheme value);

	/**
	 * @return
	 * @uml.property   name="defaultColor"
	 * @model  
	 */
	String getDefaultColor();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getDefaultColor <em>Default Color</em>}' attribute.
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @param value the new value of the '<em>Default Color</em>' attribute.
	 * @see #getDefaultColor()
	 * @generated
	 */
	void setDefaultColor(String value);

	/**
	 * Returns a list of all the FeatureEvents since the last commit.
	 * 
	 * @return a list of all the FeatureEvents since the last commit.
	 * @model transient='true' type='org.geotools.data.FeatureEvent'
	 */
	List<FeatureEvent> getFeatureChanges();

	/**
	 * @see ILayer#getMinScaleDenominator()
	 * @model
	 */
	public double getMinScaleDenominator();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getMinScaleDenominator <em>Min Scale Denominator</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the new value of the '<em>Min Scale Denominator</em>' attribute.
	 * @see #getMinScaleDenominator()
	 * @generated
	 */
	void setMinScaleDenominator(double value);

	/**
	 * @see ILayer#getMaxScaleDenominator()
	 * @model
	 */
	public double getMaxScaleDenominator();

	/**
	 * Sets the value of the '{@link net.refractions.udig.project.internal.Layer#getMaxScaleDenominator <em>Max Scale Denominator</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the new value of the '<em>Max Scale Denominator</em>' attribute.
	 * @see #getMaxScaleDenominator()
	 * @generated
	 */
	void setMaxScaleDenominator(double value);

}
