/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.command.navigation;

import java.text.MessageFormat;

import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.internal.Policy;
import net.refractions.udig.project.render.displayAdapter.IMapDisplay;

import org.eclipse.core.runtime.IProgressMonitor;

import com.vividsolutions.jts.geom.Envelope;

/**
 * Sets the viewport's bounding box. The bbox have a positive width and height and must have a
 * aspect ratio within 0.0000001 units of the value returned by
 * {@linkplain ViewportModel#getViewportAspectRatio()}.
 * 
 * @author jeichar
 * @since 0.3
 */
public class SetViewportBBoxCommand extends AbstractNavCommand {

	private Envelope newbbox = null;

	/**
	 * Creates a new instance of SetViewportBBoxCommand
	 * 
	 * @param bbox the new bounding box. The new bbox must have a positive width and height and must
	 *        have a aspect ratio within 0.0000001 units of the value returned by
	 *        {@linkplain ViewportModel#getViewportAspectRatio()}.
	 */
	public SetViewportBBoxCommand(Envelope bbox) {
		this.newbbox = bbox;
	}

	/**
	 * @see net.refractions.udig.project.internal.command.MapCommand#copy()
	 */
	public MapCommand copy() {
		return new SetViewportBBoxCommand(newbbox);
	}

	/**
	 * @see net.refractions.udig.project.internal.command.navigation.AbstractNavCommand#runImpl()
	 */
	protected void runImpl(IProgressMonitor monitor) {
		if (Math.abs(newbbox.getWidth() / newbbox.getHeight()
				- model.getAspectRatio()) > 0.00001) {
			IMapDisplay display = model.getRenderManagerInternal().getMapDisplay();
			double scaley = newbbox.getHeight() / display.getHeight();
			double scalex = newbbox.getWidth() / display.getWidth();
			double scale;
			if (scalex > scaley)
				scale = scalex;
			else
				scale = scaley;

			double height = display.getHeight() * scale;
			double width = display.getWidth() * scale;

			double dw = width / 2, dh = height / 2;
			Envelope mapBounds = newbbox;
			double x = (mapBounds.getMaxX() + mapBounds.getMinX()) / 2;
			double y = (mapBounds.getMaxY() + mapBounds.getMinY()) / 2;
			newbbox.init(x - dw, x + dw, y - dh, y + dh);
		}
		model.setBounds(newbbox);
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#getName()
	 */
	public String getName() {
		return MessageFormat
				.format(
						Policy.bind("SetViewportBBoxCommand.setViewArea"), new Object[] { newbbox }); //$NON-NLS-1$
	}

}