/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.command.navigation;

import java.text.MessageFormat;

import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.command.NavCommand;
import net.refractions.udig.project.internal.Policy;

import org.eclipse.core.runtime.IProgressMonitor;

import com.vividsolutions.jts.geom.Coordinate;

/**
 * Sets the center of the viewport. The Coordinate must be in world coordinates. The
 * {@linkplain ViewportModel#pixelToWorld(int, int)}methods can be used to calculate the value.
 * 
 * @author jeichar
 * @since TODO provide version
 */
public class SetViewportCenterCommand extends AbstractNavCommand implements
		NavCommand {

	private Coordinate center;

	/**
	 * Creates a new instance of SetViewportCenterCommand
	 * 
	 * @param center Sets the center of the viewport. The Coordinate must be in world coordinates.
	 */
	public SetViewportCenterCommand(Coordinate center) {
		this.center = center;
	}

	/**
	 * @see net.refractions.udig.project.internal.command.navigation.AbstractNavCommand#runImpl()
	 */
	protected void runImpl(IProgressMonitor monitor) throws Exception {
		model.setCenter(center);
	}

	/**
	 * @see net.refractions.udig.project.internal.command.MapCommand#copy()
	 */
	public MapCommand copy() {
		return new SetViewportCenterCommand(center);
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#getName()
	 */
	public String getName() {
		return MessageFormat
				.format(
						Policy.bind("SetViewportCenterCommand.setViewCenter"), new Object[] { center }); //$NON-NLS-1$
	}

}