/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.commands.edit;

import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.command.UndoableCommand;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Policy;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.FeatureStore;
import org.geotools.feature.Feature;
import org.geotools.filter.FilterFactory;

/**
 * TODO Purpose of net.refractions.udig.project.internal.commands.edit <p> </p>
 * @author   Jesse
 * @since   1.0.0
 */
public class DeleteFeatureCommand extends AbstractEditCommand implements
		UndoableCommand {

	Feature feature;

	private Layer sourceLayer;

	protected boolean done;

	/**
	 * Construct <code>DeleteFeatureCommand</code>.
	 */
	public DeleteFeatureCommand(Feature feature, Layer layer) {
		this.feature = feature;
		this.sourceLayer = layer;
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#run()
	 */
	public void run(IProgressMonitor monitor) throws Exception {
		sourceLayer.getResource(FeatureStore.class, null).removeFeatures(
				FilterFactory.createFilterFactory().createFidFilter(
						feature.getID()));
		map.getEditManagerInternal().setEditFeature(null, null);
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#copy()
	 */
	public MapCommand copy() {
		return new DeleteFeatureCommand(feature, sourceLayer);
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#getName()
	 */
	public String getName() {
		return Policy.bind("DeleteFeatureCommand.deleteFeature"); //$NON-NLS-1$
	}

	/**
	 * @see net.refractions.udig.project.command.UndoableCommand#rollback()
	 */
	public void rollback(IProgressMonitor monitor) throws Exception {
		map.getEditManagerInternal().setEditFeature(feature, sourceLayer);
		map.getEditManagerInternal().addFeature(feature, sourceLayer);
	}

}
