/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.commands.edit;

import java.util.Iterator;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Policy;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.FeatureResults;
import org.geotools.data.FeatureSource;
import org.geotools.data.FeatureStore;
import org.geotools.feature.Feature;
import org.geotools.filter.FilterFactory;

/**
 * TODO Purpose of net.refractions.udig.project.internal.commands.edit <p> </p>
 * @author   Jesse
 * @since   1.0.0
 */
public class NullEditFeatureCommand extends AbstractLayerManagerControlCommand {
	private Feature newVictim;

	private Layer newLayer;

	/**
	 * Creates a new instance of SetEditFeatureCommand.
	 * 
	 * @param feature the new editable Feature.
	 * @param featureStore A featurestore that contains the editable Feature.
	 */
	public NullEditFeatureCommand(Feature feature, ILayer layer) {
		this.newVictim = feature;
		this.newLayer = (Layer) layer;
	}

	/**
	 * Creates a new instance of SetEditFeatureCommand
	 * 
	 * @param feature the new editable Feature
	 */
	public NullEditFeatureCommand(Feature feature) {
		this.newVictim = feature;
	}

	/**
	 * @see net.refractions.udig.project.internal.command.MapCommand#run()
	 */
	public void run(IProgressMonitor monitor) throws Exception {
		if (newVictim == null) {
			layerManager.setEditFeature(null, null);
			return;
		}
		if (newLayer == null) {
			for (Iterator<Layer> iter = layerManager.getMapInternal().getLayersInternal().iterator(); iter.hasNext();) {
				Layer layer = iter.next();
				if (layer.isType(FeatureSource.class)) {
					FeatureStore fs = layer.getResource(FeatureStore.class,
							null);
					FeatureResults results = fs.getFeatures(FilterFactory
							.createFilterFactory().createFidFilter(
									newVictim.getID()));
					if (results.getCount() == 1) {
						newLayer = layer;
						break;
					}
				}
			}
		}
		layerManager.setEditFeature(newVictim, newLayer);
	}

	/**
	 * @see net.refractions.udig.project.internal.command.MapCommand#copy()
	 */
	public MapCommand copy() {
		return new NullEditFeatureCommand(newVictim, newLayer);
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#getName()
	 */
	public String getName() {
		return Policy.bind("SetEditFeatureCommand.setCurrentEditFeature"); //$NON-NLS-1$
	}

}
