package net.refractions.udig.project.internal.commands.edit;

import net.refractions.udig.project.command.AbstractCommand;
import net.refractions.udig.project.command.Command;
import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.command.UndoableCommand;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Policy;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureStore;
import org.geotools.feature.Feature;
import org.geotools.filter.FilterFactory;

/**
 * Replaces the edit feature with a new copy from the datastore.  Any changes that were made to the 
 * edit feature are lost.
 * @author jeichar
 *
 */
public class ResetEditFeatureCommand extends AbstractCommand implements
		MapCommand, UndoableCommand {

	private Feature oldfeature;

	public void run(IProgressMonitor monitor) throws Exception {
		this.oldfeature=getMap().getEditManager().getEditFeature();
		if( oldfeature==null )
			return;
		if ( getMap().getEditManager().getEditLayer()==null ){
			getMap().getEditManagerInternal().setEditFeature(null, null);
			return;
		}
			
		FeatureStore store=getMap().getEditManager().getEditLayer().getResource(FeatureStore.class, monitor);
		FeatureReader reader= store.getFeatures(FilterFactory.createFilterFactory().createFidFilter(oldfeature.getID())).reader();
		if( reader.hasNext() ){
			Feature feature=reader.next();
			Layer layer=getMap().getEditManagerInternal().getEditLayerInternal();
			getMap().getEditManagerInternal().setEditFeature(feature,layer);
		}else{
			getMap().getEditManagerInternal().setEditFeature(null,null);
		}
		
	}

	public Command copy() {
		return new ResetEditFeatureCommand();
	}

	public String getName() {
		return Policy.bind("ResetEditFeatureCommand.0"); //$NON-NLS-1$
	}

	public void rollback(IProgressMonitor monitor) throws Exception {
		getMap().getEditManagerInternal().setEditFeature(oldfeature,
				getMap().getEditManagerInternal().getEditLayerInternal());
	}

}
