/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.commands.edit;

import java.text.MessageFormat;

import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.command.UndoableCommand;
import net.refractions.udig.project.internal.Policy;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.FeatureStore;
import org.geotools.filter.FilterFactory;

/**
 * This command modifies an attribute of the current editFeature(the victim that is currently
 * edittable).
 * 
 * @author jeichar
 * @since 0.3
 */
public class SetAttributeCommand extends AbstractEditCommand implements
		UndoableCommand {
	protected String xpath;

	protected Object value;

	private Object oldValue;

	/**
	 * Creates a new instance of SetAttributeCommand.
	 * 
	 * @param xpath the xpath that identifies an attribute in the current edit feature.
	 * @param value the value that will replace the old attribute value.
	 */
	public SetAttributeCommand(String xpath, Object value) {
		this.xpath = xpath;
		this.value = value;
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#run()
	 */
	public void run(IProgressMonitor monitor) throws Exception {
		this.oldValue = editFeature.getAttribute(xpath);
		editFeature.setAttribute(xpath, value);
		editLayer.getResource(FeatureStore.class, null).modifyFeatures(
				editFeature.getFeatureType().getAttributeType(xpath),
				value,
				FilterFactory.createFilterFactory().createFidFilter(
						editFeature.getID()));
	}

	/**
	 * @see net.refractions.udig.project.internal.command.MapCommand#copy()
	 */
	public MapCommand copy() {
		return new SetAttributeCommand(xpath, value);
	}

	/**
	 * @see net.refractions.udig.project.internal.command.UndoableCommand#rollback()
	 */
	public void rollback(IProgressMonitor monitor) throws Exception {
		editFeature.setAttribute(xpath, oldValue);
		editLayer.getResource(FeatureStore.class, null).modifyFeatures(
				editFeature.getFeatureType().getAttributeType(xpath),
				oldValue,
				FilterFactory.createFilterFactory().createFidFilter(
						editFeature.getID()));
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#getName()
	 */
	public String getName() {
		return MessageFormat
				.format(
						Policy.bind("SetAttributeCommand.setFeatureAttribute"), new Object[] { xpath }); //$NON-NLS-1$
	}
}
