/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.internal.commands.selection;

import net.refractions.udig.project.command.AbstractCommand;
import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.command.UndoableCommand;
import net.refractions.udig.project.internal.Policy;

import org.eclipse.core.runtime.IProgressMonitor;

import com.vividsolutions.jts.geom.Envelope;

/**
 * A MapCommand that selection all features in the bounding box encapsulated by the class.
 * 
 * @author jeichar
 * @since TODO provide version
 */
public class BBoxSelectionCommand extends AbstractCommand implements
		UndoableCommand {

	/** Add bbox to layers' current filters */
	public final static int ADD = 1;

	/** Replaces layers' filters */
	public final static int NONE = 0;

	/** "and" bbox to layer's current filters */
	public final static int SUBTRACT = -1;

	Envelope bbox = null;

	private int modifiers;

	// private FilterQuery query=null;

	/**
	 * Creates a new instance of BBoxSelectionCommand
	 * 
	 * @param bbox
	 */
	public BBoxSelectionCommand(Envelope bbox, int modifiers) {
		this.bbox = bbox;
		this.modifiers = modifiers;
	}

	/**
	 * @see net.refractions.udig.project.command.UndoableCommand#rollback()
	 */
	public void rollback(IProgressMonitor monitor) throws Exception {
		// map.getSelectionModel().getSelections().remove(query);
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#run()
	 */
	public void run(IProgressMonitor monitor) throws Exception {

		// check for key modifiers for AND/OR toggle
		if (modifiers == NONE) {
			getMap().getContextModel().select(bbox);
		} else if (modifiers == ADD) {
			getMap().getContextModel().select(bbox, false);
		} else if (modifiers == SUBTRACT) {
			getMap().getContextModel().select(bbox, true);
		}
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#copy()
	 */
	public MapCommand copy() {
		return new BBoxSelectionCommand(bbox, modifiers);
	}

	/**
	 * @see net.refractions.udig.project.command.MapCommand#getName()
	 */
	public String getName() {
		return Policy.bind("BBoxSelectionCommand.boxSelection"); //$NON-NLS-1$
	}

}