package net.refractions.udig.project.internal.impl;

import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IResolve;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.FeatureSource;
import org.geotools.data.FeatureStore;

/**
 * TODO Purpose of net.refractions.udig.project.internal.impl <p> </p>
 * @author   Jesse
 * @since   1.0.0
 */
public class LayerResource extends IGeoResource {

    /**
     * The layer that owns/created the Resource
     */
    private final LayerImpl layer;

    /**
     * The resource that is wrapped by this object
     */
    private IGeoResource geoResource;

    /**
     * @uml.property   name="resources"
     * @uml.associationEnd   qualifier="key:java.lang.Object java.lang.Object"
     */
    private Map<Class, Object> resources = new HashMap<Class, Object>();

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals( Object obj ) {
        if (obj instanceof LayerResource) {
            LayerResource other = (LayerResource) obj;
            return geoResource.equals(other.geoResource);
        }
        return geoResource.equals(obj);
    }

    /**
     * @see net.refractions.udig.catalog.IGeoResource#hashCode()
     */
    public int hashCode() {
        return geoResource.hashCode();
    }

    /**
     * Construct <code>LayerImpl.LayerResource</code>.
     */
    public LayerResource( LayerImpl impl, IGeoResource resource ) {
        layer = impl;
        this.geoResource = resource;
    }

    /**
     * @see net.refractions.udig.catalog.IResolve#canResolve(java.lang.Class)
     */
    public <T> boolean canResolve( Class<T> adaptee ) {
        return geoResource.canResolve(adaptee);
    }

    /**
     * @see net.refractions.udig.catalog.IResolve#getIdentifier()
     */
    public URL getIdentifier() {
        return geoResource.getIdentifier();
    }

    /**
     * @see net.refractions.udig.catalog.IResolve#getMessage()
     */
    public Throwable getMessage() {
        return geoResource.getMessage();
    }

    /**
     * @see net.refractions.udig.catalog.IResolve#getStatus()
     */
    public Status getStatus() {
        return geoResource.getStatus();
    }

    /**
     * @see net.refractions.udig.catalog.IResolve#members(org.eclipse.core.runtime.IProgressMonitor)
     */
    public List<IResolve> members( IProgressMonitor monitor ) {
        return geoResource.members(monitor);
    }

    /**
     * @see net.refractions.udig.catalog.IGeoResource#resolve(java.lang.Class,
     *      org.eclipse.core.runtime.IProgressMonitor)
     */
    public <T> T resolve( Class<T> adaptee, IProgressMonitor monitor ) throws IOException {
        if (resources.get(adaptee) == null) {
            synchronized (this) {
                if (resources.get(adaptee) == null) {
                    if (FeatureSource.class == adaptee) {
                        if (canResolve(FeatureStore.class)) {
                            Object object = resolve(FeatureStore.class, monitor);
                            if (object != null) {
                                object = new UDIGFeatureStore((FeatureStore) object);

                                registerClasses(FeatureStore.class, object);

                                return adaptee.cast(object);
                            }
                        }
                    }
                    T obj = geoResource.resolve(adaptee, monitor);
                    if (obj instanceof FeatureStore) {
                        FeatureStore store = (FeatureStore) obj;
                        store.setTransaction(layer.getMapInternal().getEditManagerInternal()
                                .getTransaction());
                        ((FeatureStore) obj).addFeatureListener(layer.featureListener);
                    }
                    if (obj != null)
                        registerClasses(adaptee, obj);
                }
            }
        }
        return adaptee.cast(resources.get(adaptee));
    }

    @SuppressWarnings("unchecked")//$NON-NLS-1$
    private <T> void registerClasses( Class<T> clazz, Object obj ) {
        if (clazz.getSuperclass() != null && clazz.getSuperclass() != Object.class)
            registerClasses(clazz.getSuperclass(), obj);
        for( int i = 0; i < clazz.getInterfaces().length; i++ ) {
            registerClasses(clazz.getInterfaces()[i], obj);
        }
        resources.put(clazz, obj);
    }

}