/**
 * <copyright></copyright> $Id: CompositeRendererImpl.java 15417 2005-08-17 22:57:15Z jeichar $
 */
package net.refractions.udig.project.internal.render.impl;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.project.internal.ProjectPlugin;
import net.refractions.udig.project.internal.render.CompositeRenderContext;
import net.refractions.udig.project.internal.render.MultiLayerRenderer;
import net.refractions.udig.project.internal.render.RenderContext;
import net.refractions.udig.project.internal.render.RenderExecutor;
import net.refractions.udig.project.internal.render.RenderFactory;
import net.refractions.udig.project.internal.render.RenderListenerAdapter;
import net.refractions.udig.project.internal.render.RenderManager;
import net.refractions.udig.project.internal.render.RenderPackage;
import net.refractions.udig.project.internal.render.Renderer;
import net.refractions.udig.project.internal.render.RendererCreator;
import net.refractions.udig.project.internal.render.SelectionLayer;
import net.refractions.udig.project.render.IRenderContext;
import net.refractions.udig.project.render.RenderException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;

import com.vividsolutions.jts.geom.Envelope;

/**
 * <ul>
 * <li>Combines the output from several renderer into a single image.</li>
 * <li>Listens to its context (CompositeContext) and creates new
 * RenderExecutors when required.</li>
 * 
 * @author Jesse
 * @since 1.0.0
 */
public class CompositeRendererImpl extends RendererImpl implements
		MultiLayerRenderer {

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated
	 */
	public static final String copyright = "uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details."; //$NON-NLS-1$

	private List<RenderExecutor> renderExecutors = null;

	static final AffineTransform IDENTITY = new AffineTransform();

	/**
	 * @see net.refractions.udig.project.internal.render.MultiLayerRenderer#children()
	 */
	public List<Renderer> children() {
		List<Renderer> children = new ArrayList<Renderer>();
		for (RenderExecutor victim : getRenderExecutors()) {
			children.add(victim.getRenderer());
		}
		return children;
	}

	/**
	 * 
	 */
	Adapter contextListener = new AdapterImpl() {
		/**
		 * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification msg) {
			switch (msg.getFeatureID(CompositeRenderContext.class)) {
			case RenderPackage.COMPOSITE_RENDER_CONTEXT__CONTEXTS_INTERNAL: {
				switch (msg.getEventType()) {
				case Notification.ADD: {
					RenderContext context = (RenderContext) msg.getNewValue();
					if (findExecutor(context) == null) {
						Renderer renderer = getRendererCreator(context)
								.getRenderer(context);
						getRenderExecutors()
								.add(createRenderExecutor(renderer));
					}
					return;
				}
				case Notification.ADD_MANY: {
					@SuppressWarnings("unchecked") List<RenderContext> newValue = (List<RenderContext>) msg.getNewValue(); //$NON-NLS-1$
					List<RenderExecutor> renderers = new ArrayList<RenderExecutor>();
					for (RenderContext context : newValue) {
						if (findExecutor(context) == null) {
							Renderer renderer = getRendererCreator(context)
									.getRenderer(context);
							renderers.add(createRenderExecutor(renderer));
						}
					}
					if (!renderers.isEmpty()) {
						getRenderExecutors().addAll(renderers);
					}
					return;
				}
				case Notification.REMOVE: {
					RenderContext context = (RenderContext) msg.getOldValue();
					RenderExecutor executor = findExecutor(context);
					getRenderExecutors().remove(executor);
					executor.dispose();
					return;
				}
				case Notification.REMOVE_MANY: {
					@SuppressWarnings("unchecked") List<RenderContext> oldValue = (List<RenderContext>) msg.getOldValue(); //$NON-NLS-1$
					List<RenderExecutor> renderersToRemove = new ArrayList<RenderExecutor>();
					for (RenderContext context : oldValue) {
						RenderExecutor executor = findExecutor(context);
						getRenderExecutors().remove(executor);
						executor.dispose();
					}
					getRenderExecutors().removeAll(renderersToRemove);
					return;
				}
				}// switch
				return;
			}// case
			}// switch
		}// method
	};

	/**
	 * @see net.refractions.udig.project.internal.render.impl.RendererImpl#getContext()
	 */
	public CompositeRenderContext getContext() {
		return (CompositeRenderContext) super.getContext();
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	protected CompositeRendererImpl() {
		super();
	}

	@SuppressWarnings("unchecked")//$NON-NLS-1$
	protected void sortRenderers() {
		Collections.sort(getRenderExecutors(), new Comparator() {
			public int compare(Object e1, Object e2) {
				if (e1 instanceof SelectionLayer) {
					return ((RenderExecutor) e1).getContext().getLayer()
							.compareTo(
									((RenderExecutor) e2).getContext()
											.getLayer());
				} else {
					return ((RenderExecutor) e2).getContext().getLayer()
							.compareTo(
									((RenderExecutor) e1).getContext()
											.getLayer());
				}
			}
		});
	}

	/**
	 * Called to remove the corresponding RenderExecutor from the list of
	 * RenderExecutors.
	 * 
	 * @param renderer
	 *            the renderer that has been removed
	 */
	protected RenderExecutor findExecutor(RenderContext context) {
		for (Iterator<RenderExecutor> eIter = getRenderExecutors().iterator(); eIter
				.hasNext();) {
			RenderExecutor executor = (RenderExecutor) eIter.next();
			if (executor.getRenderer().getContext() == context)
				return executor;
		}
		return null;
	}

	/**
	 * @param renderer
	 */
	@SuppressWarnings("unchecked")//$NON-NLS-1$
	protected RenderExecutor createRenderExecutor(Renderer renderer) {
		RenderExecutor executor = RenderFactory.eINSTANCE.createRenderExecutor(renderer);
		executor.eAdapters().add(new RenderListenerAdapter() {

			/**
			 * @see net.refractions.udig.project.internal.render.RenderListenerAdapter#renderDisposed(org.eclipse.emf.common.notify.Notification)
			 */
			protected void renderDisposed(Notification msg) {
				EObject obj = (EObject) getTarget();
				obj.eAdapters().remove(this);
			}

			/**
			 * @see net.refractions.udig.project.internal.render.RenderListenerAdapter#renderUpdate()
			 */
			protected void renderUpdate() {
				synchronized (CompositeRendererImpl.this) {
					try {
						refreshImage();
						setState(RENDERING);
					} catch (RenderException e) {
						// TODO Catch e
					}
				}
			}

			/**
			 * @see net.refractions.udig.project.internal.render.RenderListenerAdapter#renderDone()
			 */
			protected void renderDone() {
				synchronized (CompositeRendererImpl.this) {
					for (Renderer renderer : getRenderExecutors()) {
						if (renderer.getContext().isVisible()
								&& renderer.getState() != DONE) {
							renderUpdate();
							return;
						}
					}
				}
				try {
					refreshImage();
				} catch (RenderException e) {
					ProjectPlugin.log(null, e);
				}
				setState(DONE);
			}

			protected void renderRequest() {
				setState(RENDER_REQUEST);
			}
		});
		return executor;
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @throws RenderException
	 * 
	 * @generated NOT
	 */
	public void refreshImage() throws RenderException {
		sortRenderers();
		synchronized (getContext()) {
			Graphics2D g = null;
			try {
				getContext().clearImage();
				BufferedImage image = getContext().getImage();
				g = image.createGraphics();
				List<RenderExecutor> executors = new ArrayList<RenderExecutor>(
						getRenderExecutors());
				RENDERERS: for (RenderExecutor executor : executors) {

					if (!executor.getContext().isVisible())
						continue RENDERERS;

					if (executor.getState() == NEVER) {
						executor.render((Envelope) null, null);
						continue RENDERERS;
					} else if (executor.getRenderer() instanceof MultiLayerRenderer) {
						((MultiLayerRenderer) executor.getRenderer())
								.refreshImage();
					}

					g.drawRenderedImage(executor.getContext().getImage(),
							IDENTITY);
				}
			} catch (IllegalStateException e) {
				stopRendering();
				return;
			} finally {
				if (g != null)
					g.dispose();
			}
		}

	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @throws RenderException
	 * 
	 * @generated NOT
	 */
	public void render(Graphics2D destination, IProgressMonitor monitor)
			throws RenderException {
		for (Iterator iter = getRenderExecutors().iterator(); iter.hasNext();) {
			((RenderExecutor) iter.next()).render(destination, monitor);
		}
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @throws RenderException
	 * 
	 * @generated NOT
	 */
	public synchronized void render(IProgressMonitor monitor)
			throws RenderException {
		if (getRenderExecutors().size() == 0)
			setState(DONE);
		for (Iterator iter = getRenderExecutors().iterator(); iter.hasNext();) {
			RenderExecutor executor = (RenderExecutor) iter.next();
			executor.render((Envelope) null, monitor);
		}
	}

	/**
	 * @see net.refractions.udig.project.internal.render.impl.RendererImpl#setContext(net.refractions.udig.project.render.RenderContext)
	 */
	@SuppressWarnings("unchecked")//$NON-NLS-1$
	public void setContext(IRenderContext newContext) {
		if (context != null) {
			((RenderContext) context).eAdapters().remove(contextListener);
		}
		if (newContext != null) {
			((RenderContext) newContext).eAdapters().add(contextListener);
			for (RenderContext context : ((CompositeRenderContext) newContext)
					.getContextsInternal()) {
				if (findExecutor(context) == null) {
					Renderer renderer = getRendererCreator(context)
							.getRenderer(context);
					getRenderExecutors().add(createRenderExecutor(renderer));
				}
			}
			super.setContext(newContext);
		}
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 */
	public void stopRendering() {
		for (Iterator iter = getRenderExecutors().iterator(); iter.hasNext();) {
			((RenderExecutor) iter.next()).stopRendering();
		}
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @uml.property name="renderExecutors"
	 * @generated NOT
	 */
	public List<RenderExecutor> getRenderExecutors() {
		if (renderExecutors == null)
			renderExecutors = Collections
					.synchronizedList(new ArrayList<RenderExecutor>());
		return renderExecutors;
	}

	RendererCreator rendererCreator = null;

	RendererCreator getRendererCreator(RenderContext context) {
		rendererCreator = ((RenderManager) context.getRenderManager())
				.getRendererCreator();
		rendererCreator.setContext(context);
		return rendererCreator;
	}

	/**
	 * @see net.refractions.udig.project.internal.render.impl.RendererImpl#dispose()
	 */
	public synchronized void dispose() {
		for (Iterator iter = getRenderExecutors().iterator(); iter.hasNext();) {
			Renderer renderer = (Renderer) iter.next();
			renderer.dispose();
		}
		getRenderExecutors().clear();
	}

	/**
	 * @see net.refractions.udig.project.internal.render.Renderer#getInfo(java.awt.Point,
	 *      net.refractions.udig.project.Layer)
	 * 
	 * public InfoList getInfo(Point screenLocation) throws IOException {
	 * 
	 * InfoList infos = new InfoList(screenLocation);
	 * 
	 * Iterator iter = getRenderExecutors().iterator(); while (iter.hasNext()) {
	 * Renderer renderer = ((RenderExecutor) iter.next());
	 * 
	 * List results = renderer.getInfo(screenLocation);
	 * 
	 * if (results != null) { infos.addAll(results); } }
	 * 
	 * return infos; }
	 */

	/**
	 * @throws RenderException
	 * @see net.refractions.udig.project.internal.render.Renderer#render(com.vividsolutions.jts.geom.Envelope)
	 */
	public void render(Envelope bounds, IProgressMonitor monitor)
			throws RenderException {
		if (getRenderExecutors().size() == 0)
			setState(DONE);
		for (Iterator iter = getRenderExecutors().iterator(); iter.hasNext();) {
			RenderExecutor executor = (RenderExecutor) iter.next();
			executor.render(bounds, monitor);
		}
	}

	/**
	 * @see net.refractions.udig.project.render.IMultiLayerRenderer#getIChildren()
	 */
	public List getIChildren() {
		return children();
	}

	/**
	 * @see net.refractions.udig.project.render.IMultiLayerRenderer#getIContext()
	 */
	public IRenderContext getIContext() {
		return getContext();
	}

} // CompositeRendererImpl
