/**
 * <copyright></copyright> $Id: RendererCreatorImpl.java 16045 2005-10-05 03:41:19Z jgarnett $
 */
package net.refractions.udig.project.internal.render.impl;

import java.awt.Graphics2D;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.core.internal.ExtensionPointProcessor;
import net.refractions.udig.core.internal.ExtensionPointUtil;
import net.refractions.udig.project.IBlackboard;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.ContextModel;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.ProjectPackage;
import net.refractions.udig.project.internal.ProjectPlugin;
import net.refractions.udig.project.internal.render.CompositeRenderContext;
import net.refractions.udig.project.internal.render.MultiLayerRenderer;
import net.refractions.udig.project.internal.render.RenderContext;
import net.refractions.udig.project.internal.render.RenderManager;
import net.refractions.udig.project.internal.render.RenderPackage;
import net.refractions.udig.project.internal.render.Renderer;
import net.refractions.udig.project.internal.render.RendererCreator;
import net.refractions.udig.project.internal.render.SelectionLayer;
import net.refractions.udig.project.render.IMultiLayerRenderer;
import net.refractions.udig.project.render.IRenderContext;
import net.refractions.udig.project.render.IRenderMetrics;
import net.refractions.udig.project.render.IRenderMetricsFactory;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.impl.EObjectImpl;
import org.eclipse.emf.ecore.util.EObjectEList;
import org.geotools.data.FeatureSource;

import com.vividsolutions.jts.geom.Envelope;

/**
 * TODO Purpose of net.refractions.udig.project.internal.render.impl <p> </p>
 * @author   Jesse
 * @since   1.0.0
 * @generated
 */
public class RendererCreatorImpl extends EObjectImpl implements RendererCreator {
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    public static final String copyright = "uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details."; //$NON-NLS-1$

    /**
     * The cached value of the '{@link #getContext() <em>Context</em>}' reference. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see #getContext()
     * @generated
     * @ordered
     */
    protected RenderContext context = null;

    /**
     * The cached value of the '{@link #getLayers() <em>Layers</em>}' reference list. <!--
     * begin-user-doc --> <!-- end-user-doc -->
     * 
     * @see #getLayers()
     * @generated
     * @ordered
     */
    protected EList layers = null;

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected RendererCreatorImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    protected EClass eStaticClass() {
		return RenderPackage.eINSTANCE.getRendererCreator();
	}

	/**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * @uml.property   name="context"
     * @generated
     */
    public RenderContext getContext() {
		return context;
	}

	/**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * @uml.property   name="context"
     * @generated
     */
    public void setContext(RenderContext newContext) {
		RenderContext oldContext = context;
		context = newContext;
		if (eNotificationRequired())
			eNotify(new ENotificationImpl(this, Notification.SET, RenderPackage.RENDERER_CREATOR__CONTEXT, oldContext, context));
	}

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * @uml.property   name="layers"
     * @generated  NOT
     */
    @SuppressWarnings("unchecked") //$NON-NLS-1$
	public synchronized List<Layer> getLayers() {
        if (layers == null) {
            layers = new EObjectEList(Layer.class, this, RenderPackage.RENDERER_CREATOR__LAYERS);
        }
        return layers;
    }

    /**
     * <code>MetricsMap</code> maintains a list of metrics by Layer.
     * @uml.property   name="metrics"
     * @uml.associationEnd   qualifier="key:java.lang.Object java.util.SortedSet<IRenderMetrics>"
     */
    Map<Layer, SortedSet<IRenderMetrics>> MetricsMap = new HashMap<Layer, SortedSet<IRenderMetrics>>();

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    public Renderer getRenderer( RenderContext context ) {

        // Part 1 of decision goes here
        Object o = MetricsMap.get(context.getLayer());
        if (o == null)
            createConfiguration();
        return MetricsMap.get(context.getLayerInternal()).first().createRenderer();
    }

    void createConfiguration() {
        Map<Layer, RenderContext> configuration = Collections
                .synchronizedMap(new HashMap<Layer, RenderContext>());
        initRenderMetrics();
        
        Set<Layer> configured = new HashSet<Layer>();
        List<Layer> layers = new ArrayList<Layer>();
        layers.addAll(getLayers());

        clearContexts();

        Collections.sort(layers);
        LAYERS: for( int i = 0; i < layers.size(); i++ ) {
            Layer layer = layers.get(i);

            if (configured.contains(layer)) {
                continue LAYERS;
            }
            SortedSet<IRenderMetrics> layerfactories = MetricsMap.get(layer);

            if (layerfactories.isEmpty()) {
                // nobody loves this layer
                // layer.setStatus( Layer.UNCONFIGURED );
                continue LAYERS;
            } else {
                IRenderMetrics metrics = layerfactories.first(); // sorted in order of preference

                if (metrics != null && metrics.getRenderContext() instanceof CompositeRenderContext) {
                    ((CompositeRenderContext) metrics.getRenderContext()).getContextsInternal().clear();
                    CONTEXT: for( int j = i; j < layers.size(); j++ ) {
                        SortedSet<IRenderMetrics> layerfactories2 = MetricsMap.get(layers.get(j));
                        IRenderMetrics metrics2 = layerfactories2.first();
                        try {
                            if (isResourceSame(metrics, metrics2) && metrics.canAddLayer(layers.get(j))) {
                                ((CompositeRenderContext) metrics.getRenderContext()).getContextsInternal()
                                        .add((RenderContext) metrics2.getRenderContext());
                                if( !configured.contains(layers.get(j)))
                                	configured.add(layers.get(j));
                                
                                configuration.put(layers.get(j), (RenderContext) metrics
                                        .getRenderContext());
                            } else {
                                break CONTEXT;
                            }
                        } catch (Exception e) {
                            break CONTEXT;
                        }
                    }
                }
                configuration.put(layer, (RenderContext) metrics.getRenderContext());
            }
        }
        if (this.configuration == null || this.configuration.size()!=configuration.size())
            this.configuration = configuration;
        else {
            for( Map.Entry<Layer, RenderContext> entry : this.configuration.entrySet() ) {
                if (configuration.get(entry.getKey()) != entry.getValue()) {
                    this.configuration = configuration;
                }
            }
        }
    }

    /**
     * Checks to see if both metrics objects use the same resource
     * 
     */
    private boolean isResourceSame( IRenderMetrics metrics, IRenderMetrics metrics2 )
            throws IOException {
        return metrics2.getRenderContext().getGeoResource().parent(null) == (metrics
                .getRenderContext().getGeoResource().parent(null));
    }
    /**
     * clears all composite context so that have no subcontexts.
     */
    private void clearContexts() {
        if (configuration != null) {
            for( RenderContext context : configuration.values() ) {
                if (context instanceof CompositeRenderContext) {
                    ((CompositeRenderContext) context).getContextsInternal().clear();
                }
            }
        }
    }
    /**
     * Initialize all known render metrics for a given layer
     */
    private void initRenderMetrics() {
        for( Layer layer : getLayers() ) {
            if (!MetricsMap.containsKey(layer))
                initFactories(layer);
        }
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    public RenderContext getRenderContext( Layer layer ) {
        if (configuration == null)
            createConfiguration();
        return configuration.get(layer);
    }

    /**
     * <!-- begin-user-doc --> <!-- end-user-doc -->
     * 
     * @generated NOT
     */
    public synchronized void changed( Notification event ) {
        if (((event.getNotifier() instanceof ContextModel || event.getNotifier() instanceof RenderManager)
                && event.getFeatureID(ContextModel.class) == ProjectPackage.CONTEXT_MODEL__LAYERS)
                || event.getNotifier() instanceof Layer && event.getFeatureID(Layer.class) == ProjectPackage.LAYER__STYLE_BLACKBOARD) {
            switch( event.getEventType() ) {
            case Notification.ADD: {
                Layer layer = (Layer) event.getNewValue();
                getLayers().add(layer);
                if (layer.isType(FeatureSource.class))
                    getLayers().add(new SelectionLayer(layer));
                break;
            }
            case Notification.ADD_MANY: {
                List<Layer> layers = new ArrayList<Layer>();
                for( Layer layer : (Collection<? extends Layer>) event.getNewValue() ) {
                    layers.add(layer);
                    try {
                        if (layer.getResource(FeatureSource.class, null) != null
                                && findSelectionLayer(layer) == null)
                            layers.add(new SelectionLayer(layer));
                    } catch (IOException e) {
                        ProjectPlugin.log( null, e);
                    }
                }
                getLayers().addAll(layers);
                break;
            }
            case Notification.REMOVE: {
                getLayers().remove(event.getOldValue());
                Layer selectionLayer = findSelectionLayer((Layer) event.getOldValue());
                if (selectionLayer != null)
                    getLayers().remove(selectionLayer);
                break;
            }
            case Notification.REMOVE_MANY: {
                List<Layer> layers = new ArrayList<Layer>();
                layers.addAll((Collection<? extends Layer>) event.getOldValue());
                for( Layer layer : (Collection<? extends Layer>) event.getOldValue() ) {
                    Layer selectionLayer = findSelectionLayer(layer);
                    if (selectionLayer != null)
                        layers.add(selectionLayer);
                }
                getLayers().removeAll(layers);
                break;
            }
            default:
                break;
            }
            createConfiguration();
        }
    }

    /**
     * Locates the selection layer for layer or returns null;
     * 
     * @return the selection layer for layer or returns null;
     */
    public SelectionLayer findSelectionLayer( ILayer targetLayer ) {
        try {
            if (targetLayer.getResource(FeatureSource.class, null) == null)
                return null;
        } catch (IOException e) {
            return null;
        }
        for( Layer layer : getLayers() )
            if (layer instanceof SelectionLayer)
                if (((SelectionLayer) layer).getWrappedLayer() == targetLayer)
                    return (SelectionLayer) layer;

        return null;
    }

    /**
     * @uml.property   name="configuration"
     * @uml.associationEnd   qualifier="key:java.lang.Object net.refractions.udig.project.internal.render.RenderContext"
     */
    Map<Layer, RenderContext> configuration;

    IRenderMetrics getRenderMetrics( Layer layer ) {
        if (!configuration.containsKey(layer)) {
            SortedSet<IRenderMetrics> layerfactories = MetricsMap.get(layer);
            if (layerfactories == null) {
                initFactories(layer);
                layerfactories = MetricsMap.get(layer);
            }
            int current = 0;
            IRenderMetrics bestfit = null;
            for( Iterator iter = layerfactories.iterator(); iter.hasNext(); ) {
                IRenderMetrics factory = (IRenderMetrics) iter.next();
                int tmp = rate(factory);
                if (tmp > current)
                    bestfit = factory;
            }
            try {
                if (bestfit != null) {
                    configuration.put(layer, (RenderContext) bestfit.getRenderContext());
                } else {
                    configuration.put(layer, new DumbRendererMetrics(layer, null)
                            .getRenderContext());
                }
            } catch (Exception e) {
                ProjectPlugin.getPlugin().log(e);
                configuration.put(layer, new DumbRendererMetrics(layer, bestfit.getRenderContext())
                        .getRenderContext());
            }
        }
        return (IRenderMetrics) configuration.get(layer);
    }

    /**
     * TODO Purpose of net.refractions.udig.project.internal.render.impl <p> </p>
     * @author   Jesse
     * @since   1.0.0
     */
    class DumbRendererMetrics implements IRenderMetrics {

        private RenderContext context;

        /**
         * @param layer
         */
        public DumbRendererMetrics( Layer layer, IRenderContext context ) {
            this.context = (RenderContext) context;
        }

        /**
         * @see net.refractions.udig.project.render.IRenderMetrics#createRenderer()
         */
        public Renderer createRenderer() {
            return new DumbRenderer();
        }

        
        /**
         * @see net.refractions.udig.project.render.IRenderMetrics#getRenderContext()
         */
        public RenderContext getRenderContext() {
            return context;
        }

        /**
         * @see net.refractions.udig.project.render.IRenderMetrics#setRenderContext(net.refractions.udig.project.render.RenderContext)
         */
        public void setRenderContext( IRenderContext context ) {
            // do nothing;
        }

        /**
         * @see net.refractions.udig.project.render.IRenderMetrics#getRenderMetricsFactory()
         */
        public IRenderMetricsFactory getRenderMetricsFactory() {
            return null;
        }

        public boolean canAddLayer( ILayer layer ) {
            return false;
        }

        public boolean canStyle( String SyleID, Object value ) {
            return false;
        }

    }

    class DumbRenderer extends RendererImpl implements Renderer {

        /**
         * @see net.refractions.udig.project.internal.render.impl.RendererImpl#render(java.awt.Graphics2D)
         */
        public void render( Graphics2D destination, IProgressMonitor monitor ) {
            // do nothing
        }

        /**
         * @see net.refractions.udig.project.internal.render.impl.RendererImpl#getInfo(java.awt.Point)
         *
        public InfoList getInfo( Point screenLocation ) {
            // do nothing
            return null;
        }*/

        /**
         * @see net.refractions.udig.project.internal.render.impl.RendererImpl#stopRendering()
         */
        public void stopRendering() {
            // do nothing
        }

        /**
         * @see net.refractions.udig.project.internal.render.impl.RendererImpl#dispose()
         */
        public void dispose() {
            // do nothing
        }

        /**
         * @see net.refractions.udig.project.internal.render.impl.RendererImpl#render(com.vividsolutions.jts.geom.Envelope,
         *      org.eclipse.core.runtime.IProgressMonitor)
         */
        public void render( Envelope bounds, IProgressMonitor monitor ) {
            // do nothing
        }

    }

    private int rate( IRenderMetrics factory ) {
        return 1;
    }

    private void initFactories( Layer layer ) {
        Processor p = new Processor(layer);

        ExtensionPointUtil.process(ProjectPlugin.getPlugin(), Renderer.RENDER_EXT, p);
        MetricsMap.put(layer, p.rFactories);

    }

    /**
     * TODO Purpose of net.refractions.udig.project.internal.render.impl <p> </p>
     * @author   Jesse
     * @since   1.0.0
     */
    private class Processor implements ExtensionPointProcessor {
        Layer layer;

        SortedSet<IRenderMetrics> rFactories = new TreeSet<IRenderMetrics>(
                new RenderMetricsSorter());

        /**
         * Creates an new instance of Processor
         * 
         * @param layer The layer which needs to be rendered
         */
        public Processor( Layer layer ) {
            this.layer = layer;
        }

        /**
         * @see net.refractions.udig.core.internal.ExtensionPointProcessor#process(org.eclipse.core.runtime.IExtension,
         *      org.eclipse.core.runtime.IConfigurationElement)
         */
        public void process( IExtension extension, IConfigurationElement element ) {
            IConfigurationElement[] elements = extension.getConfigurationElements();

            try {
                IRenderMetricsFactory metricsFactory = (IRenderMetricsFactory) elements[0]
                        .createExecutableExtension("class"); //$NON-NLS-1$

                List data = layer.getGeoResources();
                for( Iterator iter = data.iterator(); iter.hasNext(); ) {
                    IGeoResource resource = (IGeoResource) iter.next();

                    RenderContext context;

                    if (IMultiLayerRenderer.class
                            .isAssignableFrom(metricsFactory.getRendererType())) {
                        context = RenderFactoryImpl.eINSTANCE.createCompositeRenderContext();
                    } else {
                        context = RenderFactoryImpl.eINSTANCE
                                .createRenderContext(layer instanceof SelectionLayer);
                    }
                    context.setMapInternal(getContext().getMapInternal());
                    context.setRenderManagerInternal(getContext().getRenderManagerInternal());
                    context.setLayerInternal(layer);
                    context.setGeoResourceInternal(resource);

                    if (metricsFactory.canRender(context)) {
                        rFactories.add(metricsFactory.createMetrics(context));
                    }
                }
            } catch (CoreException e) {
                // TODO log the exception
                e.printStackTrace();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    public Object eGet(EStructuralFeature eFeature, boolean resolve) {
		switch (eDerivedStructuralFeatureID(eFeature)) {
			case RenderPackage.RENDERER_CREATOR__CONTEXT:
				return getContext();
			case RenderPackage.RENDERER_CREATOR__LAYERS:
				return getLayers();
		}
		return eDynamicGet(eFeature, resolve);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    public void eSet(EStructuralFeature eFeature, Object newValue) {
		switch (eDerivedStructuralFeatureID(eFeature)) {
			case RenderPackage.RENDERER_CREATOR__CONTEXT:
				setContext((RenderContext)newValue);
				return;
			case RenderPackage.RENDERER_CREATOR__LAYERS:
				getLayers().clear();
				getLayers().addAll((Collection)newValue);
				return;
		}
		eDynamicSet(eFeature, newValue);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    public void eUnset(EStructuralFeature eFeature) {
		switch (eDerivedStructuralFeatureID(eFeature)) {
			case RenderPackage.RENDERER_CREATOR__CONTEXT:
				setContext((RenderContext)null);
				return;
			case RenderPackage.RENDERER_CREATOR__LAYERS:
				getLayers().clear();
				return;
		}
		eDynamicUnset(eFeature);
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * @generated
	 */
    public boolean eIsSet(EStructuralFeature eFeature) {
		switch (eDerivedStructuralFeatureID(eFeature)) {
			case RenderPackage.RENDERER_CREATOR__CONTEXT:
				return context != null;
			case RenderPackage.RENDERER_CREATOR__LAYERS:
				return layers != null && !layers.isEmpty();
		}
		return eDynamicIsSet(eFeature);
	}

    class RenderMetricsSorter implements Comparator<IRenderMetrics> {

        /**
         * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
         */
        public int compare( IRenderMetrics o1, IRenderMetrics o2 ) {
            if (rate(o1) == rate(o2))
                return 0;
            return rate(o1) > rate(o2) ? -1 : 1;
        }

        private int rateAppearance( final IRenderMetrics metrics ) {
            ILayer layer = metrics.getRenderContext().getLayer();
            final IBlackboard style = layer.getStyleBlackboard();
            class Evaluator implements ExtensionPointProcessor {
                int rating = 0;
                public void process( IExtension extension, IConfigurationElement element ) throws Exception {
                    String id = element.getAttribute( "id" ); //$NON-NLS-1$
                    Object value = style.get( id );
                    if( metrics.canStyle( id, value ) ){
                        rating++; // more stylish
                    }
                }                
            };                        
            Evaluator judge = new Evaluator();
            ExtensionPointUtil.process( ProjectPlugin.getPlugin(),
                    "net.refractions.udig.project.style",  judge ); //$NON-NLS-1$
            return judge.rating;
        }
        private int rate( IRenderMetrics metrics ) {
            int rating = rateAppearance( metrics );
                        
            if ( rating == 0)
                return -1;
            if (MultiLayerRenderer.class.isAssignableFrom(metrics.getRenderMetricsFactory()
                    .getRendererType())) {
                rating += hasSameResource(getLayers()
                        .indexOf(metrics.getRenderContext().getLayer()), -1, null, rating);
                rating += hasSameResource(getLayers()
                        .indexOf(metrics.getRenderContext().getLayer()), +1, null, rating);
            }
            return rating;
        }

        /**
         * @param layer
         * @param rating
         * @return
         */
        private int hasSameResource( int layerIndex, int d, IService matchedResource, int rating ) {
            Layer currentlayer = getLayers().get(layerIndex);
            if (layerIndex + d < 0 || layerIndex + d == getLayers().size())
                return 1;
            Layer nextLayer = getLayers().get(layerIndex + d);
            if (matchedResource != null) {
                if (nextLayer.getGeoResources().contains(matchedResource))
                    return hasSameResource(layerIndex + d, d, matchedResource, rating) + 1;
            } else {
                for( Iterator iter = nextLayer.getGeoResources().iterator(); iter.hasNext(); ) {
                    IGeoResource resource = (IGeoResource) iter.next();
                    for( Iterator iterator = currentlayer.getGeoResources().iterator(); iterator
                            .hasNext(); ) {

                        IGeoResource resource2 = (IGeoResource) iterator.next();
                        try {
                            if (resource.parent(null) == resource2.parent(null))
                                return hasSameResource(layerIndex + d, d, resource.service(null),
                                        rating) + 1;
                        } catch (IOException e) {
                            return 1;
                        }
                    }
                }
            }
            return 1;
        }

    }

    Set<RenderContext> contexts = new HashSet<RenderContext>();

    /**
     * @see net.refractions.udig.project.internal.render.RendererCreator#getConfiguration()
     * @uml.property   name="configuration"
     */
    public Collection<RenderContext> getConfiguration() {
        if (configuration == null) {
            createConfiguration();
        }
        contexts.clear();
        for( RenderContext context : configuration.values() ) {
            contexts.add(context);
        }
        contexts=Collections.synchronizedSet(contexts);
        return contexts;
    }

} // RendererCreatorImpl
