/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.ui;

import java.util.Iterator;

import net.refractions.udig.project.IProjectElement;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Project;
import net.refractions.udig.project.internal.ProjectElement;

import org.eclipse.emf.common.ui.action.WorkbenchWindowActionDelegate;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.geotools.feature.Feature;

/**
 * Calls implemented operate method on the each element of selection
 * @see Rename
 * 
 * @author jeichar
 * @since 0.6.0
 */
public abstract class UDIGGenericAction extends WorkbenchWindowActionDelegate {

    private IStructuredSelection selection;


    /**
     * @see org.eclipse.ui.IActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    public void run( IAction action ) {
        for( Iterator iter = selection.iterator(); iter.hasNext(); ) {
            Object element = iter.next();
            
        if( element instanceof Project ){
            operate((Project)element);
        }
        if( element instanceof IProjectElement ){
            operate((ProjectElement)element);
        }        
        if( element instanceof Layer ){
            operate((Layer)element);
        }
        if( element instanceof Feature ){
            operate((Feature)element);
        }

        }
    }
    

    /**
     * Operates on a Feature.  Default Implementation does nothing.
     * 
     * @param feature
     */
    protected void operate( Feature feature ) {
    	//do nothing
	}

    /**
     * Operates on a layer. Default Implementation does nothing.
     * 
     * @param layer
     */
    protected void operate( Layer layer ) {
    	//do nothing
	}

    /**
     * Operates on a IProjectElement. Default Implementation does nothing.
     * 
     * @param element
     */
    protected void operate( ProjectElement element ) {
    	//do nothing
	}


    /**
     * Operates on a Project. Default Implementation does nothing.
     * 
     * @param project
     */
    protected void operate( Project project ) {
    	//do nothing
	}


    /**
     * @see org.eclipse.emf.common.ui.action.WorkbenchWindowActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged( IAction action, ISelection selection ) {
        if( selection instanceof IStructuredSelection)
            this.selection=(IStructuredSelection)selection;
    }

}
