/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import java.util.Iterator;

import net.refractions.udig.project.IMap;
import net.refractions.udig.project.command.BasicCommandFactory;
import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.provider.CRSChooser;
import net.refractions.udig.project.internal.render.ViewportModel;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.dialogs.PropertyPage;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

/**
 * A user interface that allow a user to modify/create and set the current and default
 * CoordinateReferenceSystems used for maps.
 * <p>
 * Responsibilities:
 * <ul>
 * <li>Allow creation of user defined CRSs</li>
 * <li>Allow modification of the current CRS</li>
 * <li>Allow the CRS of a map to be set</li>
 * <li>Allow the default CRS of a workbench to be set</li>
 * </ul>
 * </p>
 * 
 * @author jeichar
 * @since 0.3
 */
public class CRSPropertyPage extends PropertyPage {

    private IWorkbench workbench = null;

    CRSChooser chooser = new CRSChooser(this);

    private IMap map;

    public CRSPropertyPage( IMap map ) {
        setTitle(Policy.bind("CRSPropertyPage.coordinateSystems.title")); //$NON-NLS-1$
        this.map = map;
    }

    /**
     * @see org.eclipse.jface.preference.IPreferencePage#performOk()
     */
    public boolean performOk() {
        CoordinateReferenceSystem crs = chooser.getCRS();
        if (crs == null)
            return false;

        BasicCommandFactory factory = BasicCommandFactory.getInstance();
        MapCommand command = factory.createChangeCRS(map, crs);
        map.sendCommandSync(command);

        return super.performOk();
    }

    /**
     * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
     */
    protected void performDefaults() {
        chooser.clearSearch();
        chooser.gotoCRS(ViewportModel.DEFAULT_CRS);
        //        
        // PropertyDialog dialog = (PropertyDialog) getContainer();
        //
        // Iterator iter = ((StructuredSelection) dialog.getSelection()).iterator();
        // while( iter.hasNext() ) {
        // Object item = iter.next();
        // if (item instanceof Map) {
        // Map map = (Map) item;
        // BasicCommandFactory factory = BasicCommandFactory.getInstance();
        // MapCommand command = factory.createChangeCRS(map, ViewportModel.DEFAULT_CRS);
        // map.sendCommandSync(command);
        //                
        // //map.getViewportModelInternal().setCRS(ViewportModel.DEFAULT_CRS);
        // }
        // }
    }

    /**
     * @see org.eclipse.jface.preference.PreferencePage#getPreferenceStore()
     */
    public IPreferenceStore getPreferenceStore() {
        if (workbench != null) {
            return workbench.getPreferenceStore();
        }

        return ProjectUIPlugin.getDefault().getPreferenceStore();
    }

    /**
     * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
     * @param parent
     */
    protected Control createContents( Composite parent ) {
        CoordinateReferenceSystem crs = null;
        Control control;

        crs = map.getViewportModel().getCRS();
        if (crs == null)
            control = chooser.createControl(parent);
        else
            control = chooser.createControl(parent, crs);
        return control;
    }

    /**
     * @see org.eclipse.jface.preference.PreferencePage#doComputeSize()
     */
    protected Point doComputeSize() {
        return new Point(200, 300);
    }

    /**
     * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
     */
    public void init( IWorkbench workbench ) {
        this.workbench = workbench;
    }

}
