/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.ui.internal;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.internal.ui.UiPlugin;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.IProjectElement;
import net.refractions.udig.project.internal.ContextModel;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.ProjectFactory;
import net.refractions.udig.project.internal.ProjectPackage;
import net.refractions.udig.project.internal.ProjectPlugin;
import net.refractions.udig.project.ui.AdapterFactoryLabelProviderDecorator;
import net.refractions.udig.project.ui.internal.tool.display.ToolManager;
import net.refractions.udig.ui.UDIGDragDropUtilities;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;

/**
 * The Layers View.
 * 
 * @author jeichar
 * @since 0.6.0
 */
public class LayersView extends ViewPart {
	
	public static final String ID="net.refractions.udig.project.ui.layerManager";  //$NON-NLS-1$
	ContextModel currentContextModel = null;

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		disposeInternal();
		super.dispose();
	}

	protected void disposeInternal() {
	    if( PlatformUI.getWorkbench().isClosing()){
	        ProjectPlugin.getPlugin().turnOffEvents();
        }
        contentProvider.dispose();
        viewer.getLabelProvider().dispose();
        
		getSite().getWorkbenchWindow().getPartService().removePartListener(
				partServiceListener);

        
		setCurrentMap(null);
	}

	/**
	 * @return Returns the currentContextModel.
	 */
	public ContextModel getCurrentContextModel() {
		return currentContextModel;
	}

	/**
	 * Focus view on this context model, use <code>null</code> to "turn off
	 * this view.
	 * <p>
	 * XXX: Note the current newContextModel null during application shutdown.
	 * </p>
	 * 
	 * @param currentContextModel
	 *            The currentContextModel, or <code>null </code> if we don't
	 *            have one
	 */
	public void setCurrentContextModel(ContextModel newContextModel) {
		if (currentContextModel != null) {
			currentContextModel.getDeepAdapters().remove(
					checkboxContextListener);
		}
		if (newContextModel != null) {
			newContextModel.getDeepAdapters().add(checkboxContextListener);
		}
		this.currentContextModel = newContextModel;
	}

	/**
	 * @return Returns the currentMap.
	 */
	public Map getCurrentMap() {
		return currentMap;
	}

	/**
	 * @param currentMap
	 *            The currentMap to set.
	 */
	public void setCurrentMap(Map currentMap) {
		this.currentMap = currentMap;
		if (currentMap != null) {
			setCurrentContextModel(currentMap.getContextModel());
		} else {
			setCurrentContextModel(null);
		}
	}

	private class LayerSorter extends ViewerSorter {
		/**
		 * @see org.eclipse.jface.viewers.ViewerSorter#compare(org.eclipse.jface.viewers.Viewer,
		 *      java.lang.Object, java.lang.Object)
		 */
		public int compare(Viewer viewer, Object e1, Object e2) {
			return ((Layer) e1).compareTo((Layer) e2);
		}

	}

	private class EditorListener implements IPartListener {
		private IWorkbenchPart currentPart;

		/**
		 * @see org.eclipse.ui.IPartListener#partActivated(org.eclipse.ui.IWorkbenchPart)
		 */
		public void partActivated(IWorkbenchPart part) {
			if (part == currentPart)
				return;
			if (part instanceof MapEditor) {
				currentPart = part;
				setCurrentMap(((MapEditor) part).getMap());
				setInput(getCurrentContextModel());
				List<Layer> layers = getCurrentMap().getLayersInternal();
				if (layers.size() > 0)
					viewer.setSelection(new StructuredSelection(layers
							.get(layers.size() - 1)));
			}
			// TODO Page Editor action
		}

		// private LayerApplicabilityMenuCreator applicabilityCreator;
		// private LayerApplicabilityMenuCreator getApplicabilityMenu() {
		// if (applicabilityCreator == null) {
		// applicabilityCreator = new LayerApplicabilityMenuCreator();
		// }
		//
		// return applicabilityCreator;
		// }
		// private void addLayersMenu(EditorPart editor) {
		// IMenuManager
		// manager=editor.getEditorSite().getActionBars().getMenuManager();
		// IMenuManager layerMenu=manager.findMenuUsingPath("layer");
		// //$NON-NLS-1$
		// layerMenu.add(getApplicabilityMenu().getMenuManager());
		// editor.getEditorSite().getActionBars().updateActionBars();
		// }

		/**
		 * @see org.eclipse.ui.IPartListener#partBroughtToTop(org.eclipse.ui.IWorkbenchPart)
		 */
		public void partBroughtToTop(IWorkbenchPart part) {
			// TODO Auto-generated method stub
		}

		ContextModel placeholder = ProjectFactory.eINSTANCE
				.createContextModel();

		/**
		 * @see org.eclipse.ui.IPartListener#partClosed(org.eclipse.ui.IWorkbenchPart)
		 */
		public void partClosed(IWorkbenchPart part) {
			if (part == this) {
				disposeInternal();
				return;
			}
			if (part != currentPart)
				return;
			currentPart = null;
			if (part instanceof MapEditor) {
				setCurrentContextModel(null);
				setInput(placeholder);
				if (part.getSite().getPage().getEditorReferences().length == 0)
					removeApplicabilityMenu((EditorPart) part);
				viewer.refresh(true);
			}
		}

		private void removeApplicabilityMenu(EditorPart part) {

//			IMenuManager manager = part.getEditorSite().getActionBars()
//					.getMenuManager();
			// manager.findMenuUsingPath("layer/,)
		}

		/**
		 * @see org.eclipse.ui.IPartListener#partDeactivated(org.eclipse.ui.IWorkbenchPart)
		 */
		public void partDeactivated(IWorkbenchPart part) {
			// nothing
		}

		/**
		 * @see org.eclipse.ui.IPartListener#partOpened(org.eclipse.ui.IWorkbenchPart)
		 */
		public void partOpened(IWorkbenchPart part) {
			// partActivated(part);
		}
	}

	private abstract class LayerAction extends Action implements
			ISelectionListener {

		protected IStructuredSelection selection;

		/**
		 * Construct <code>LayerAction</code>.
		 */
		public LayerAction() {
			getSite().getPage().addSelectionListener(this);
		}

		/**
		 * @see org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse.ui.IWorkbenchPart,
		 *      org.eclipse.jface.viewers.ISelection)
		 */
		public void selectionChanged(IWorkbenchPart part, ISelection selection) {
			if (!(selection instanceof IStructuredSelection))
				return;
			this.selection = (IStructuredSelection) selection;
			if (part instanceof LayersView && selection != null
					&& !selection.isEmpty())
				setEnabled(true);
		}

		/**
		 * @see org.eclipse.jface.action.Action#setEnabled(boolean)
		 */
		public void setEnabled(boolean enabled) {
			super.setEnabled(false);

			if (!enabled || selection == null || selection.isEmpty())
				return;

			for (Iterator iter = selection.iterator(); iter.hasNext();) {
				Object obj = iter.next();
				if (!(obj instanceof Layer))
					return;
			}
			super.setEnabled(true);
		}

	}

	static CheckboxTreeViewer viewer;

	net.refractions.udig.project.ui.internal.LayersView.LayerAction downAction;

	Map currentMap;

	net.refractions.udig.project.ui.internal.LayersView.LayerAction upAction;

	private EditorListener partServiceListener = new EditorListener();

	void updateCheckboxes() {
		if (currentContextModel == null)
			return;
		final List<Layer> checkedLayers = new ArrayList<Layer>();
		for (Layer layer : currentContextModel.getLayers())
			if (layer.isVisible())
				checkedLayers.add(layer);
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				if( PlatformUI.getWorkbench().isClosing() )
					return;
				viewer.refresh(true);
				viewer.setCheckedElements(checkedLayers.toArray());
			}
		});
	}

	void updateCheckbox(final Layer layer) {
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				if( PlatformUI.getWorkbench().isClosing() )
					return;
				viewer.refresh(true);
				viewer.setChecked(layer, layer.isVisible());
			}
		});
	}

	void setInput(ContextModel currentContextModel) {
		viewer.setInput(currentContextModel);
		updateCheckboxes();
	}

	Adapter checkboxContextListener = new AdapterImpl() {
		public void notifyChanged(Notification msg) {
			if (msg.getNotifier() instanceof ContextModel) {
				if (msg.getFeatureID(ContextModel.class) == ProjectPackage.CONTEXT_MODEL__LAYERS) {
					if (msg.getEventType() == Notification.ADD) {
						updateCheckbox((Layer) msg.getNewValue());
					} else if (msg.getEventType() == Notification.ADD_MANY) {
						updateCheckboxes();
					}
					Display.getDefault().asyncExec(new Runnable() {
						public void run() {
							if( PlatformUI.getWorkbench().isClosing() )
								return;

							if (getCurrentMap().getLayersInternal().size() > 0) {
								IStructuredSelection selection = (IStructuredSelection) viewer
										.getSelection();
								if (!getCurrentContextModel().getLayers()
										.contains(selection.getFirstElement())) {
									final List<Layer> layers = getCurrentMap()
											.getLayersInternal();
									viewer
											.setSelection(new StructuredSelection(
													layers
															.get(layers.size() - 1)));
								}
							}
						}
					});
				}
			} else if (msg.getNotifier() instanceof Layer) {
				if (msg.getFeatureID(ContextModel.class) == ProjectPackage.LAYER__VISIBLE)
					if (msg.getNewBooleanValue() != msg.getOldBooleanValue())
						viewer.setChecked(msg.getNotifier(), msg
								.getNewBooleanValue());
			}
		}
	};
    private AdapterFactoryContentProvider contentProvider;
	private static LayersView instance;

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {

		ToolManager.addToolScope(getSite());
		getSite().getWorkbenchWindow().getPartService().addPartListener(
				partServiceListener);

		viewer = new CheckboxTreeViewer(parent, SWT.MULTI);
        contentProvider=new UDIGAdapterFactoryContentProvider(
                ProjectUIPlugin.getDefault().getAdapterFactory());
		viewer.setContentProvider(contentProvider);
		viewer.setLabelProvider(new AdapterFactoryLabelProviderDecorator(
				ProjectExplorer.getProjectExplorer().getAdapterFactory(),
				viewer));

		if (getSite().getPage().getActiveEditor() != null
				&& getSite().getPage().getActiveEditor() instanceof MapEditor) {
			currentMap = ((MapEditor) getSite().getPage().getActiveEditor())
					.getMap();
			viewer.setInput(currentMap.getContextModel());
		}
		viewer.setSorter(new LayerSorter());

		// sets the layer visibility to match the check box setting.
		viewer.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				if (((Layer) event.getElement()).isVisible() != event
						.getChecked())
					((Layer) event.getElement()).setVisible(event.getChecked());
			}
		});
		createContextMenuFor(viewer);
		addMenuActions();
		addToobarActions();

		getViewSite().setSelectionProvider(viewer);
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			public void selectionChanged(SelectionChangedEvent event) {
				if (event.getSelection().isEmpty()) {
					return;
				}

				if (!(event.getSelection() instanceof IStructuredSelection))
					return;
				IStructuredSelection structured = (IStructuredSelection) event
						.getSelection();
				getViewSite().getActionBars().getStatusLineManager()
						.setMessage(
								((ILayer) structured.getFirstElement())
										.getStatusMessage());
			}

		});
		viewer.addDoubleClickListener(new IDoubleClickListener() {

			public void doubleClick(DoubleClickEvent event) {
				final Object obj = ((IStructuredSelection) viewer
						.getSelection()).getFirstElement();
				if (!(obj instanceof IProjectElement)) {
					Display.getDefault().asyncExec(new Runnable() {

						public void run() {
							// TODO
						}
					});
					return;
				}
			}

		});


        UDIGDragDropUtilities.addDragDropSupport(viewer,this);
	}

	/**
	 * This is how the framework determines which interfaces we implement. <!--
	 * begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @param key
	 *            The desired class
	 * @return An object of type key or null;
	 */
	public Object getAdapter(Class key) {
		if (key.equals(IPropertySheetPage.class))
			return ProjectUIPlugin.getDefault().getPropertySheetPage();
		return super.getAdapter(key);
	}

	/**
	 * TODO summary sentence for addToobarActions ...
	 * 
	 */
	private void addToobarActions() {
		IToolBarManager mgr = getViewSite().getActionBars().getToolBarManager();
		mgr.add(upAction());
		mgr.add(downAction());
	}

	/**
	 * Create an action that moves a layer down in the rendering order
	 */
	private LayerAction downAction() {
		downAction = new LayerAction() {
			/**
			 * @see org.eclipse.jface.action.Action#run()
			 */
			public void run() {
				// System.out.println("down");
				for (Iterator iter = selection.iterator(); iter.hasNext();) {
					Layer layer = (Layer) iter.next();
					currentMap.getContextModel().lowerLayer(layer);
				}
			}
		};
		downAction.setEnabled(false);
		downAction.setToolTipText(Policy.bind("LayersView.down.tooltip")); //$NON-NLS-1$
		downAction.setImageDescriptor(Images
				.getDescriptor(ImageConstants.DOWN_CO));
		return downAction;
	}

	/**
	 * Create an action that moves a layer up in the rendering order
	 */
	private LayerAction upAction() {
		upAction = new LayerAction() {
			/**
			 * @see org.eclipse.jface.action.Action#run()
			 */
			public void run() {
				for (Iterator iter = selection.iterator(); iter.hasNext();) {
					Layer layer = (Layer) iter.next();
					currentMap.getContextModel().raiseLayer(layer);
				}
			}
		};
		upAction.setEnabled(false);
		upAction.setToolTipText(Policy.bind("LayersView.up.tooltip")); //$NON-NLS-1$
		upAction.setImageDescriptor(Images.getDescriptor(ImageConstants.UP_CO));
		return upAction;
	}

	/**
	 * TODO summary sentence for addMenuActions ...
	 * 
	 */
	private void addMenuActions() {
		// do nothing
	}

	/**
	 * Creates a context menu
	 * 
	 * @param viewer2
	 */
	private void createContextMenuFor(final Viewer viewer2) {
		final MenuManager contextMenu = new MenuManager();

		contextMenu.setRemoveAllWhenShown(true);
		contextMenu.addMenuListener(new IMenuListener() {

			public void menuAboutToShow(IMenuManager mgr) {
				contextMenu.add(new GroupMarker(
						IWorkbenchActionConstants.MB_ADDITIONS));
				LayerApplicabilityMenuCreator creator = getApplicabilityMenu();
				if (creator != null)
					contextMenu.add(creator.getMenuManager());
				contextMenu.add(UiPlugin.getDefault().getOperationMenuFactory()
						.getContextMenu());
			}

			private LayerApplicabilityMenuCreator applicabilityCreator;

			private LayerApplicabilityMenuCreator getApplicabilityMenu() {
				if (applicabilityCreator == null) {
					applicabilityCreator = new LayerApplicabilityMenuCreator();
				}

				IStructuredSelection selection = (IStructuredSelection) viewer2
						.getSelection();
				for (Iterator iter = selection.iterator(); iter.hasNext();) {
					Object element = iter.next();
					if (!(element instanceof Layer))
						return null;
				}

				return applicabilityCreator;
			}

		});

		// Create menu.
		Menu menu = contextMenu.createContextMenu(viewer2.getControl());
		viewer2.getControl().setMenu(menu);

		// Register menu for extension.
		getSite().registerContextMenu(contextMenu, viewer2);

	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	static Viewer getViewer(){
		if( viewer!=null )
			return viewer;
		return null;
	}

	@Override
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
		instance=this;
	}
	
	public static Object getViewPart() {
		return instance;
	}

	
}
