/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.core.internal.CorePlugin;
import net.refractions.udig.project.internal.ContextModel;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.ProjectFactory;
import net.refractions.udig.ui.ExceptionDisplayer;

import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;

/**
 * summary sentence.
 * <p>
 * Paragraph ...
 * </p>
 * <p>
 * Responsibilities:
 * <ul>
 * <li>
 * <li>
 * </ul>
 * </p>
 * <p>
 * Example:
 * 
 * <pre><code>
 * 
 *  
 *   
 *    LayersViewDropAdapter x = new LayersViewDropAdapter( ... );
 *    TODO code example
 *    
 *   
 *  
 * </code></pre>
 * 
 * </p>
 * 
 * @author jones
 * @since 0.6.0
 */
public class LayersViewDropAdapter extends ViewerDropAdapter {

    /**
     * Construct <code>LayersViewDropAdapter</code>.
     * 
     * @param viewer
     */
    public LayersViewDropAdapter( Viewer viewer ) {
        super(viewer);
    }

    private boolean drop( Object data ) {
        if (data instanceof Layer) {
            return drop((Layer) data);
        }
        if (data instanceof IGeoResource) {
            return drop((IGeoResource) data);
        }
        if (data instanceof IService) {
            return drop((IService) data);
        }
        return false;
    }

    /**
     * Provides the logic for reordering of layers.
     * 
     * It observes the following rules:
     * <ul>
     * <li>If a layer is dragged onto another, it will placed one element higher than the target.</li>
     * <li>If a layer is dragged between two layers, it will be placed there.</li>
     * <li>If a layer is dragged onto itself, or between itself and another, it shall not move.</li>
     * <li>If a layer is dragged into an unreadable area (such as the blank area below the list), it shall be put in the lowest element.</li>
     * </ul>
     * 
     * @param data
     * @return true if the drop succeeds, false otherwise
     */
    private boolean drop( Layer data ) {
        
        
        if (data.getContextModel() != getContextModel()) {
            //This layer is coming from an outside source. Need to clone it!
            
            data = (Layer) EcoreUtil.copy(data);
            getContextModel().getLayers().add(data);
        }
        
        if (data.equals(getCurrentTarget())) {
            return false;
        }
        
        if (getCurrentLocation() == LOCATION_NONE) {
            data.setZorder(0);
            return true;
        }
        
        Layer targetLayer = (Layer) getCurrentTarget();
        
        //Moving something AFTER a layer is the same as moving something BEFORE a layer.
        //So we will use BEFORE as much as possible to prevent duplication here.
        //This code will retrieve the layer before. Or the first one, if we are at the 
        //beginning of the list.
        if (getCurrentLocation() == LOCATION_AFTER) {
            int i = ((Layer) getCurrentTarget()).getZorder()-1;
            
            //In this case, they are dropping it AFTER the lowest (0) element
            if (i < 0) {
                targetLayer = targetLayer.getContextModel().getLayers().get(0);
                data.setZorder(targetLayer.getZorder());
                return true;
            }
            targetLayer = targetLayer.getContextModel().getLayers().get(i);
            
            if (data.equals(targetLayer)) {
                return false;
            }
        }
        if (data.getZorder() < targetLayer.getZorder()) {
            data.setZorder(targetLayer.getZorder());            
        } else {
            data.setZorder(targetLayer.getZorder()+1);
        }

        return true;

    }
    private boolean drop( IGeoResource data ) {
        try {
            Layer layer = ProjectFactory.eINSTANCE.createLayerFactory().createLayer(data);
            getContextModel().getLayers().add(layer);
            drop(layer);
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    private boolean drop( IService data ) {
        try {
            List<Layer> layers = ProjectFactory.eINSTANCE.createLayerFactory().getLayers(data); 
            
            getContextModel().getLayers().addAll(layers);
            
            for (Layer layer : layers) {
                drop(layer);
            }
            
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    /**
     * @see org.eclipse.jface.viewers.ViewerDropAdapter#performDrop(java.lang.Object)
     */
    public boolean performDrop( Object dragData ) {
                
        if (dragData instanceof IStructuredSelection) {
            IStructuredSelection selection = (IStructuredSelection) dragData;
            for( Iterator iter = selection.iterator(); iter.hasNext(); ) {
                Object obj = iter.next();
                drop(obj);
            }
            return true;
        }
        
        if (dragData instanceof String || dragData instanceof String[]) {

            List<Object> urls = new ArrayList<Object>();
            
            if (dragData instanceof String) {
                urls.addAll(CorePlugin.stringsToURLs((String) dragData));
            } else {
                urls.addAll(CorePlugin.stringsToURLs((String[]) dragData));
            }

            extractAndProcessSLDs(urls);
            
            List<Layer> layers = MapFactory.instance().processResources(null, urls);
            if (getContextModel() == null) {
                MapFactory.instance().getMap(null, null, true).getContextModel().getLayers().addAll(layers);
                
                return true;
            }
            
            for ( Layer layer : layers ) {
                getContextModel().getLayers().add(layer);
                drop(layer);
            }
            return true;
        }
        
        return drop(dragData);

    }
    
    private void extractAndProcessSLDs( List<Object> urls ) {
        List<Throwable> errors = new ArrayList<Throwable>();
        
        Iterator iter = urls.iterator(); 
        while (iter.hasNext()) {
            URL url = (URL) iter.next();
            
            if (url.toExternalForm().toLowerCase().endsWith(".sld")) { //$NON-NLS-1$
                iter.remove();
                Object target = getCurrentTarget();
                if (target instanceof Layer) {
                    try {
                        handleSLDURL(url, (Layer) target);
                    } catch (IOException e) {
                        errors.add(e);
                    }
                }
            }
        }
        
        if (errors.size() > 0) {
            String message = null;
            if (errors.size() > 1) {
                message = "Multiple errors occured while parsing SLD files";
            } else {
                message = "An error has occured while parsing the SLD file";
            }
            
            ExceptionDisplayer.displayExceptions(errors, message, ProjectUIPlugin.ID);
        }
    }

    private void handleSLDURL(URL url, Layer target) throws IOException {

        if (target == null) {
            return;
        }
        
        target.getStyleBlackboard().put(url, null);
    }

    /**
     * @see org.eclipse.jface.viewers.ViewerDropAdapter#validateDrop(java.lang.Object, int,
     *      org.eclipse.swt.dnd.TransferData)
     */
    public boolean validateDrop( Object target, int operation, TransferData transferType ) {
        return true;
    }

    /**
     * @see org.eclipse.jface.viewers.ViewerDropAdapter#dragEnter(org.eclipse.swt.dnd.DropTargetEvent)
     */
    public void dragEnter( DropTargetEvent event ) {
        event.detail = DND.DROP_COPY;
        super.dragEnter(event);
    }
    
    private ContextModel getContextModel() {
        return (ContextModel) getViewer().getInput();
    }

}
