/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.internal.ui.UDIGDropTargetProcessor;
import net.refractions.udig.internal.ui.UDIGEditorDropListener;
import net.refractions.udig.internal.ui.UDigByteAndLocalTransfer;
import net.refractions.udig.internal.ui.UiPlugin;
import net.refractions.udig.project.internal.EditManager;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.Project;
import net.refractions.udig.project.internal.ProjectPackage;
import net.refractions.udig.project.internal.ProjectPlugin;
import net.refractions.udig.project.internal.render.RenderManager;
import net.refractions.udig.project.ui.UDIGEditorInput;
import net.refractions.udig.project.ui.commands.IDrawCommand;
import net.refractions.udig.project.ui.internal.adapters.UDIGAdapterSelectionProvider;
import net.refractions.udig.project.ui.internal.commands.draw.DrawFeatureCommand;
import net.refractions.udig.project.ui.internal.render.displayAdapter.impl.ViewportPaneJava;
import net.refractions.udig.project.ui.internal.render.displayAdapter.impl.ViewportPaneSWT;
import net.refractions.udig.project.ui.internal.tool.display.ToolManager;
import net.refractions.udig.project.ui.render.displayAdapter.ViewportPane;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IContributionManager;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.StatusLineLayoutData;
import org.eclipse.jface.action.StatusLineManager;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.IPreferenceNode;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceDialog;
import org.eclipse.jface.preference.PreferenceManager;
import org.eclipse.jface.preference.PreferenceNode;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.HTMLTransfer;
import org.eclipse.swt.dnd.RTFTransfer;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.SubActionBars2;
import org.eclipse.ui.dialogs.PropertyPage;
import org.eclipse.ui.part.EditorPart;
import org.geotools.data.Transaction;
import org.geotools.feature.Feature;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

/**
 * This class is the Eclipse editor Part in which a ViewportPane is embedded. The ViewportPane
 * displays and edits Maps. MapViewport is used to intialize ViewportPane and the RenderManager.
 * 
 * @author Jesse Eichar
 * @version $Revision: 1.9 $
 */
public class MapEditor extends EditorPart {
    /** The id of the MapViewport View */
    public final static String ID = "net.refractions.udig.project.ui.mapEditor"; //$NON-NLS-1$

    RenderManager renderManager = null;

    final MapEditor editor = this;

    private boolean dirty = false;

    Map map;

    ISelectionProvider provider;

    Adapter mapListener = new AdapterImpl(){
        /**
         * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
         */
        public void notifyChanged( Notification msg ) {
            if (msg.getNotifier() instanceof Map)
                mapChanged(msg);
            else if (msg.getNotifier() instanceof EditManager) {
                editManagerChanged(msg);
            }

        }

        private void editManagerChanged( Notification msg ) {
            if (msg.getFeatureID(EditManager.class) == ProjectPackage.EDIT_MANAGER__TRANSACTION_TYPE) {
                if (msg.getNewValue() == Transaction.AUTO_COMMIT.getClass())
                    setDirty(false);
                else
                    setDirty(true);
            }

        }

        @SuppressWarnings({"unchecked", "synthetic-access"})//$NON-NLS-1$ //$NON-NLS-2$ 
        private void mapChanged( Notification msg ) {
            switch( msg.getFeatureID(Map.class) ) {
            case ProjectPackage.MAP__EDIT_MANAGER_INTERNAL:
                if (msg.getOldValue() != null) {
                    ((EditManager) msg.getOldValue()).eAdapters().remove(this);
                }
                if (msg.getNewValue() != null) {
                    ((EditManager) msg.getNewValue()).eAdapters().add(this);
                }
                break;
            case ProjectPackage.MAP__NAME:
                setPartName(getMap().getName());
                break;
            }
        }
    };

    private Composite composite;

    Menu menu;

    private ViewportPane viewportPane;

    private DropTarget dropTarget;

    /**
     * Creates a new MapViewport object.
     */
    public MapEditor() {
        super();
        // Make sure the featureEditorProcessor has been started.
        ProjectUIPlugin.getDefault().getFeatureEditProcessor();
    }

    class StatusBarLabel extends ContributionItem {
        static final String CRS_ITEM_ID = "CRS Display"; //$NON-NLS-1$

        static final String BOUNDS_ITEM_ID = "Bounds Display"; //$NON-NLS-1$

        private Button button;
        private String value;

        private String full;

        /**
         * Create new StatusBarLabel object
         */
        public StatusBarLabel( String id, String initialValue ) {
            super(id);
            setText(initialValue);
        }

        /**
         * sets the current text.
         */
        public void setText( String text ) {
            value = text;
            full = value;
            if (value.length() > 14) {
                value = value.substring(0, 6)
                        + "..." + value.substring(value.length() - 6, value.length()); //$NON-NLS-1$
                System.out.println(value.length());
            }
            if (button != null) {
                button.setText(value);
                button.setToolTipText(full);
            }
        }
        /**
         * @see org.eclipse.jface.action.IContributionItem#isDynamic()
         */
        public boolean isDynamic() {
            return true;
        }
        /**
         * @see org.eclipse.jface.action.ContributionItem#dispose()
         */
        public void dispose() {
            button.dispose();
        }
        /**
         * @see org.eclipse.jface.action.ContributionItem#fill(org.eclipse.swt.widgets.Composite)
         */
        @Override
        public void fill( Composite c ) {
            Label separator = new Label(c, SWT.SEPARATOR);
            StatusLineLayoutData data = new StatusLineLayoutData();
            separator.setLayoutData(data);
            data.widthHint = 1;
            data.heightHint = 15;
            button = new Button(c, SWT.PUSH);
            data = new StatusLineLayoutData();
            button.setLayoutData(data);
            button.setText(value);
            button.setToolTipText(full);
            button.addListener(SWT.Selection, new Listener(){

                public void handleEvent( Event event ) {
                    IEditorPart current = getSite().getPage().getActiveEditor();
                    assert current instanceof MapEditor;
                    PropertyPage page = new CRSPropertyPage(((MapEditor) current).getMap());
                    PreferenceManager mgr = new PreferenceManager();
                    IPreferenceNode node = new PreferenceNode("1", page); //$NON-NLS-1$
                    mgr.addToRoot(node);

                    PreferenceDialog dialog = new PreferenceDialog(getSite().getShell(), mgr);
                    dialog.create();
                    dialog.open();
                    updateCRS();
                }

            });
            data.widthHint = 100;
            data.heightHint = 15;
        }

        /**
         * Get current text.
         */
        public String getText() {
            return full;
        }
    }

    /**
     * Updates the crs label in the statusbar.
     */
    protected void updateCRS() {
        CoordinateReferenceSystem crs = getMap().getViewportModel().getCRS();
        final String full = crs.getName().getCode().toString();
        if (isSame(full))
            return;

        Display display = PlatformUI.getWorkbench().getDisplay();
        if (display == null)
            display = Display.getDefault();

        display.asyncExec(new Runnable(){
            public void run() {

                IContributionManager bar = getActionbar().getStatusLineManager();
                if (bar == null)
                    return;
                StatusBarLabel label = (StatusBarLabel) bar.find(StatusBarLabel.CRS_ITEM_ID);
                if (label == null) {
                    label = new StatusBarLabel(StatusBarLabel.CRS_ITEM_ID, full);
                    bar.appendToGroup(StatusLineManager.MIDDLE_GROUP, label);
                    label.setVisible(true);
                    bar.update(true);
                    return;
                }
                label.setText(full);
            }
        });

    }

    private boolean isSame( String crs ) {
        IContributionManager bar = getActionbar().getStatusLineManager();

        if (bar != null) {
            StatusBarLabel label = (StatusBarLabel) bar.find(StatusBarLabel.CRS_ITEM_ID);
            if (label != null && crs.equals(label.getText()))
                return true;
        }
        return false;
    }

    /**
     * @see org.eclipse.ui.IWorkbenchPart#dispose()
     */
    public void dispose() {
        if (PlatformUI.getWorkbench().isClosing()) {
            ProjectPlugin.getPlugin().turnOffEvents();
            
            //save the map's URI in the preferences so that it will be loaded the next time udig is run.

            Resource resource = getMap().eResource();
            if (resource != null) {
                try {
                    IPreferenceStore p = ProjectUIPlugin.getDefault().getPreferenceStore();
                    int numEditors=p.getInt(ID);
                    String id=ID+":"+numEditors; //$NON-NLS-1$
                    numEditors++;
                    p.setValue(ID, numEditors);
                    String value=resource.getURI().toString();
                    p.setValue(id, value);
                } catch (Exception e) {
                    ProjectUIPlugin.log("Failure saving which maps are open", e); //$NON-NLS-1$
                }
            }
        }

        renderManager.dispose();
        map.getEditManagerInternal().setEditFeature(null, null);
        try {
            getSite().getWorkbenchWindow().run(false, false, new IRunnableWithProgress(){

                public void run( IProgressMonitor monitor ) throws InvocationTargetException,
                        InterruptedException {
                    Project p = map.getProjectInternal();
                    try {
                        if (p != null) {
                            if (p.eResource().isModified())
                                p.eResource().save(ProjectPlugin.getPlugin().saveOptions);
                            saveAndUnload();
                        } else {
                            saveAndUnload();
                        }
                    } catch (IOException e) {
                        ProjectPlugin.log(Policy.bind("MapEditor.saveError"), e); //$NON-NLS-1$
                    }

                    // need to kick the Project so viewers will update
                    p.eNotify(new ENotificationImpl((InternalEObject) p, Notification.SET,
                            ProjectPackage.PROJECT__ELEMENTS_INTERNAL, null, null));
                }

                private void saveAndUnload() throws IOException {
                    final Resource resource = map.eResource();
                    resource.save(ProjectPlugin.getPlugin().saveOptions);
                    resource.unload();
                }
            });
        } catch (Exception e) {
            ProjectPlugin.log(Policy.bind("MapEditor.saveError"), e); //$NON-NLS-1$
        }
        super.dispose();

    }

    /**
     * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
     */
    public void doSave( IProgressMonitor monitor ) {
        Platform.run(new SafeRunnable(){

            public void run() throws Exception {
                map.getEditManagerInternal().commitTransaction();
            }
        });
        setDirty(false);
    }

    /**
     * @see org.eclipse.ui.ISaveablePart#doSaveAs()
     */
    public void doSaveAs() {
        // TODO
    }

    /**
     * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
     */
    public void init( IEditorSite site, IEditorInput input ) {
        setSite(site);
        setInput(input);
        // initialize ToolManager
        ProjectUIPlugin.getDefault().getToolManager();
    }

    /**
     * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
     */
    protected void setInput( IEditorInput input ) {
        if (getEditorInput() != null) {
            Map map = (Map) ((UDIGEditorInput) getEditorInput()).getProjectElement();
            // removeListener(map);
            map.eAdapters().remove(mapListener);
        }
        if (input != null) {
            map = (Map) ((UDIGEditorInput) input).getProjectElement();
            addListener(map);
        }

        super.setInput(input);
    }

    @SuppressWarnings("unchecked")//$NON-NLS-1$
    private void addListener( Map map2 ) {
        map.eAdapters().add(mapListener);
    }

    /**
     * @see org.eclipse.ui.ISaveablePart#isDirty()
     */
    public boolean isDirty() {
        return dirty;
    }

    /**
     * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
     */
    public boolean isSaveAsAllowed() {
        return true;
    }

    void setDirty( boolean dirty ) {
        if (dirty == this.dirty)
            return;

        this.dirty = dirty;

        Display.getDefault().asyncExec(new Runnable(){
            /**
             * @see java.lang.Runnable#run()
             */
            @SuppressWarnings("synthetic-access")//$NON-NLS-1$
            public void run() {
                firePropertyChange(PROP_DIRTY);
            }
        });
    }

    /**
     * @see org.eclipse.ui.part.EditorPart#isSaveOnCloseNeeded()
     */
    public boolean isSaveOnCloseNeeded() {
        return true;
    }

    /**
     * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
     */
    public void createPartControl( final Composite parent ) {

        this.composite = parent;
        setPartName(map.getName());

        ToolManager.addToolScope(getSite());

        setTitleToolTip(Policy.bind("MapEditor.titleToolTip")); //$NON-NLS-1$
        setTitleImage(Images.get(ISharedImages.MAP_OBJ));

        if (System.getProperty("java.awt.headless") != null && //$NON-NLS-1$
                System.getProperty("java.awt.headless").equals("true")) //$NON-NLS-1$ //$NON-NLS-2$
            viewportPane = new ViewportPaneSWT(parent, this);
        else
            viewportPane = new ViewportPaneJava(parent, this);

        initMap();
        createContextMenu();
        ProjectUIPlugin.getDefault().getToolManager().setCurrentEditor(MapEditor.this);
        getSite().getPage().addPartListener(partlistener);
        registerFeatureFlasher();

        dropTarget = new DropTarget(this.composite, DND.DROP_COPY | DND.DROP_LINK | DND.DROP_MOVE);
        List<Transfer> transfers = UDIGDropTargetProcessor.getTransfers();
        transfers.add(UDigByteAndLocalTransfer.getInstance());
        transfers.add(TextTransfer.getInstance());
        transfers.add(FileTransfer.getInstance());
        transfers.add(HTMLTransfer.getInstance());
        transfers.add(RTFTransfer.getInstance());
        dropTarget.setTransfer(transfers.toArray(new Transfer[transfers.size()]));
        dropTarget.addDropListener(new UDIGEditorDropListener());
    }

    /**
     * registers a listener with the current page that flashes a feature each time the current
     * selected feature changes.
     */
    protected void registerFeatureFlasher() {
        getSite().getPage().addPostSelectionListener(new ISelectionListener(){
            List<IDrawCommand> commands;

            public void selectionChanged( IWorkbenchPart part, ISelection selection ) {
                List<Feature> features = new ArrayList<Feature>();
                StructuredSelection s = (StructuredSelection) selection;

                for( Iterator iter = s.iterator(); iter.hasNext(); ) {
                    @SuppressWarnings("unchecked") Object element = iter.next(); //$NON-NLS-1$

                    if (element instanceof Feature) {
                        Feature feature = (Feature) element;
                        features.add(feature);
                    }
                }
                if (features.size() == 0)
                    return;
                commands = createDrawCommands(features);
                if (!renderManager.isDisposed())
                    getFlashFeatureJob().schedule();

            }

            private Job getFlashFeatureJob() {
                if (flashFeatureJob == null) {
                    flashFeatureJob = new Job("flashFeatures"){ //$NON-NLS-1$

                        @Override
                        protected IStatus run( IProgressMonitor monitor ) {
                            for( int i = 0; i < 3; i++ )
                                flashFeature(commands);
                            return Status.OK_STATUS;
                        }
                    };
                    flashFeatureJob.setSystem(true);
                }

                return flashFeatureJob;
            }

            Job flashFeatureJob;

            private List<IDrawCommand> createDrawCommands( List<Feature> current ) {
                final List<IDrawCommand> commands = new ArrayList<IDrawCommand>();
                for( Feature feature : current ) {
                    IDrawCommand command = null;
                    if (feature instanceof IAdaptable) {
                        Layer layer = (Layer) ((IAdaptable) feature).getAdapter(Layer.class);
                        if (layer != null)
                            try {
                                command = new DrawFeatureCommand(feature, layer);
                            } catch (IOException e) {
                                // do nothing... thats life
                            }
                    }
                    if (command == null)
                        command = new DrawFeatureCommand(feature);
                    commands.add(command);
                }
                return commands;
            }

            void flashFeature( List<IDrawCommand> commands ) {
                for( IDrawCommand command : commands ) {
                    command.setValid(true);
                    ((ViewportPane) renderManager.getMapDisplay()).addDrawCommand(command);
                }
                ((ViewportPane) renderManager.getMapDisplay()).repaint();
                synchronized (this) {
                    try {
                        wait(100);
                    } catch (InterruptedException e) {
                        // do nothing
                    }
                }
                for( IDrawCommand command : commands ) {
                    command.setValid(false);
                }
                ((ViewportPane) renderManager.getMapDisplay()).repaint();
                synchronized (this) {
                    try {
                        wait(100);
                    } catch (InterruptedException e) {
                        // do nothing
                    }
                }
            }

        });
    }

    @SuppressWarnings("unchecked")//$NON-NLS-1$
    void initMap() {
        if (map.getRenderManager() == null)
            map.setRenderManagerInternal(new RenderManagerDynamic());

        renderManager = map.getRenderManagerInternal();

        viewportPane.setRenderManager(renderManager);
        renderManager.setMapDisplay(viewportPane);
        MapEditorSelectionProvider mapSelectionProvider = new MapEditorSelectionProvider(this);

        getMap().getEditManagerInternal().eAdapters().add(mapSelectionProvider);

        provider = new UDIGAdapterSelectionProvider(mapSelectionProvider);
        map.getRenderManagerInternal().setRenderExecutor(null);
        getSite().getPage().addPostSelectionListener(
                new LayerSelectionListener(new LayerSelectionListener.Callback(){

                    public void callback( List<Layer> layers ) {
                        map.getEditManagerInternal().setSelectedLayer(layers.get(0));
                    }

                }));
    }

    void createContextMenu() {
        final MenuManager contextMenu = new MenuManager();

        contextMenu.setRemoveAllWhenShown(true);
        contextMenu.addMenuListener(new IMenuListener(){
            public void menuAboutToShow( IMenuManager mgr ) {
                ToolManager tm = ProjectUIPlugin.getDefault().getToolManager();
                contextMenu.add(tm.getBACKWARD_HISTORYAction());
                contextMenu.add(tm.getFORWARD_HISTORYAction());
                contextMenu.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
                if (map.getEditManager().getEditFeature() != null) {
                    contextMenu.add(ProjectUIPlugin.getDefault().getFeatureEditProcessor()
                            .getEditFeatureAction(provider.getSelection()));

                    contextMenu.add(ProjectUIPlugin.getDefault().getFeatureEditProcessor()
                            .getEditWithFeatureMenu(provider.getSelection()));
                }
                contextMenu.add(UiPlugin.getDefault().getOperationMenuFactory().getContextMenu());
            }
        });

        // Create menu.
        menu = contextMenu.createContextMenu(composite);

        getSite().registerContextMenu(contextMenu, provider);
    }

    /**
     * @see org.eclipse.ui.IWorkbenchPart#setFocus()
     */
    public void setFocus() {
        composite.setFocus();
        getSite().setSelectionProvider(provider);
        updateCRS();
    }

    /**
     * Returns the map that this editor edits
     * 
     * @return Returns the map that this editor edits
     */
    public Map getMap() {
        return map;
    }

    /**
     * Returns the ActionbarContributor for the Editor.
     * 
     * @return the ActionbarContributor for the Editor.
     */
    public SubActionBars2 getActionbar() {
        return (SubActionBars2) getEditorSite().getActionBars();
    }

    IPartListener2 partlistener = new IPartListener2(){

        public void partActivated( IWorkbenchPartReference partRef ) {
            // do nothing
        }

        public void partBroughtToTop( IWorkbenchPartReference partRef ) {
        }

        public void partClosed( IWorkbenchPartReference partRef ) {
            if (partRef.getPart(false) instanceof MapEditor)
                if (((MapEditor) partRef.getPart(false)).getMap() == getMap())
                    if (registered) {
                        ((ViewportPane) getMap().getRenderManagerInternal().getMapDisplay())
                                .removePaneListener(getMap().getViewportModelInternal());
                        getMap().getRenderManagerInternal().dispose();
                        getMap().getViewportModelInternal().setInitialized(false);
                        registered = false;
                    }
        }

        public void partDeactivated( IWorkbenchPartReference partRef ) {
            // do nothing
        }

        public void partOpened( IWorkbenchPartReference partRef ) {
            // do nothing
        }

        public void partHidden( IWorkbenchPartReference partRef ) {
            if (partRef.getPart(false) instanceof MapEditor)
                if (((MapEditor) partRef.getPart(false)).getMap() == getMap())
                    if (registered) {
                        ((ViewportPane) getMap().getRenderManagerInternal().getMapDisplay())
                                .removePaneListener(getMap().getViewportModelInternal());
                        registered = false;
                    }
        }

        boolean registered = false;

        public void partVisible( IWorkbenchPartReference partRef ) {
            if (partRef.getPart(false) instanceof MapEditor)
                if (((MapEditor) partRef.getPart(false)).getMap() == getMap())
                    if (!registered) {
                        ((ViewportPane) getMap().getRenderManager().getMapDisplay())
                                .addPaneListener(getMap().getViewportModelInternal());
                        registered = true;
                    }
        }

        public void partInputChanged( IWorkbenchPartReference partRef ) {
            // TODO implement method body
        }

    };

    /**
     * Opens the map's context menu.
     */
    public void openContextMenu() {
        getEditorSite().getShell().getDisplay().asyncExec(new Runnable(){
            public void run() {
                menu.setVisible(true);
            }
        });
    }

}