/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;





import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.ui.palette.ColourScheme;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;

/**
 * @author ptozer
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class PaletteCombo {
	Point fExtent = null;
	Composite composite = null;
    Combo colourLetterCombo = null;
    Button colourIndicatorButton = null;
    Color swtColour = null;
    Image image = null;
    List<AWTColor> colours = null;
    Button checkbox = null;
    Layer layerReference = null;
    
	
	/**
	 * Constructor
	 * @param parent
	 */
	public PaletteCombo(Composite parent) {
		composite = new Composite(parent, SWT.NONE);
		fExtent= computeImageSize(composite);
        image = new Image(parent.getDisplay(), fExtent.x, fExtent.y);
	}
		

    /**
     * @param colourToUse
     * @param layerNumber
     * @param colourLetters
     * @return
     */
    public Control getPaletteCombo( ColourScheme mapScheme,  Layer layer, String [] colourLetters) {
        layerReference = layer;
        String layerName = layer.getName();
        ColourScheme layerScheme = layer.getColourScheme();
        
        String currentColourLetter = layer.getDefaultColor();
        
        ColourScheme currentScheme = null;
        
        //HACK
        //if colour scheme for layer is different, then we display its colour:
        //colours to use are from layer scheme
        List<AWTColor> coloursToUse = new ArrayList<AWTColor>();
        AWTColor currentColour = null;
        if(!(layerScheme.equals(mapScheme))) {
            currentScheme = layerScheme;
            for(int i = 0; i < colourLetters.length; i++) {
                coloursToUse.add( new AWTColor( layerScheme.getColour(colourLetters[i]) ) );
            }
         }else {
             currentScheme = mapScheme;
            for(int i = 0; i < colourLetters.length; i++) {
                coloursToUse.add( new AWTColor( mapScheme.getColour(colourLetters[i]) ) );
            }
        }
       // layer.setColourScheme(currentScheme);
        currentColour = new AWTColor(currentScheme.getColour(currentColourLetter));
        
        int currentColourIndex = -1;
        
        //iterate colour array and find current colour index
        Iterator<AWTColor> colourIter = coloursToUse.iterator();
        while(colourIter.hasNext()) {
            AWTColor c = (AWTColor) colourIter.next();
            if(c.equals(currentColour)) {
                currentColourIndex = coloursToUse.indexOf(c);
            }
        }

        colours = coloursToUse;

        layerReference = layer;
        
        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 4;  
        
        composite.setLayout( gridLayout );
        
        Label layerLabel = new Label(composite, SWT.NONE);
        layerLabel.setText( layerName + ":" ); //$NON-NLS-1$ //$NON-NLS-2$
    
        GridData data = new GridData();
        data.horizontalSpan = 1;
        layerLabel.setLayoutData(data);

        colourIndicatorButton = new Button(composite, SWT.FLAT | SWT.TRAIL);
        updateButtonColourDisplay(currentColourIndex);

        data = new GridData();
        colourIndicatorButton.setLayoutData(data);
        
        colourLetterCombo = new Combo(composite,SWT.DROP_DOWN | SWT.READ_ONLY );
        colourLetterCombo.setItems(colourLetters);
        colourLetterCombo.select(currentColourIndex);  
        colourLetterCombo.addSelectionListener(new SelectionListener(){
            public void widgetSelected(SelectionEvent e) {
                int selectIndex = colourLetterCombo.getSelectionIndex();
                updateButtonColourDisplay(selectIndex);
            }

            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub
                
            }
        });
       
        data = new GridData();
        colourLetterCombo.setLayoutData(data);
        
        checkbox =   new Button(composite, SWT.CHECK);
        checkbox.setSelection(true);
        
        data = new GridData();
        checkbox.setLayoutData(data);
        
        //composite.layout(true);
        return composite;
    }


	/**
	 * 
	 */
	public void dispose(){
		//clean up  the colours and images
		if(composite != null ){
            Control [] kids = composite.getChildren();
            for(int i = 0; i < kids.length; i++) {
                Control c = kids[i];
                if(c instanceof Button) {
                    Image img = ((Button)c).getImage();
                    if( img != null ) {
                        img.dispose();
                        ((Button)c).getImage().dispose();
                    }
                }
                c.dispose();
            }            
        }
        if(swtColour != null ) {
            swtColour.dispose();
        }
        
         fExtent = null;
         composite = null;
         colourLetterCombo = null;
         colourIndicatorButton = null;
         swtColour = null;
         image = null;
        colours = null;
         checkbox = null;
         layerReference = null;
	}
	
     /**
     * @param index
     */
    protected void updateButtonColourDisplay(int index) {
         GC gc = new GC(image);
         gc.drawRectangle(0, 2, fExtent.x, fExtent.y);

         if (swtColour != null)
             swtColour.dispose();
         
         AWTColor newColour = colours.get(index);
         swtColour = new Color(composite.getDisplay(), 
                 new RGB(newColour.getColour().getRed(), newColour.getColour().getGreen(), newColour.getColour().getBlue())) ;
         gc.setBackground(swtColour);
         gc.fillRectangle(0, 2, fExtent.x, fExtent.y);
         gc.dispose();

         colourIndicatorButton.setImage(image);
     }
    
	/**
	 * @param window
	 * @return
	 */
	protected Point computeImageSize(Control window) {
		GC gc= new GC(window);
		Font f= JFaceResources.getFontRegistry().get(JFaceResources.DEFAULT_FONT);
		gc.setFont(f);
		int height= gc.getFontMetrics().getHeight();
		gc.dispose();
		Point p= new Point(height * 3 - 6, height);
		return p;
	}

    /**
     * @return Returns the colourLetterCombo.
     */
    public Combo getColourLetterCombo() {
        return colourLetterCombo;
    }

    /**
     * @return Returns the checkbox.
     */
    public Button getCheckbox() {
        return checkbox;
    }
}
