/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import java.io.File;
import java.io.IOException;

import net.refractions.udig.core.enums.TestResults;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

/**
 * A Wizard for Creating a new project
 * 
 * @author jgarnett
 * @since 0.3
 */
public class ProjectNewPage extends WizardPage implements KeyListener {

	/** Text control representing the current file */
	Text text;

	String path;

	/**
	 * Construct <code>ProjectNewPage</code>.
	 */
	public ProjectNewPage() {
		super(
				Policy.bind("ProjectNewPage.newProject"), Policy.bind("ProjectNewPage.newProject"), Images.getDescriptor(ImageConstants.NEWPROJECT_WIZBAN)); //$NON-NLS-1$ //$NON-NLS-2$
		setDescription(Policy.bind("ProjectNewPage.newProject.description")); //$NON-NLS-1$
	}

	/**
	 * Set up this page for use.
	 * 
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 * @param parent
	 */
	public void createControl(Composite parent) {

		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout(3, false));

		Label label = new Label(composite, SWT.NONE);
		label.setText(Policy.bind("ProjectNewPage.label.newProject.text")); //$NON-NLS-1$
		label.setLayoutData(new GridData());
		label.setToolTipText(Policy
				.bind("ProjectNewPage.label.newProject.tooltip")); //$NON-NLS-1$

		text = new Text(composite, SWT.BORDER);
		text.setLayoutData(new GridData(SWT.FILL, SWT.DEFAULT, true, false));

		final IPath homepath = Platform.getLocation();

		path = new File(homepath.toString() + File.separatorChar
				+ Policy.bind("ProjectNewPage.newProject.filename")).getAbsolutePath(); //$NON-NLS-1$
		text.setText(path);
		text.addKeyListener(this);

		Button button = new Button(composite, SWT.PUSH);
		button.setLayoutData(new GridData());

		button.setText(Policy.bind("ProjectNewPage.button.browse.text")); //$NON-NLS-1$
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Display display = Display.getCurrent();
				if (display == null) { // not on the ui thread?
					display = Display.getDefault();
				}
				FileDialog dialog = new FileDialog(display.getActiveShell(),
						SWT.OPEN);

				dialog.setFilterPath(homepath.toOSString());
				dialog.setFilterExtensions(new String[] { "*.udig" }); //$NON-NLS-1$
				path = dialog.open();
				if (path == null)
					return;
				text.setText(path);
			}
		});
		setControl(composite);
		setPageComplete(true);
	}

	/**
	 * Wait for keyReleased before checking path.
	 * 
	 * @see org.eclipse.swt.events.KeyListener#keyPressed(org.eclipse.swt.events.KeyEvent)
	 * @param e
	 */
	public void keyPressed(KeyEvent e) {
		// everything interesting happens at keyReleased
	}

	/**
	 * File provieded by the user, or <code>null</code> if not valid
	 * 
	 * @return File or <code>null</code> if not valid
	 */
	public String getFile() {
		return path;
	}


	/**
	 * True if path is valid
	 * 
	 * @param path
	 * @return true if path is valid
	 */
	public static TestResults test(String path) {
		if (path == null)
			return TestResults.INVALID;
		path = path.trim();

		if ("".equals(path)) //$NON-NLS-1$
			return TestResults.INVALID;

		File file = new File(path);
		if (file.exists()) {
			return TestResults.EXISTS; // file already exists
		}

		File parent = file.getParentFile();
		// System.out.println("parent:" + parent);
		if (false) {
			if (parent == null)
				return TestResults.INVALID; // we are not in a valid directory!

			try {
				parent.getCanonicalFile();
			} catch (IOException e) {
				return TestResults.INVALID; // parent not valid
			}
		}
		String name = file.getName();
		if (name == null)
			return TestResults.INVALID; // cannot check name

		name = name.toLowerCase();
		int split = name.lastIndexOf("."); //$NON-NLS-1$
		if (split != -1) {
			String suffix = name.substring(split + 1);
			// System.out.println("suffix:" + suffix);
			if ("udig".equals(suffix)) { //$NON-NLS-1$
				return TestResults.ACCEPTABLE; // valid udig project extention
			}
			return TestResults.INVALID; // not a valid extention
		}
		return TestResults.ACCEPTABLE;
	}

	/**
	 * Check that the path is valid and update message.
	 * 
	 * @see org.eclipse.swt.events.KeyListener#keyReleased(org.eclipse.swt.events.KeyEvent)
	 * @param event
	 */
	public void keyReleased(KeyEvent event) {
		path = text.getText();
		
		switch (test(path)) {
		case ACCEPTABLE:
			this.setErrorMessage(null);
			this.setMessage(Policy.bind("ProjectNewPage.newProject.description"));  //$NON-NLS-1$
			setPageComplete(true);
			break;
		case INVALID:
			this.setErrorMessage(null);
			this.setMessage(Policy.bind("ProjectNewPage.error.filename")); //$NON-NLS-1$
			setPageComplete(false);
			break;
		case EXISTS:
			this.setErrorMessage(Policy.bind("ProjectNewPage.exists")); //$NON-NLS-1$
			setPageComplete(false);			
			break;

		default:
			break;
		}

	}

}
