/*
 * Created on Nov 24, 2004 TODO To change the template for this generated file go to Window -
 * Preferences - Java - Code Style - Code Templates
 */
package net.refractions.udig.project.ui.internal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.ContextModel;
import net.refractions.udig.project.internal.ContextModelListenerAdapter;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.ProjectPackage;
import net.refractions.udig.project.internal.render.CompositeRenderContext;
import net.refractions.udig.project.internal.render.MultiLayerRenderer;
import net.refractions.udig.project.internal.render.RenderContext;
import net.refractions.udig.project.internal.render.RenderListenerAdapter;
import net.refractions.udig.project.internal.render.RenderManager;
import net.refractions.udig.project.internal.render.RenderPackage;
import net.refractions.udig.project.internal.render.SelectionLayer;
import net.refractions.udig.project.internal.render.ViewportModel;
import net.refractions.udig.project.internal.render.impl.RenderManagerImpl;
import net.refractions.udig.project.render.RenderException;
import net.refractions.udig.project.ui.render.displayAdapter.ViewportPane;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.geotools.data.FeatureEvent;

import com.vividsolutions.jts.geom.Envelope;

/**
 * The main listeners that the RenderManagerDynamic uses to listen to the model.
 * @author jeichar 
 */
public class RenderManagerAdapters {

	/**
	 * Listens to the ViewportModel and refreshes when the CRS or Bounds change
	 * @param manager
	 * @return
	 */
	static Adapter createViewportListener(final RenderManagerImpl manager) {
		return new AdapterImpl() {
			/**
			 * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
			 */
			public void notifyChanged(Notification msg) {
                manager.checkState();
				switch (msg.getFeatureID(ViewportModel.class)) {
				case RenderPackage.VIEWPORT_MODEL__BOUNDS: {
					refreshDirtyArea(msg);
					break;
				}
				case RenderPackage.VIEWPORT_MODEL__CRS: {
					manager.refresh(null);
					break;
				}
				}// switch
			}

			private void refreshDirtyArea(Notification msg) {
				manager.refresh(null);
			}
		};
	}

	/** 
	 * Listens for changes to the layer composition of the map and updates accordingly.
	 * 
	 * @param manager
	 * @return
	 */
	static ContextModelListenerAdapter createContextModelListener(
			final RenderManagerDynamic manager) {
		return new ContextModelListenerAdapter() {
			/**
			 * @see net.refractions.udig.project.ContextModelListenerAdapter#commandExecuted(org.eclipse.emf.common.notify.Notification)
			 */
			public void notifyChanged(Notification msg) {
				super.notifyChanged(msg);
			}

			/**
			 * @see net.refractions.udig.project.ContextModelListenerAdapter#zorderChanged(org.eclipse.emf.common.notify.Notification)
			 */
			protected void zorderChanged(Notification msg) {
				synchronizeRenderers(manager, msg);

				updateImage();
			}

			/**
			 * @see net.refractions.udig.project.ContextModelListenerAdapter#layerAdded(org.eclipse.emf.common.notify.Notification)
			 */
			protected void layerAdded(Notification msg) {
				synchronizeRenderers(manager, msg);
				updateImage();
			}

			/**
			 * @see net.refractions.udig.project.ContextModelListenerAdapter#layerRemoved(org.eclipse.emf.common.notify.Notification)
			 */
			protected void layerRemoved(Notification msg) {
				synchronizeRenderers(manager, msg);
				updateImage();
			}

			/**
			 * @see net.refractions.udig.project.ContextModelListenerAdapter#manyLayersAdded(org.eclipse.emf.common.notify.Notification)
			 */
			protected void manyLayersAdded(Notification msg) {
				synchronizeRenderers(manager, msg);
				updateImage();
			}

			/**
			 * @see net.refractions.udig.project.ContextModelListenerAdapter#manyLayersRemoved(org.eclipse.emf.common.notify.Notification)
			 */
			protected void manyLayersRemoved(Notification msg) {
				synchronizeRenderers(manager, msg);
				updateImage();
			}

			public void updateImage() {
				try {
					((MultiLayerRenderer) manager.getRenderExecutor()
							.getRenderer()).refreshImage();
				} catch (RenderException e) {
					ProjectUIPlugin.log("Error updating image", e); //$NON-NLS-1$
				}
				((ViewportPane) manager.getMapDisplay()).renderDone();
			}

		};

	}

	static void synchronizeRenderers(RenderManagerDynamic manager,
			Notification msg) {
        manager.checkState();
        
        HashMap<RenderContext, RenderContext> oldToCopy = new HashMap<RenderContext, RenderContext>();
		if (manager.configuration != null)
			for (RenderContext context : manager.configuration) {
				oldToCopy.put(context, (RenderContext) context.clone());
			}

		manager.getRendererCreator().changed(msg);
		manager.validateRendererConfiguration();
		List<Layer> toRender = new ArrayList<Layer>();
		for (RenderContext newcontext : manager.configuration) {
			if (!oldToCopy.containsKey(newcontext)
					&& !(newcontext.getLayer() instanceof SelectionLayer)) {
				toRender.add(newcontext.getLayerInternal());
			} else {
				if (newcontext instanceof CompositeRenderContext) {
					List<Layer> oldLayers = ((CompositeRenderContext) oldToCopy
							.get(newcontext)).getLayersInternal();
					for (Layer layer : ((CompositeRenderContext) newcontext)
							.getLayersInternal()) {
						if (!oldLayers.contains(layer)) {
							toRender.add(newcontext.getLayerInternal());
							break;
						}
						oldLayers.remove(layer);
						// XXX
					}
					if (!oldLayers.isEmpty())
						toRender.add(newcontext.getLayerInternal());
				}
				oldToCopy.remove(newcontext);
			}
		}

		for (Layer layer : toRender) {
			manager.refresh(layer, null);
		}
	}

	/**
	 * Listens for events where the Map or the map's ViewportModel is changed(The reference is changed)
	 */
	static Adapter createViewportModelChangeListener(
			final RenderManagerImpl manager, final Adapter viewportListener,
			final ContextModelListenerAdapter contextModelListener) {
		return new AdapterImpl() {
			/**
			 * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
			 */
			@SuppressWarnings("unchecked")//$NON-NLS-1$
			public void notifyChanged(Notification msg) {
                manager.checkState();
                
                switch (msg.getFeatureID(RenderManager.class)) {
				case RenderPackage.RENDER_MANAGER__VIEWPORT_MODEL_INTERNAL: {
					if (msg.getOldValue() != null)
						((ViewportModel) msg.getOldValue()).eAdapters().remove(
								viewportListener);
					if (msg.getNewValue() != null)
						((ViewportModel) msg.getNewValue()).eAdapters().add(
								viewportListener);
					break;
				}// case
				case RenderPackage.RENDER_MANAGER__MAP_INTERNAL: {
					if (msg.getOldValue() != null)
						((Map) msg.getOldValue()).getContextModel().eAdapters()
								.remove(contextModelListener);
					if (msg.getNewValue() != null)
						((Map) msg.getNewValue()).getContextModel().eAdapters()
								.add(contextModelListener);
					break;
				}// case

				}
			}
		};
	}

	/**
	 * Listens to layer events such as style changes
	 * @param manager
	 * @return
	 */
	static Adapter createLayerListener(final RenderManagerDynamic manager) {
		return new AdapterImpl() {
			/**
			 * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
			 */
			public void notifyChanged(Notification event) {
                manager.checkState();
                
                if (event.getNotifier() instanceof Map) {
					if (ProjectPackage.MAP__CONTEXT_MODEL == event
							.getFeatureID(Map.class)) {
						if (event.getOldValue() != null) {
							((ContextModel) event.getOldValue())
									.getDeepAdapters().remove(this);
						}
						if (event.getNewValue() != null) {
							((ContextModel) event.getNewValue())
									.getDeepAdapters().add(this);
						}
					}
				} else if (event.getNotifier() instanceof Layer) {
					switch (event.getFeatureID(Layer.class)) {
					case ProjectPackage.LAYER__FILTER:
						manager.refreshSelection((Layer) event.getNotifier(),
								null);					
						break;
					case ProjectPackage.LAYER__STYLE_BLACKBOARD:
						synchronizeRenderers(manager, event);
						break;
					case ProjectPackage.LAYER__FEATURE_CHANGES:
						FeatureEvent featureEvent=(FeatureEvent) event.getNewValue();
						if( featureEvent==null ){
							break;
						}
						Envelope bounds=featureEvent.getBounds();
						if( featureEvent.getBounds()!=null && !featureEvent.getBounds().isNull() )
							manager.refresh((ILayer) event.getNotifier(), bounds );
						break;

					default:
						break;
					}


				}
			}
		};
	}

	/**
	 * Listens for render events
	 */
	static class RenderExecutorListener extends RenderListenerAdapter {

		private RenderManager manager;

		/**
		 * Construct <code>RenderManagerAdapters.RenderExecutorListener</code>.
		 */
		public RenderExecutorListener(RenderManager manager) {
			this.manager = manager;
		}

		/**
		 * @see net.refractions.udig.project.render.RenderListenerAdapter#renderStarting()
		 */
		protected void renderStarting() {
			((ViewportPane) manager.getMapDisplay()).renderStarting();
		}

		/**
		 * @see net.refractions.udig.project.render.RenderListenerAdapter#renderUpdate()
		 */
		protected void renderUpdate() {
			((ViewportPane) manager.getMapDisplay()).renderUpdate();
		}

		/**
		 * @see net.refractions.udig.project.render.RenderListenerAdapter#renderDone()
		 */
		protected void renderDone() {
			((ViewportPane) manager.getMapDisplay()).renderDone();
		}
	}

	static RenderExecutorListener getRenderExecutorListener(
			RenderManager manager) {
		return new RenderExecutorListener(manager);
	}

}
