/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.refractions.udig.catalog.CatalogPlugin;
import net.refractions.udig.catalog.IGeoResource;
import net.refractions.udig.catalog.IGeoResourceInfo;
import net.refractions.udig.catalog.IResolve;
import net.refractions.udig.catalog.IService;
import net.refractions.udig.catalog.ui.IDataWizard;
import net.refractions.udig.catalog.ui.ResolveLabelProviderSimple;
import net.refractions.udig.catalog.ui.ResolveTitlesDecorator;
import net.refractions.udig.catalog.ui.UDIGImportPage;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IPageChangedListener;
import org.eclipse.jface.dialogs.PageChangedEvent;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;

/**
 * A page that allows the user to select the resources he/she wish to include in the map.
 * 
 * @author jeichar
 * @since 0.9.0
 */
public class ResourceSelectionPage extends WizardPage 
	implements IPageChangedListener {

    private List<IResolve> resources = new ArrayList<IResolve>();
    private Map<IService, List<IGeoResource>> geoResources = new HashMap<IService, List<IGeoResource>>();
    private CheckboxTreeViewer viewer;

    /** url from workbench selection * */
    private URL url;
    private ResolveTitlesDecorator titleDecorator;
    private int work;
	private boolean isPageComplete;

    public ResourceSelectionPage( String pageName ) {
        super(pageName);
        setTitle(Policy.bind("ResourceSelectionPage.title")); //$NON-NLS-1$
        setMessage(Policy.bind("ResourceSelectionPage.message")); //$NON-NLS-1$
        setImageDescriptor(Images.getDescriptor(ImageConstants.CHOOSE_LAYER_WIZBAN));
    }

    public List<IResolve> getResources() {
        return resources;
    }

    public void setResources( List<IResolve> serviceList, IProgressMonitor monitor ) {
        this.resources = serviceList;
        setViewerInput(monitor);
    }

    public List<IGeoResource> getGeoResources( final IService service ) {
        if (geoResources.get(service) == null || geoResources.isEmpty()) {
            final List<IGeoResource> list = new ArrayList<IGeoResource>();

            try {
                getContainer().run(false, true, new IRunnableWithProgress(){

                    public void run( IProgressMonitor monitor ) {
                        try {
                            list.addAll(service.members(monitor));
                        } catch (Exception e) {
                            // do nothing
                        }
                    }

                });
                // list.addAll(service.members(null));
            } catch (Exception e) {
                // do nothing
            }
            geoResources.put(service, list);
        }
        return geoResources.get(service);
    }

    public List<Object> getCheckedElements() {
        List<Object> list = new ArrayList<Object>();
        for( Object object : viewer.getCheckedElements() ) {
            if (object instanceof IGeoResource) {
                IGeoResource resource = (IGeoResource) object;
                list.add(resource);
            }
        }
        return list;
    }

    /**
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl( Composite parent ) {
        Composite composite = new Composite(parent, SWT.NULL);
        composite.setLayout(new GridLayout());
        viewer = new CheckboxTreeViewer(composite);
        viewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        viewer.addPostSelectionChangedListener(new ISelectionChangedListener(){

            public void selectionChanged( SelectionChangedEvent event ) {
                getWizard().getContainer().updateButtons();
            }

        });
        viewer.addCheckStateListener(new ICheckStateListener(){
            public void checkStateChanged( CheckStateChangedEvent event ) {
                // set all children to same check state
                Object o = event.getElement();
                ServiceTreeProvider p = (ServiceTreeProvider) viewer.getContentProvider();
                Object[] children = p.getChildren(o);
                if (children != null && children.length > 0) {
                    for( int i = 0; i < children.length; i++ )
                        viewer.setChecked(children[i], event.getChecked());
                }

                // if checked set parent checked
                if (event.getChecked()) {
                    Object parent = p.getParent(o);
                    if (parent != null)
                        viewer.setChecked(parent, true);

                }
            }
        });
        viewer.setContentProvider(new ServiceTreeProvider());

        titleDecorator = new ResolveTitlesDecorator();
        LabelProvider labelProvider = new DecoratingLabelProvider(new ResolveLabelProviderSimple(),
                titleDecorator);

        viewer.setLabelProvider(labelProvider);
        viewer.setAutoExpandLevel(3);
        setViewerInput(null); // initialize viewer input to nothing
        setControl(composite);

        // check selection for context sensitivity
        IStructuredSelection selection=null;
        if (getWizard() instanceof IDataWizard) {
        	IDataWizard wizard = (IDataWizard) getWizard();
        	selection = wizard.getSelection();
		}
        URL url = null;
        if (selection != null)
            for( Iterator itr = selection.iterator(); itr.hasNext(); ) {
                @SuppressWarnings("unchecked") Object data = itr.next(); //$NON-NLS-1$
                url = CatalogPlugin.locateURL(data);

                if (url != null)
                    break;
            }

        if (url != null && url.getRef() != null && !url.getRef().equals("")) { //$NON-NLS-1$
            this.url = url;
        }
        
        if (getContainer() instanceof WizardDialog) {
        	WizardDialog dialog = (WizardDialog)getContainer();
        	dialog.addPageChangedListener(this);
        }
        //        
        // //look for a resource in the url, and check it
        // final String ref = url.getRef();
        // final ArrayList s = new ArrayList();
        // final ArrayList r = new ArrayList();
        //            
        // if (ref != null && !ref.equals("")) {
        // IRunnableWithProgress runnable = new IRunnableWithProgress() {
        //
        // public void run( IProgressMonitor monitor )
        // throws InvocationTargetException, InterruptedException {
        //                        
        // try {
        // //look for resource with a name matching the ref
        // if (resources == null || resources.isEmpty()) {
        // IWizardPage page = getPreviousPage();
        // if (page instanceof UDIGImportPage) {
        // final UDIGImportPage importPage = (UDIGImportPage)page;
        // try {
        // resources = importPage.getResources(monitor);
        // }
        // catch (Exception e) {
        // String msg = "Error obtaining info from resource";
        // CorePlugin.log(ProjectUIPlugin.getDefault(),msg,e);
        // }
        // }
        // }
        // for (IResolve resolve : resources) {
        // if( resolve instanceof IService ){
        // List<IGeoResource> resources=getGeoResources((IService)resolve);
        //                                
        // for (IGeoResource resource : resources) {
        // IGeoResourceInfo info = resource.getInfo(monitor);
        // if (ref.equals(info.getName())) {
        // s.add(resolve);
        // r.add(resource);
        // }
        // }
        //                  
        // }
        // }
        // }
        // catch (IOException e) {
        // String msg = "Error obtaining info from resource";
        // CorePlugin.log(ProjectUIPlugin.getDefault(),msg,e);
        // }
        // }
        // };
        // try {
        // getContainer().run(false, false, runnable);
        // for (int i = 0; i < r.size(); i++) {
        // viewer.setChecked(r.get(i), true);
        // viewer.setChecked(s.get(i), true);
        // }
        // }
        // catch (InvocationTargetException e) {
        // // do nothing
        // }
        // catch (InterruptedException e) {
        // //do nothing
        // }
        // }
        // }
    }

    private void setViewerInput( IProgressMonitor monitor ) {
        if (viewer == null || viewer.getControl().isDisposed()) {
            return;
        }

        viewer.getControl().getDisplay().asyncExec(new Runnable(){
            public void run() {
            	viewer.setInput(resources != null ? resources : new ArrayList());
                checkElements();
                // viewer.setCheckedElements(resources.toArray());
            }
        });
    }

    private void checkElements() {
    	for( IResolve resolve : resources ) {
            if (resolve instanceof IService) {
                // check if service has <= threshhold resources
                IService service = (IService) resolve;
                try {
                    List members = service.members(null);
                    if (members.size() > ProjectUIPlugin.MAX_RESOURCES_IN_SERVICE) {
                        viewer.setChecked(resolve, false);
                    } else {
                        viewer.setChecked(resolve, true);
                        for( Object o : members )
                            viewer.setChecked(o, true);
                    }
                } catch (IOException e) {
                    // do nothing
                }
            } else if (resolve instanceof IGeoResource) {
                // single resource, check
                viewer.setChecked(resolve, true);
            }
        }
    }
    /**
     * @see org.eclipse.jface.wizard.WizardPage#isPageComplete()
     */
    @Override
    public boolean isPageComplete() {
    	isPageComplete = false;
		if (resources == null || resources.size() == 0) {

            // obtain resources from previous page
            IWizardPage page = getPreviousPage();
            if (page instanceof UDIGImportPage) {
                final UDIGImportPage importPage = (UDIGImportPage) page;

                // first figure out how much work is going to need to take
                // place in order to report
                work = 0;
                IRunnableWithProgress runnable1 = new IRunnableWithProgress(){

                    public void run( IProgressMonitor monitor ) throws InvocationTargetException,
                            InterruptedException {

                        try {

                            List<IResolve> resolves = importPage.getResources(monitor);

                            for( IResolve resolve : resolves ) {
                                if (resolve instanceof IService) {
                                    IService service = (IService) resolve;

                                                    List members = service.members(monitor);
                                    if (members != null)
                                        work += members.size();
                                }
                                if (resolve instanceof IGeoResource) {
                                    work++;
                                }
                            }
                        } catch (Exception e) {
                            String msg = Policy.bind("ResourceSelectionPage.error"); //$NON-NLS-1$
                            ProjectUIPlugin.log( null, e);
                        }

                    }
                };

                final ArrayList<Boolean> cancelled = new ArrayList<Boolean>();
                cancelled.add(false);
                
                // now do the heavy lifting, ie, accessign info objects on
                // resource handles
                IRunnableWithProgress runnable2 = new IRunnableWithProgress(){
                    public void run( IProgressMonitor monitor ) throws InvocationTargetException,
                            InterruptedException {

                        try {

                            List<IResolve> resolves = importPage.getResources(monitor);

                            String msg = Policy
                                    .bind("ResourceSelectionPage.loadingResourceMessage"); //$NON-NLS-1$
                            monitor.beginTask(msg, work);

                            // touch each resource by grabbing its info
                            for( IResolve resolve : resolves ) {
                            	if (resolve instanceof IService) {
                                    IService service = (IService) resolve;
                                    List< ? extends IGeoResource> members = service.members(null);
                                    for( IGeoResource resource : members ) {
                                        String task = resource.getIdentifier().toString();
                                        
                                    	monitor.subTask(task);
                                        monitor.worked(1);

                                        try {
                                            resource.getInfo(monitor);
                                            if (monitor.isCanceled()) {
                                            	cancelled.set(0, true);
                                            	return;
                                            }
                                        } catch (Throwable t) {
                                            ProjectUIPlugin.log(Policy.bind("ResourceSelectionPage.connectionError"), t); //$NON-NLS-1$
                                        }
                                        
                                    }

                                } else if (resolve instanceof IGeoResource) {
                                    ((IGeoResource) resolve).getInfo(monitor);
                                }
                            }
                            setResources(resolves, monitor);

                            // if there was a url selected, try to match up with
                            // a georesource
                            if (url == null)
                                return;

                            for( IResolve resolve : resolves ) {
                                if (resolve instanceof IService) {
                                    final IService service = (IService) resolve;
                                    for( final IGeoResource resource : service.members(monitor) ) {
                                        IGeoResourceInfo info = resource.getInfo(monitor);
                                        if (info != null) {
                                            if (url.getRef().equals(info.getName())) {
                                                viewer.getControl().getDisplay().asyncExec(
                                                        new Runnable(){
                                                            public void run() {
                                                                viewer.setChecked(resource, true);
                                                                viewer.setChecked(service, true);
                                                            };
                                                        });
                                                return;
                                            }
                                        }
                                    }
                                }
                            }
                        } catch (Exception e) {
                            String msg = Policy.bind("ResourceSelectionPage.error"); //$NON-NLS-1$
                            ProjectUIPlugin.log( null, e);
                        }
                    }
                };
                try {
                    getContainer().run(false, false, runnable1);
                    getContainer().run(true, true, runnable2);
                    if (cancelled.get(0)) {
                    	isPageComplete = false;
                    	return isPageComplete;
                    }
                    
                } catch (InvocationTargetException e) {
                    String msg = Policy.bind("ResourceSelectionPage.error"); //$NON-NLS-1$
                    ProjectUIPlugin.log( null, e);
                } catch (InterruptedException e) {
                    String msg = Policy.bind("ResourceSelectionPage.error"); //$NON-NLS-1$
                    ProjectUIPlugin.log( null, e);
                }

                // perform a check for no resources
                if (work == 0) {
                	String msg = 
                		Policy.bind("ResourceSelectionPage.noResourceMessage"); //$NON-NLS-1$
                	
                	setErrorMessage(msg);
                	
                	isPageComplete = false;
                	return isPageComplete;
                }
                else {
                	setErrorMessage(null);
                }
                
                // perform a short circuit test here, if every service has less
                // then the max, or has a selection, force wizard to the next page
                boolean canFinish = true;
                O: for( IResolve resolve : resources ) {
                    if (resolve instanceof IService) {
                        List<IGeoResource> resources = getGeoResources((IService) resolve);

                        // for (IGeoResource resource : resources) {
                        // if (viewer.getChecked(resource))
                        // continue O;
                        // }
                        if (resources != null
                                && resources.size() > ProjectUIPlugin.MAX_RESOURCES_IN_SERVICE)
                            canFinish = false;
                    }
                }
                if (canFinish) {
                    getShell().getDisplay().asyncExec(new Runnable(){

                        public void run() {
                            // TODO: this should be extracted out into IDataWizard
                            if (getContainer() != null) {
                                IWizardPage page = getContainer().getCurrentPage();
                                IWizardPage next = getWizard().getNextPage(page);

                                if (next != null) {
                                    getContainer().showPage(next);
                                } else {
                                    // finish the wizard
                                    findButton(getShell().getChildren(), IDialogConstants.FINISH_ID)
                                            .notifyListeners(SWT.Selection, new Event());
                                }
                            }

                        }
                    });

                    // return true; //we are done
                }
            }
        }

        // we have killed ourselves already, so be safe about accessing the viewer
        if (viewer.getControl().isDisposed()) {
        	isPageComplete = true;
            return isPageComplete;
        }

        // figure out if any elements are checked
        isPageComplete = viewer.getCheckedElements().length != 0; 
        return isPageComplete;
        //        
        // boolean servicesHaveOneResource=true;
        // for (IResolve resolve : resources) {
        // if( resolve instanceof IService ){
        // List<IGeoResource> resources=getGeoResources((IService)resolve);
        // if( resources.size()!=1 ){
        // if( resources.size()>0){
        // servicesHaveOneResource=false;
        // break;
        // }
        // }
        // }
        // }
        // return servicesHaveOneResource || (isReady() && !(viewer==null ||
        // viewer.getCheckedElements().length==0));
    }

    Button findButton( Control[] children, int id ) {
        if (((Integer) getShell().getDefaultButton().getData()).intValue() == id) //$NON-NLS-1$
            return getShell().getDefaultButton();

        for( Control child : children ) {
            if (child instanceof Button) {
                Button button = (Button) child;
                if (((Integer) button.getData()).intValue() == id)
                    return button;
            }
            if (child instanceof Composite) {
                Composite composite = (Composite) child;
                Button button = findButton(composite.getChildren(), id);
                if (button != null)
                    return button;
            }
        }
        return null;
    }
    
    @Override
    /**
     * This method is overidden beacuse isPageComplete blocks. 
     * TODO: rework 
     */
    public boolean canFlipToNextPage() {
    	return isPageComplete && getNextPage() != null;
    }

    public class ServiceTreeProvider extends LabelProvider implements ITreeContentProvider {

        /**
         * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
         */
        public Object[] getElements( Object inputElement ) {
            return getChildren(inputElement);
        }

        /**
         * @see org.eclipse.jface.viewers.IContentProvider#dispose()
         */
        public void dispose() {
        }

        /**
         * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
         *      java.lang.Object, java.lang.Object)
         */
        public void inputChanged( Viewer viewer, Object oldInput, Object newInput ) {
        }

        public Object[] getChildren( Object parentElement ) {
            if (parentElement instanceof List) {
                List<IService> list = (List<IService>) parentElement;
                List<IService> services = new ArrayList<IService>();

                for( IService service : list ) {
                    if (getGeoResources(service).size() > 0)
                        services.add(service);
                }
                return services.toArray();
            }
            if (parentElement instanceof IService) {
                IService service = (IService) parentElement;
                List<IGeoResource> children = getGeoResources(service);
                return children.toArray();
            }
            return null;
        }

        public Object getParent( Object element ) {
            if (element instanceof IGeoResource) {
                IGeoResource resource = (IGeoResource) element;
                try {
                    return resource.parent(null);
                } catch (IOException e) {
                    ProjectUIPlugin.log( null, e);
                }
            }
            return null;
        }

        public boolean hasChildren( Object element ) {
            return element instanceof IService;
        }

        @Override
        public String getText( Object element ) {
            if (element instanceof IResolve) {
                IResolve resolver = (IResolve) element;
                return resolver.getIdentifier().toString();
            }
            return null;
        }

    }

    public boolean isReady() {
        for( IResolve resolve : resources ) {
            if (resolve instanceof IService) {
                int numResource = getGeoResources((IService) resolve).size();
                if (numResource > ProjectUIPlugin.MAX_RESOURCES_IN_SERVICE)
                    return true;
            }
        }
        return false;
    }

	public void pageChanged(PageChangedEvent event) {
		if (!this.equals(event.getSelectedPage())) {
			resources = new ArrayList<IResolve>();
		}
		
	}
}
