/**
 * 
 */
package net.refractions.udig.project.ui.internal;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import net.refractions.udig.internal.ui.UDIGDropAdapter;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.internal.ProjectPlugin;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IStartup;

/**
 * Loads the maps from the previous session and processes the commandline arguments. If the DND
 * extension point will be called so that it will act as if the url/string whatever was dropped on
 * the application. So if a url to shapefile is on the commandline a new map will be openned and
 * will display the shapefile.
 * 
 * @author jones
 */
public class StartupOpenMaps implements IStartup {

    /**
     * @see org.eclipse.ui.IStartup#earlyStartup()
     */
    public void earlyStartup() {

        // loads the maps from the previous session

         try {
             IPreferenceStore p = ProjectUIPlugin.getDefault().getPreferenceStore();
             int numEditors=p.getInt(MapEditor.ID);
             p.setValue(MapEditor.ID, 0);
             
             for( int i = 0; i < numEditors; i++ ) {
                 String id=MapEditor.ID+":"+i; //$NON-NLS-1$
                 String name = p.getString(id);
                 if( name==null || name.equals("")) //$NON-NLS-1$
                     continue;
                 p.setValue(id, ""); //$NON-NLS-1$
                 URI mapResourceURI = URI.createURI(name);
                 Resource resource = ProjectPlugin.getPlugin().getProjectRegistry().eResource()
                         .getResourceSet().getResource(mapResourceURI, true);
                 Map map = (Map) resource.getContents().get(0);
                 ProjectExplorer.getProjectExplorer().open(map);
                 
            }
        } catch (Exception e1) {
            ProjectUIPlugin.log("Error loading preferences for loading maps",e1); //$NON-NLS-1$
        } 

        String[] args = Platform.getApplicationArgs();
        for( String string : args ) {
            String urlString = string;
            try {
                if (urlString.startsWith("-")) { //$NON-NLS-1$
                    continue;
                }
                // try to make string a url
                if (!urlString.contains("://")) { //$NON-NLS-1$
                    File file = new File(urlString);
                    if (!file.exists())
                        continue;
                    urlString = "file://" + file.getAbsolutePath(); //$NON-NLS-1$
                }
                if (urlString.startsWith("file://")) {//$NON-NLS-1$
                    File file = new File(urlString.substring(6));
                    if (!file.exists())
                        continue;
                }

                URL url = new URL(urlString);
                Viewer viewer = LayersView.getViewer();
                if (viewer == null)
                    System.err.println("Layers View is not available");

                UDIGDropAdapter adapter = new UDIGDropAdapter(viewer, LayersView.getViewPart());
                adapter.performDrop(url);

            } catch (MalformedURLException e) {
                // ignore
            }
        }
    }

}
