package net.refractions.udig.project.ui.internal.actions;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.ui.internal.Policy;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;
import net.refractions.udig.style.sld.SLDContent;
import net.refractions.udig.ui.IDropAction;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.styling.Style;

public class SLDDropAction extends IDropAction {

	/** the sld url **/
	URL url;
	
	@Override
	public boolean accept(Object source, Object destination) {
		//make sure we can turn the object into an sld
		try {
			if (source instanceof URL) {
				url = (URL)source;
			}
			else if (source instanceof File) {
				url = ((File)source).toURL();
			}
			else if (source instanceof String) {
				try {
					url = new URL((String)source);	
				}
				catch(MalformedURLException e) {
					//try attaching a file protocol
					url = new URL("file://"+(String)source); //$NON-NLS-1$
				}
				
			}
		} 
		catch (MalformedURLException e) {
			String msg = Policy.bind("SLDDropAction.badSldUrl"); //$NON-NLS-1$
			ProjectUIPlugin.log(msg, e);
		}
		
		return url != null;
	}

	@Override
	public void perform(Object source, Object destination, IProgressMonitor monitor) {
		
		//grab the actual target
		Object target = getDropHandler().getTarget();
		if (target != null && target instanceof Layer) {
			Layer layer = (Layer)target;
			//parse the sld object
			
			try {
				Style style = SLDContent.parse(url);
				if (style != null) {
					SLDContent.apply(layer, style, monitor);
				}
				
			}
			catch(IOException e) {
				String msg = Policy.bind("SLDDropAction.sldParseError"); //$NON-NLS-1$
				ProjectUIPlugin.log(msg, e);
			}
		}
		
	}
	
}