/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.ui.internal.actions;

import java.io.IOException;
import java.util.Iterator;

import net.refractions.udig.project.command.NavigationCommandFactory;
import net.refractions.udig.project.internal.Layer;
import net.refractions.udig.project.internal.Map;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionDelegate;

import com.vividsolutions.jts.geom.Envelope;

/**
 * An action that sets the zoom to include all the data in the layer.
 * @author jeichar
 * @since 0.6.0
 */
public class ZoomToLayer extends ActionDelegate implements IViewActionDelegate {
    IStructuredSelection selection;
    
    /**
     * @see org.eclipse.ui.actions.ActionDelegate#selectionChanged(org.eclipse.jface.action.IAction, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged( IAction action, ISelection selection ) {
        try{
        this.selection=(IStructuredSelection) selection;
        }catch (Exception e) { //do nothing
        }
    }
    
    /**
     * @see org.eclipse.ui.actions.ActionDelegate#runWithEvent(org.eclipse.jface.action.IAction, org.eclipse.swt.widgets.Event)
     */
    public void runWithEvent( IAction action, Event event ) {
        try {
			PlatformUI.getWorkbench().getActiveWorkbenchWindow().run(false, true, new IRunnableWithProgress(){

				public void run(IProgressMonitor monitor){
			        Envelope bounds = new Envelope();
			        bounds.setToNull();
			        Map map=((Layer)selection.getFirstElement()).getContextModel().getMap();
			        for( Iterator iter = (selection).iterator(); iter.hasNext(); ) {
			            Layer layer = (Layer) iter.next();
			            	
			            if( layer.getContextModel().getMap()!=map )
			                return;
			            Envelope bbox=null;
                        try {
                            bbox = layer.getBounds(monitor, map.getViewportModel().getCRS());
                        } catch (IOException e) {
                            // TODO Catch e
                        }
			            if( bbox==null)
			                continue;
			            if( bounds.isNull() )
			                bounds.init(bbox);
			            else
			                bounds.expandToInclude( bbox );
			        if( !bounds.isNull() ){
			            map.sendCommandASync(NavigationCommandFactory.getInstance().createSetViewportBBoxCommand(bounds));
			        }
			        }
				}
			});
		} catch (Exception e) {
            ProjectUIPlugin.log( null, e);
		}
    }

    /**
     * @see org.eclipse.ui.IViewActionDelegate#init(org.eclipse.ui.IViewPart)
     */
    public void init( IViewPart view ) {
    	//do nothing
    }
}
