/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal.properties;

import java.util.ArrayList;
import java.util.List;

import net.refractions.udig.core.internal.GeometryBuilder;
import net.refractions.udig.project.ui.internal.Policy;

import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource2;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;

/**
 * A PropertySource for an array of coordinates.
 * 
 * @author jeichar
 * @since 0.3
 */
public class CoordinateSetPropertySource implements IPropertySource2 {
    /**
     * Provides ...TODO summary sentence
     * <p>
     * TODO Description
     * </p>
     * <p>
     * Responsibilities:
     * <ul>
     * <li>
     * <li>
     * </ul>
     * </p>
     * <p>
     * Example Use:
     * 
     * <pre><code>
     * 
     *  CoordinatePropertyDescriptor x = new CoordinatePropertyDescriptor( ... );
     *  TODO code example
     *  
     * </code></pre>
     * 
     * </p>
     * 
     * @author jones
     * @since 0.3
     */
    public class CoordinateValidator implements ICellEditorValidator {
        GeometryBuilder builder=GeometryBuilder.create();
        
        /**
         * @see org.eclipse.jface.viewers.ICellEditorValidator#isValid(java.lang.Object)
         */
        public String isValid( Object newValue ) {
            try {
                Coordinate[] coords=parseString((String) newValue);
                builder.createGeometry(geom.getClass(), coords);
            } catch (Exception e) {
                return e.getLocalizedMessage();
            }
            return null;
        }

    }

    private Coordinate[] coords;
    private Coordinate[] oldcoords;
    IPropertyDescriptor[] descriptors;
    protected Geometry geom;
    /**
     * 
     * Construct <code>CoordinateSetPropertySource</code>.
     *
     * @param coords
     * @param geom
     */
    public CoordinateSetPropertySource( Coordinate[] coords, Geometry geom ) {
        this(coords);
        this.geom = geom;
    }

    /**
     * Creates a new instance of CoordinatePropertySource
     * 
     * @param coords
     */
    public CoordinateSetPropertySource( Coordinate[] coords ) {
        oldcoords = this.coords = coords;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
     */
    public Object getEditableValue() {
        return coords;
    }

    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
     */
    public IPropertyDescriptor[] getPropertyDescriptors() {
        if (descriptors == null) {
            if (geom != null) {
                descriptors = new PropertyDescriptor[]{new TextPropertyDescriptor(coords, Policy.bind("CoordinateSetPropertySource.coordinates"))}; //$NON-NLS-1$
                ((TextPropertyDescriptor) descriptors[0]).setValidator(new CoordinateValidator());
            } else
                descriptors = new PropertyDescriptor[]{new PropertyDescriptor(coords, Policy.bind("CoordinateSetPropertySource.coordinates"))}; //$NON-NLS-1$
        }
        return descriptors;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
     */
    public Object getPropertyValue( Object id ) {
        return coordsToString(coords);
    }

    private String coordsToString( Coordinate[] coords ) {

        StringBuffer cBuffer = new StringBuffer();
        for( int j = 0; j < coords.length; j++ ) {
            if (j != 0)
                cBuffer.append(", "); //$NON-NLS-1$
            cBuffer.append("("); //$NON-NLS-1$
            cBuffer.append(coords[j].x);
            cBuffer.append(", "); //$NON-NLS-1$
            cBuffer.append(coords[j].y);
            cBuffer.append(")"); //$NON-NLS-1$
        }
        return cBuffer.toString();
    }

    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
     */
    public boolean isPropertySet( Object id ) {
        return coords == oldcoords;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
     */
    public void resetPropertyValue( Object id ) {
        coords = oldcoords;
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
     */
    public void setPropertyValue( Object id, Object value ) {
        coords = parseString((String) value);
    }
    /**
     * @see org.eclipse.ui.views.properties.IPropertySource2#isPropertyResettable(java.lang.Object)
     */
    public boolean isPropertyResettable( Object id ) {
        return true;
    }

    Coordinate[] parseString( String stringValue ) {
        String[] parts = stringValue.split(","); //$NON-NLS-1$
        List newCoords = new ArrayList();
        double[] points = new double[2];
        int q = 0;
        for( int i = 0; i < parts.length; i++ ) {
            String part = parts[i].trim();
            String[] values = part.split("\\s"); //$NON-NLS-1$
            for( int j = 0; j < values.length; j++ ) {
                String value = values[j].trim();
                if (value.startsWith("(")) { //$NON-NLS-1$
                    points[q] = Double.parseDouble(value.substring(1));
                } else if (value.endsWith(")")) { //$NON-NLS-1$
                    points[q] = Double.parseDouble(value.substring(0, value.length() - 1));
                } else {
                    points[q] = Double.parseDouble(value.substring(1));
                }
                q++;
                if (q == 2) {
                    q = 0;
                    newCoords.add(new Coordinate(points[0], points[1]));
                }
            }
        }
        return (Coordinate[]) newCoords.toArray(new Coordinate[newCoords.size()]);
    }

}
