/**
 * 
 */
package net.refractions.udig.project.ui.internal.render.displayAdapter.impl;

import java.awt.Dimension;

import net.refractions.udig.project.render.displayAdapter.MapDisplayEvent;
import net.refractions.udig.project.ui.internal.Policy;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseWheelEvent;
import net.refractions.udig.project.ui.render.displayAdapter.ViewportPane;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

class EventHandler implements Listener {
    EventJob eventJob;

	ViewportPane pane;

	// ControlListener, MouseListener, MouseMoveListener, MouseTrackListener {

	/**
	 * @param java
	 */
	EventHandler(ViewportPane pane, EventJob eventJob) {
		this.eventJob=eventJob;
		this.pane=pane;
	}

	/**
     * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
     */
    public void handleEvent( Event event ) {
        switch( event.type ) {
        case SWT.MouseDoubleClick:
            mouseDoubleClick(event);
            break;
        case SWT.MouseMove:
            mouseMove(event);
            break;
        case SWT.MouseDown:
            mouseDown(event);
            break;
        case SWT.MouseEnter:
            mouseEnter(event);
            break;
        case SWT.MouseExit:
            mouseExit(event);
            break;
        case SWT.MouseHover:
            mouseHover(event);
            break;
        case SWT.MouseUp:
            mouseUp(event);
            break;
        case SWT.MouseWheel:
            mouseWheel(event);
            break;
        case SWT.Resize:
            controlResized(event);
        case SWT.KeyDown:
        // System.out.println("keydown");
        }
    }

    /**
     * TODO summary sentence for mouseWheel ...
     * 
     * @param event
     */
    private void mouseWheel( Event e ) {
        MapMouseEvent m = new MapMouseWheelEvent(pane, getState(e, getButton(e.button)), e.x,
                e.y, e.count);
        eventJob.fire(EventJob.WHEEL, m);
    }

    Dimension size;

    Job timer = new Job(Policy.bind("ViewportPaneJava.resize.title")){ //$NON-NLS-1$

        protected IStatus run( IProgressMonitor monitor ) {
            // System.out.println("Running Resize event");
            eventJob.fire(EventJob.RESIZED, new MapDisplayEvent(pane, size,
                    pane.getDisplaySize()));
            size = pane.getDisplaySize();
            return Status.OK_STATUS;
        }
    };
    boolean running = false;
    boolean scheduled = false;
    {
        timer.setPriority(Job.INTERACTIVE);
        timer.addJobChangeListener(new JobChangeAdapter(){
            public void scheduled( IJobChangeEvent event ) {
                scheduled = true;
            }
            /**
             * @see org.eclipse.core.runtime.jobs.JobChangeAdapter#running(org.eclipse.core.runtime.jobs.IJobChangeEvent)
             */
            public void running( IJobChangeEvent event ) {
                running = true;
                scheduled = false;
            }

            /**
             * @see org.eclipse.core.runtime.jobs.JobChangeAdapter#done(org.eclipse.core.runtime.jobs.IJobChangeEvent)
             */
            public void done( IJobChangeEvent event ) {
                running = false;
            }

        });
    }

    /**
     * @see org.eclipse.swt.events.ControlListener#controlResized(org.eclipse.swt.events.ControlEvent)
     */
    public synchronized void controlResized( Event e ) {
        if (!running && scheduled) {
            timer.cancel();
            scheduled = false;
        }

        timer.schedule(1000);

    }

    /**
     * @see org.eclipse.swt.events.MouseListener#mouseDoubleClick(org.eclipse.swt.events.MouseEvent)
     */
    public void mouseDoubleClick( Event e ) {
        MapMouseEvent m = new MapMouseEvent(pane, getState(e, getButton(e.button)), e.x, e.y,
                getButton(e.button));
        eventJob.fire(EventJob.DOUBLE_CLICK, m);
    }

    private int getState( Event e, int button ) {
        int state = 0;
        if ((e.stateMask & SWT.ALT) != 0)
            state = MapMouseEvent.ALT_DOWN_MASK;
        if ((e.stateMask & SWT.CONTROL) != 0)
            state = state | MapMouseEvent.CTRL_DOWN_MASK;
        if ((e.stateMask & SWT.SHIFT) != 0)
            state = state | MapMouseEvent.SHIFT_DOWN_MASK;

        state = state | button;
        return state;
    }

    int buttondown = 0;

    private int getButton( int button ) {
        int state = 0;
        if (button == 1)
            state = state | MapMouseEvent.BUTTON1;
        if (button == 2)
            state = state | MapMouseEvent.BUTTON2;
        if (button == 3)
            state = state | MapMouseEvent.BUTTON3;
        return state;
    }

    /**
     * @see org.eclipse.swt.events.MouseListener#mouseDown(org.eclipse.swt.events.MouseEvent)
     */
    public void mouseDown( Event e ) {
        MapMouseEvent m = new MapMouseEvent(pane, getState(e, getButton(e.button)), e.x, e.y,
                getButton(e.button));
        buttondown = getButton(e.button);
        eventJob.fire(EventJob.PRESSED, m);
    }

    /**
     * @see org.eclipse.swt.events.MouseListener#mouseUp(org.eclipse.swt.events.MouseEvent)
     */
    public void mouseUp( Event e ) {
        MapMouseEvent m = new MapMouseEvent(pane, getState(e, getButton(e.button)), e.x, e.y,
                getButton(e.button));
        buttondown = 0;
        eventJob.fire(EventJob.RELEASED, m);
    }

    /**
     * @see org.eclipse.swt.events.MouseMoveListener#mouseMove(org.eclipse.swt.events.MouseEvent)
     */
    public void mouseMove( Event e ) {
        MapMouseEvent m = new MapMouseEvent(pane, getState(e, buttondown), e.x, e.y, buttondown);
        if (buttondown == 0)
            eventJob.fire(EventJob.MOVED, m);
        else {
            eventJob.fire(EventJob.DRAGGED, m);
        }
    }

    /**
     * @see org.eclipse.swt.events.MouseTrackListener#mouseEnter(org.eclipse.swt.events.MouseEvent)
     */
    public void mouseEnter( Event e ) {
        MapMouseEvent m = new MapMouseEvent(pane, getState(e, getButton(e.button)), e.x, e.y,
                getButton(e.button));
        eventJob.fire(EventJob.ENTERED, m);
    }

    /**
     * @see org.eclipse.swt.events.MouseTrackListener#mouseExit(org.eclipse.swt.events.MouseEvent)
     */
    public void mouseExit( Event e ) {
        MapMouseEvent m = new MapMouseEvent(pane, getState(e, getButton(e.button)), e.x, e.y,
                getButton(e.button));
        eventJob.fire(EventJob.EXITED, m);
    }

    /**
     * @see org.eclipse.swt.events.MouseTrackListener#mouseHover(org.eclipse.swt.events.MouseEvent)
     */
    public void mouseHover( Event e ) {
        // do nothing
    }
}