/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.internal.render.displayAdapter.impl;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.image.BufferedImage;

import net.refractions.udig.project.internal.render.RenderExecutor;
import net.refractions.udig.project.internal.render.RenderManager;
import net.refractions.udig.project.internal.render.Renderer;
import net.refractions.udig.project.render.IRenderContext;
import net.refractions.udig.project.render.IRenderer;
import net.refractions.udig.project.render.displayAdapter.IMapDisplayListener;
import net.refractions.udig.project.ui.commands.IDrawCommand;
import net.refractions.udig.project.ui.internal.MapEditor;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseListener;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseMotionListener;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseWheelListener;
import net.refractions.udig.project.ui.render.displayAdapter.ViewportPane;
import net.refractions.udig.ui.graphics.SWTGraphics;

import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

/**
 * The ViewportPaneImpl is a java.awt.Panel that is the display area for a Map.
 * It Registers itself with a RenderStack and obtains the image from the
 * RenderStack if the RenderStack is "ready"
 * 
 * @author Jesse Eichar
 * @version $Revision: 1.9 $
 */
public class ViewportPaneSWT extends Canvas implements ViewportPane {
	ViewportPane pane = this;

	private static final long serialVersionUID = 1L;

	private ViewportPainter painter = new ViewportPainter(this);

	EventJob eventJob = new EventJob();

	SWTGraphics swtGraphics;

	private RenderManager renderManager;

	private EventHandler handler;

	private MapEditor editor;

	private Display display;

	Dimension displaySize = new Dimension(0, 0);

	private org.eclipse.swt.graphics.Image swtImage;

	/**
	 * Create a image that is compatable with this ViewportPane.
	 * <p>
	 * This image is a large block of memory that will be blitted into an SWT
	 * Image. Currently this is BufferedImage.TYPE_4BYTE_ABGR although you
	 * should never depend on the Image type directly.
	 * </p>
	 * <p>
	 * This image is *not* expetcted to be hardware accelarated, althought the
	 * blit process will be.
	 * </p>
	 * 
	 * @see net.refractions.udig.project.render.ViewportPane#aquireImage(int,
	 *      int)
	 * @param w
	 *            width
	 * @param h
	 *            height
	 * @return BufferedImage with same color model as SWT Image.
	 */
	public BufferedImage image(int w, int h) {
		return new BufferedImage(w, h, BufferedImage.TYPE_4BYTE_ABGR);
	}

	/**
	 * Creates a new ViewportPaneImpl object.
	 * 
	 * @param comp
	 *            The Composite that this pane will be embedded into
	 * @param renderStack
	 *            The renderstack that is rendering onto this viewport
	 * @param vmodel
	 *            The Viewport model that models this viewport
	 */
	public ViewportPaneSWT(Composite comp, MapEditor editor) {
		super(comp, SWT.NO_BACKGROUND);
		ProjectUIPlugin.trace("ViewportPaneSWT created", null); //$NON-NLS-1$
		display = getShell().getDisplay();

		this.editor = editor;
		handler = new EventHandler(this, eventJob);
		// setBackground(Color.WHITE);

		addListener(SWT.Resize, new Listener() {
			public void handleEvent(Event event) {
				Point size = getSize();
				displaySize = new Dimension(size.x, size.y);
			}
		});
		addListener(SWT.MouseDoubleClick, handler);
		addListener(SWT.MouseDown, handler);
		addListener(SWT.MouseEnter, handler);
		addListener(SWT.MouseExit, handler);
		addListener(SWT.MouseHover, handler);
		addListener(SWT.MouseMove, handler);
		addListener(SWT.MouseUp, handler);
		addListener(SWT.MouseWheel, handler);
		addListener(SWT.Resize, handler);
		addListener(SWT.KeyDown, handler);
		addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent event) {
				paint(event.gc, event.display);
			}
		});
	}

	/**
	 * @see net.refractions.udig.project.ui.render.displayAdapter.ViewportPane#setRenderManager(net.refractions.udig.project.render.RenderManager)
	 */
	public void setRenderManager(RenderManager manager) {
		this.renderManager = manager;
	}

	/**
	 * Called by the Paint listener to update the canvas
	 * 
	 * @param display
	 *            the display of the canvas.
	 * @param g
	 *            the GC object to use to draw with.
	 */
	public void paint(GC g, Display display) {
		if (swtGraphics != null)
			swtGraphics.dispose();

		swtGraphics = new SWTGraphics(g, display, getDisplaySize());

		org.eclipse.swt.graphics.Image buffer = getImage();
		Rectangle bounds = buffer.getBounds();
		int minWidth = Math.min(bounds.width, getWidth());
		int minHeight = Math.min(bounds.height, getHeight());
		painter.paint(swtGraphics, buffer, minWidth, minHeight);
		swtGraphics.dispose();
	}

	org.eclipse.swt.graphics.Image getImage() {
		if (!Platform.isRunning())
			return null;
		if (swtImage != null) {
			return swtImage;
		}

		try {
			RenderExecutor renderExecutor = renderManager.getRenderExecutor();
			Renderer renderer = renderExecutor.getRenderer();
			if (renderer.getState() == IRenderer.DISPOSED)
				return null;
			IRenderContext context = renderer.getContext();
			swtImage = SWTGraphics.createSWTImage(context.getImage(getWidth(),
					getHeight()), false);
			return swtImage;
		} catch (Throwable e) {
			ProjectUIPlugin.log(null, e);
			return null;
		}
	}

	void initMap() {
		swtImage = null;
	}

	/**
	 * @see net.refractions.udig.project.ui.render.displayAdapter.ViewportPane#renderStarting()
	 */
	public void renderStarting() {
		painter.renderStart();
		repaint();
	}

    /**
     * @see net.refractions.udig.project.ui.render.displayAdapter.ViewportPane#renderDone()
     */
    public void renderDone() {
        renderUpdate();
        painter.renderDone();
    }


	/**
	 * @see net.refractions.udig.project.ui.render.displayAdapter.ViewportPane#renderUpdate()
	 */
	public void renderUpdate() {
		initMap();
		if (getImage() != null) {
			painter.renderUpdate();
			repaint();
		}
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#setTransform(AffineTransform)
	 */

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#dispose()
	 */
	public void dispose() {
		// TODO
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#addDrawCommand(net.refractions.udig.project.internal.commands.draw.IDrawCommand)
	 */
	public void addDrawCommand(IDrawCommand command) {
		painter.addDrawCommand(command);
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#repaint()
	 */
	public void repaint() {
		if (display.getThread() != Thread.currentThread())
			Display.getDefault().asyncExec(new Runnable() {

				public void run() {
					if (!isDisposed())
						redraw();
					else
						ProjectUIPlugin
								.log(
										"gettingDisplaySize() in ViewportPane SWT", new Exception()); //$NON-NLS-1$
				}

			});
		else
			redraw();
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#setCursor(java.awt.Cursor)
	 */
	public void setCursor(final Cursor cursor) {
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				String name = cursor.getName();
				if (name.equals("Default Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_ARROW));
				} else if (name.equals("Crosshair Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_CROSS));
				} else if (name.equals("Text Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_IBEAM));
				} else if (name.equals("Wait Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_WAIT));
				} else if (name.equals("SW Resize Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_SIZESW));
				} else if (name.equals("SE Resize Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_SIZESE));
				} else if (name.equals("NW Resize Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_SIZENW));
				} else if (name.equals("NE Resize Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_SIZENE));
				} else if (name.equals("N Resize Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_SIZEN));
				} else if (name.equals("S Resize Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_SIZES));
				} else if (name.equals("W Resize Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_SIZEW));
				} else if (name.equals("E Resize Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_SIZEE));
				} else if (name.equals("Hand Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_HAND));
				} else if (name.equals("Move Cursor")) { //$NON-NLS-1$
					setCursor(new org.eclipse.swt.graphics.Cursor(Display
							.getDefault(), SWT.CURSOR_SIZEALL));
				}
			}
		});
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#removeMouseListener(net.refractions.udig.project.render.MapMouseListener)
	 */
	public void removeMouseListener(MapMouseListener l) {
		eventJob.removeMouseListener(l);
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#removeMouseMotionListener(net.refractions.udig.project.render.MapMouseMotionListener)
	 */
	public void removeMouseMotionListener(MapMouseMotionListener l) {
		eventJob.removeMouseMotionListener(l);
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#removeMouseWheelListener(net.refractions.udig.project.render.MapMouseWheelListener)
	 */
	public void removeMouseWheelListener(MapMouseWheelListener l) {
		eventJob.removeMouseWheelListener(l);
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#addMouseListener(net.refractions.udig.project.render.MapMouseListener)
	 */
	public void addMouseListener(MapMouseListener l) {
		eventJob.addMouseListener(l);
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#addMouseMotionListener(net.refractions.udig.project.render.MapMouseMotionListener)
	 */
	public void addMouseMotionListener(MapMouseMotionListener l) {
		eventJob.addMouseMotionListener(l);
	}

	/**
	 * @see net.refractions.udig.project.render.ViewportPane#addMouseWheelListener(net.refractions.udig.project.render.MapMouseWheelListener)
	 */
	public void addMouseWheelListener(MapMouseWheelListener l) {
		eventJob.addMouseWheelListener(l);
	}

	/**
	 * @see net.refractions.udig.project.render.MapDisplay#getDisplaySize()
	 */
	public Dimension getDisplaySize() {
		return displaySize;
	}

	/**
	 * @see net.refractions.udig.project.render.MapDisplay#getWidth()
	 */
	public int getWidth() {
		return getDisplaySize().width;
	}

	/**
	 * @see net.refractions.udig.project.render.MapDisplay#getHeight()
	 */
	public int getHeight() {
		return getDisplaySize().height;
	}

	/**
	 * @see net.refractions.udig.project.ui.render.displayAdapter.ViewportPane#addPaneListener(net.refractions.udig.project.render.displayAdapter.MapDisplayListener)
	 */
	public void addPaneListener(IMapDisplayListener listener) {
		eventJob.addMapEditorListener(listener);
	}

	/**
	 * @see net.refractions.udig.project.ui.render.displayAdapter.ViewportPane#removePaneListener(net.refractions.udig.project.render.displayAdapter.MapDisplayListener)
	 */
	public void removePaneListener(IMapDisplayListener listener) {
		eventJob.removeMapEditorListener(listener);
	}

	/**
	 * @see net.refractions.udig.project.ui.render.displayAdapter.ViewportPane#getMapEditor()
	 */
	public MapEditor getMapEditor() {
		return editor;
	}

	public int getDPI() {
		Point dpi=getDisplay().getDPI();
		return dpi.x*dpi.y;
	}
}