/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.project.ui.internal.tool.display;


import java.awt.Color;
import java.text.MessageFormat;

import net.refractions.udig.project.ui.PlatformGIS;
import net.refractions.udig.project.ui.internal.Policy;
import net.refractions.udig.project.ui.internal.ProjectUIPlugin;
import net.refractions.udig.project.ui.tool.ActionTool;
import net.refractions.udig.project.ui.tool.IToolContext;
import net.refractions.udig.project.ui.tool.ModalTool;
import net.refractions.udig.project.ui.tool.Tool;
import net.refractions.udig.ui.graphics.Glyph;

import org.eclipse.core.commands.IHandler;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 * The tool proxy allows tools to be loaded lazily. It acts as a proxy for a
 * tool as far as the Map editors are concerned.
 * 
 * @author Jesse Eichar
 * @version $Revision: 1.9 $
 */
public class ToolProxy extends ModalItem implements ModalTool, ActionTool {
	/**
	 * Handles lazy cursor loading.
	 * 
	 * @author jeichar
	 * @since 0.9.0
	 */
	public static class CursorLoader {

		private Cursor cursor;
		private String imagePath;
		private String hotspotX;
		private String hotspotY;
		private String cursorId;
		private String pluginID;
		/**
		 * Construct <code>ToolProxy.CursorLoader</code>.
		 *
		 */
		public CursorLoader(IConfigurationElement cursorDef) {
			if( cursorDef!=null ){
	    	imagePath=cursorDef.getAttribute("image"); //$NON-NLS-1$
	    	hotspotX=cursorDef.getAttribute("hotspotX"); //$NON-NLS-1$
	    	hotspotY=cursorDef.getAttribute("hotspotY"); //$NON-NLS-1$
	    	cursorId=cursorDef.getAttribute("id"); //$NON-NLS-1$
	    	pluginID=cursorDef.getNamespace();
			}else{
				cursorId=ModalTool.DEFAULT_CURSOR;
			}
		}
		
		/**
		 * @return Returns the cursor.
		 */
		public Cursor getCursor() {
			if (cursor == null) {
				synchronized (this) {
					if (cursor == null) {
						if( imagePath==null ){
							cursor=parseCursorId(cursorId);
						}else{
							ImageDescriptor imageDescriptor = AbstractUIPlugin.imageDescriptorFromPlugin(pluginID, imagePath );
							int x;
							try{
								x=Integer.parseInt(hotspotX);
							}catch (Exception e) {
								x=0;
							}
							int y;
							try{
								y=Integer.parseInt(hotspotY);
							}catch (Exception e) {
								y=0;
							}
							if( imageDescriptor==null || imageDescriptor.getImageData()==null )
								cursor=parseCursorId(cursorId);
							else
								cursor=new Cursor(Display.getDefault(), imageDescriptor.getImageData(), x, y );
						}
					}
				}
			}

			return cursor;
		}
		
	    Cursor parseCursorId(String cursor) {
	        if (cursor == null)
	            return defaultCursor;
	        Display display=PlatformUI.getWorkbench().getDisplay();
	        if (cursor.equals(ModalTool.CROSSHAIR_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_CROSS);
	        if (cursor.equals(ModalTool.E_RESIZE_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_SIZEE);
	        if (cursor.equals(ModalTool.HAND_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_HAND);
	        if (cursor.equals(ModalTool.MOVE_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_SIZEALL);
	        if (cursor.equals(ModalTool.N_RESIZE_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_SIZEN);
	        if (cursor.equals(ModalTool.NE_RESIZE_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_SIZENE);
	        if (cursor.equals(ModalTool.NW_RESIZE_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_SIZENW);
	        if (cursor.equals(ModalTool.S_RESIZE_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_SIZES);
	        if (cursor.equals(ModalTool.SE_RESIZE_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_SIZESE);
	        if (cursor.equals(ModalTool.SW_RESIZE_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_SIZESW);
	        if (cursor.equals(ModalTool.TEXT_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_IBEAM);
	        if (cursor.equals(ModalTool.W_RESIZE_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_SIZESW);
	        if (cursor.equals(ModalTool.WAIT_CURSOR))
	            return display.getSystemCursor(SWT.CURSOR_WAIT);
	        return defaultCursor;
	    }

		/**
		 * Dispose the cursor.
		 */
		public void dispose() {
			if( cursor!=null )
				cursor.dispose();
			cursor=null;
		}

		
	}

	/** The currently active tool */
	public static ToolProxy activeItem = null;
	/** The default tool */
	public static ToolProxy defaultItem = null;
	
	Tool tool = null;
    IToolContext toolContext=null;
    IConfigurationElement element = null;
    boolean hasControl = false;
    static Cursor currentCursor = defaultCursor;
    CursorLoader toolCursor;
    private boolean onToolbar = false;
    int type = -1;
    static final int MODAL = 1;
    static final int BACKGROUND = 2;
    static final int ACTION = 3;
    private String categoryId;
    private static final String DEFAULT_ID = "net.refractions.udig.tools.Zoom"; //$NON-NLS-1$

    private IAction action;
	private Job job;
    private String menuPath;

    /**
     * Creates an new instance of MapViewport.ToolAction
     * 
     * @param extension
     *            The Tool extension
     * @param tool
     *            The configuration element which describes the tool
     */
    public ToolProxy(IExtension extension, IConfigurationElement tool) {
        super();
        categoryId=tool.getAttribute("categoryId"); //$NON-NLS-1$
        String type = tool.getName();
        String pluginid = extension.getNamespace();
        String id = tool.getAttribute("id"); //$NON-NLS-1$
        String name = tool.getAttribute("name"); //$NON-NLS-1$
        if( name==null ) name=Policy.bind("ToolProxy.unnamed"); //$NON-NLS-1$
        String toolTip = tool.getAttribute("tooltip"); //$NON-NLS-1$
        String iconID = tool.getAttribute("icon"); //$NON-NLS-1$
        toolCursor=parseCursor(tool);
        String bool = tool.getAttribute("hasCustomControl"); //$NON-NLS-1$
        hasControl = ((bool != null) && bool.equalsIgnoreCase("true")) ? true : false; //$NON-NLS-1$
        bool = tool.getAttribute("onToolbar"); //$NON-NLS-1$
        onToolbar = ((bool != null) && bool.equalsIgnoreCase("true")) ? true : false; //$NON-NLS-1$
        menuPath = tool.getAttribute("menuPath"); //$NON-NLS-1$
        if (iconID != null) {
            ImageDescriptor icon = AbstractUIPlugin.imageDescriptorFromPlugin(pluginid, iconID);
            setImageDescriptor(icon);
        }else{
        	setImageDescriptor(Glyph.geometry(Color.RED, Color.RED));
        }
        this.element = tool;
        setName(name);
        setToolTipText(toolTip);
        setId(id);
        if (type.equals("modalTool")) //$NON-NLS-1$
            this.type = MODAL;
        else if (type.equals("backgroundTool")) //$NON-NLS-1$
            this.type = BACKGROUND;
        else if (type.equals("actionTool")) //$NON-NLS-1$
            this.type = ACTION;
        handlerType=tool.getAttribute(HandlerProxy.ID);
        
        String unparsedCommandIds=tool.getAttribute("commandIds"); //$NON-NLS-1$
        if( unparsedCommandIds!=null && unparsedCommandIds.length()>0)
            commandIds=unparsedCommandIds.split(","); //$NON-NLS-1$
        else
            commandIds=new String[0];
        
        if( id.equals(DEFAULT_ID) ){
        	defaultItem=this;
        }
    }

    CursorLoader parseCursor(IConfigurationElement toolDef){
    	
    	//should only be one
    	IConfigurationElement[] children=toolDef.getChildren("cursor"); //$NON-NLS-1$
    	
    	return new CursorLoader(children.length>0?children[0]:null);

    }
    
    /**
     * @see org.eclipse.jface.action.Action#getStyle()
     */
    public int getStyle() {
        if (type == MODAL)
            return IAction.AS_RADIO_BUTTON;
        return IAction.AS_PUSH_BUTTON;
    }

    /**
     * @see net.refractions.udig.project.ui.tool.Tool#setContext(net.refractions.udig.project.ui.tool.IToolContext)
     */
    public void setContext(IToolContext tools) {
        this.toolContext=tools;
        tools.getViewportPane().setCursor(currentCursor);
        if (type == BACKGROUND) {
            getTool().setContext(tools);
        } else if ((activeItem != null) && (activeItem == this)) {
            activeItem.getTool().setContext(tools);
            setActive(true);
        } else if (tool != null) {
            getTool().setContext(tools);
            if (tool instanceof ModalTool)
                getModalTool().setActive(false);
        }
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return getId();
    }
    ModalTool getModalTool() {
        return (ModalTool) getTool();
    }
    ActionTool getModelessTool() {
        return (ActionTool) getTool();
    }
    Tool getTool() {
        if (tool == null) {
            String klassName = element.getAttribute("class"); //$NON-NLS-1$
            if (klassName != null) {
                try {
                    Object o = element.createExecutableExtension("class"); //$NON-NLS-1$
                    tool = (Tool) o;
                    tool.setContext(toolContext);
                    if (tool instanceof ModalTool) {
                        ((ModalTool) tool).setActive(false);
                    }
                } catch (CoreException e) {
                    Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
                    ErrorDialog.openError(shell, Policy.bind("ToolProxy.error.title"),  //$NON-NLS-1$
                                    MessageFormat.format(Policy.bind("ToolProxy.error.message"), new Object[] { e.getMessage() }), //$NON-NLS-1$
                                    e.getStatus());
                    e.printStackTrace();
                }
            }
        }
        return tool;
    }
    
    /**
	 * @return Returns the type.
	 */
	public int getType() {
		return type;
	}

    /**
     * If the current action is a ActionTool then the menu path of the tool action is returned
     * 
     * @return the menu path of the tool action or null if the current tool is not an ActionTool.
     */
    public String getMenuPath(){
        return menuPath;
    }
    /**
     * @return Returns the onToolbar.
     */
    public boolean isOnToolbar() {
        return onToolbar;
    }
    
    /**
     * @see net.refractions.udig.project.ui.tool.Tool#getContext()
     */
    public IToolContext getContext() {
        return toolContext;
    }


    /**
     * TODO summary sentence for getCategoryId ...
     * 
     * @return
     */
    public String getCategoryId() {
        return categoryId;
    }
    
	/**
	 * Creates an action that will run the tool referenced by this proxy
	 * @return an action that will run the tool referenced by this proxy
	 */
	public IAction getAction() {
		if (action == null) {
			synchronized (this) {
				if (action == null) {
					action = new ToolAction(this);
				}
			}
		}

		return action;
	}
	
	/**
	 * @see net.refractions.udig.project.ui.internal.tool.display.ModalItem#runModeless()
	 */
	protected boolean runModeless() {
	    if (type == ACTION) {
	    	PlatformGIS.run(new ISafeRunnable() {
			
				public void run() throws Exception {
					getModelessTool().run();
				}
			
				public void handleException(Throwable exception) {
					ProjectUIPlugin.log( "Error occured while executing tool: "+getId(), exception); //$NON-NLS-1$
				}
			
			});
	    	return true;
	    }

	    	return false;
	}

	/**
	 * @see net.refractions.udig.project.ui.internal.tool.display.ModalItem#setActive(boolean)
	 */
	public void setActive(boolean active) {
        if( toolContext==null )
            return;
	        setChecked(active);
	        if( active )
		        currentCursor = toolCursor.getCursor();
	        else
	        	currentCursor = defaultCursor;

            toolContext.getViewportPane().setCursor(currentCursor);

            if (getTool() instanceof ModalTool) {
	            if (activeItem == this && !active){
	                activeItem.setChecked(false);
	            }
	            getModalTool().setActive(active);
	        }
	    }

	/**
	 * @see net.refractions.udig.project.ui.internal.tool.display.ModalItem#dispose()
	 */
	public void dispose() {
	    if (tool != null) {
	        tool.dispose();
	    }
        if ( this==activeItem )
            activeItem=null;
        
	    toolCursor.dispose();
	}

	/**
	 * @see net.refractions.udig.project.ui.internal.tool.display.ModalItem#getHandler(java.lang.String)
	 */
	public IHandler getHandler(String commandId) {
	    if( handlerType!=null && handlerType.length()>0 )   
	        return new HandlerProxy(element, this, commandId);
	    return null;
	}

	/**
	 * @see net.refractions.udig.project.ui.internal.tool.display.ModalItem#getDefaultItem()
	 */
	protected ModalItem getDefaultItem() {
		return defaultItem;
	}

	/**
	 * @see net.refractions.udig.project.ui.internal.tool.display.ModalItem#getActiveItem()
	 */
	protected ModalItem getActiveItem() {
		return activeItem;
	}

	/**
	 * @see net.refractions.udig.project.ui.internal.tool.display.ModalItem#setActiveItem(net.refractions.udig.project.ui.internal.tool.display.ModalItem)
	 */
	protected void setActiveItem(ModalItem item) {
		activeItem=(ToolProxy) item;
	}
}