/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.project.ui.render.displayAdapter;

import java.awt.Point;

import net.refractions.udig.project.render.displayAdapter.IMapDisplay;

/**
 * Encapsulates a mouse event.  MapMouseListeners receive MapMouse events.
 * 
 * 
 * 
 * @author jeichar
 */
public class MapMouseEvent {
	
	/** Indicates that the alt key is down */
	public static final int ALT_DOWN_MASK = 1;

	/** Indicates that the ctrl key is down */
	public static final int CTRL_DOWN_MASK = 1 << 1;

	/** Indicates that the shift key is down */
	public static final int SHIFT_DOWN_MASK = 1 << 2;

	/** Indicates that the 1st mouse button, the left button on right handed mouses */
	public static final int BUTTON1 = 1 << 3;

	/** Indicates that the 2nd mouse button. */
	public static final int BUTTON2 = 1 << 4;

	/** Indicates that the 3rd mouse button. */
	public static final int BUTTON3 = 1 << 5;

	/** The Viewport pane that raised the event. */
	public final IMapDisplay source;

	/** Contains all the state of the event ORed together. */
	public final int state;

	/** indicates the button that last changed */
	public final int button;


	/** indicates the x position of the event */
	public final int x;

	/** indicates the y position of the event */
	public final int y;

	/**
	 * Construct <code>MapMouseEvent</code>.
	 * 
	 * @param source The object that raised the event
	 * @param state the state of the event ORed together
	 * @param x the x position of the event
	 * @param y the y position of the event
	 * @param button the button that last changed
	 */
	public MapMouseEvent(IMapDisplay source, int state, int x, int y, int button) {
		this.source = source;
		this.state = state;
		this.x = x;
		this.y = y;
		this.button = button;
	}

	/**
	 * Returns the location of the event.
	 * 
	 * @return the location of the event.
	 * @see Point
	 */
	public Point getPoint() {
		return new Point(x, y);
	}

	/**
	 * Returns true if shift key is down.
	 * 
	 * @return true if shift key is down.
	 */
	public boolean isShiftDown() {
		return (state & SHIFT_DOWN_MASK) != 0;
	}

	/**
	 * Returns true if control key is down.
	 * 
	 * @return true if control key is down.
	 */
	public boolean isControlDown() {
		return (state & CTRL_DOWN_MASK) != 0;
	}

	/**
	 * Returns true if alt key is down.
	 * 
	 * @return true if alt key is down.
	 */
	public boolean isAltDown() {
		return (state & ALT_DOWN_MASK) != 0;
	}

}
