 /*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004, Refractions Research Inc. This
 * library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the License. This library is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.render.internal.feature.basic;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.StyleBlackboard;
import net.refractions.udig.project.internal.render.ViewportModel;
import net.refractions.udig.project.internal.render.impl.RendererImpl;
import net.refractions.udig.project.internal.render.impl.Styling;
import net.refractions.udig.project.render.RenderException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.FeatureSource;
import org.geotools.feature.Feature;
import org.geotools.map.DefaultMapContext;
import org.geotools.map.DefaultMapLayer;
import org.geotools.map.MapContext;
import org.geotools.map.MapLayer;
import org.geotools.renderer.lite.LiteRenderer2;
import org.geotools.renderer.RenderListener;
import org.geotools.styling.Style;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;

/**
 * The default victim renderer. Based on the Lite-Renderer from Geotools.
 * 
 * @author Jesse Eichar
 * @version $Revision: 1.9 $
 */
public class BasicFeatureRenderer extends RendererImpl{

    LiteRenderer2 lite = null;

    private MapContext map = null;

    MapLayer[] layers = null;

    BasicRenderListener listener=new BasicRenderListener();
    
//    private static final int LAYER_SIZE_CUTOFF;
//    static{
//        //This is magic and should be a preference.
//        LAYER_SIZE_CUTOFF=(int) ((Runtime.getRuntime().maxMemory()/2)/10000);
//    }
    /**
     * 
     */
    public BasicFeatureRenderer() {
        // Enable logging if we in trace mode
        if (RendererPlugin.isDebugging( Trace.FINEST )){
            Logger.getLogger("org.geotools.rendering").setLevel( Level.FINE ); //$NON-NLS-1$
        }
    }
    /**
     * @see net.refractions.udig.project.internal.render.impl.RendererImpl#render(java.awt.Graphics2D, org.eclipse.core.runtime.IProgressMonitor)
     */
    public void render( Graphics2D destination, IProgressMonitor monitor ) throws RenderException {
    	render( destination, getContext().getViewportModel().getBounds(), monitor);
    }

    private final static int NOT_INITIALIZED=-2; 
    int count=NOT_INITIALIZED;
//    int count=-1;
//    private void setFeatureLoading(IProgressMonitor monitor) {
//        boolean preloading=false;
//        System.out.println(LAYER_SIZE_CUTOFF);
//        try {
//            FeatureSource source=getContext().getLayer().getResource(FeatureSource.class, monitor);
//            if( getContext().getLayer() instanceof SelectionLayer || (source instanceof FeatureStore && getContext().getLayer().isDirty()) ){
////                if( source instanceof FeatureStore && getContext().getEditManager().isEditing() ){
//                RendererPlugin.trace( getContext().getLayer().getName()+" features are not loaded into memory", null );
//            }else{
//                if( count==NOT_INITIALIZED)
//                    count=source.getCount(Query.ALL);
//                if( count ==-1 )
//                    RendererPlugin.trace( getContext().getLayer().getName()+" count=-1 therefore unknown number of features exists.", null);
//                else if( count < LAYER_SIZE_CUTOFF ){
//                    preloading=true;
//                    RendererPlugin.trace( getContext().getLayer().getName()+" features are being loaded into memory", null );
//                }else
//                    RendererPlugin.trace( getContext().getLayer().getName()+" features are not loaded into memory", null);
//            }
//        } catch (IOException e) {
//            // do nothing.
//        }
//        lite.setMemoryPreloadingEnabled(preloading);
//    }


    private void setQueries() {
            try{
                layers[0].setQuery(getContext().getFeatureQuery());
            }catch(Exception e){
                //do nothing.
            }
    }

    /**
     * does some additional initialization in preparation for drawing. It only needs to be done once so there is a quick
     * shortcircuit check in the beginning Obtains the features source, creates the MapLayer and Map context objects
     * required for Lite renderer and creates the lite renderer.
     * @throws IOException 
     */
    private void prepareDraw(IProgressMonitor monitor) throws IOException {
        //check for style information on the blackboard
        StyleBlackboard styleBlackboard = (StyleBlackboard) getContext().getLayer().getStyleBlackboard();
        FeatureSource featureSource = 
            getContext().getLayer().getResource(FeatureSource.class, null);
        
        //pull style information off the blackboard
        Style style = (Style) styleBlackboard.lookup(Style.class);
        if (style == null) {
            style = Styling.createLineStyle(featureSource.getSchema().getTypeName(),Color.BLUE);
        }
        
        if( layers!=null){
            layers[0].setStyle(style);
            return;
        }

        layers = new MapLayer[1];
        
        layers[0] = new DefaultMapLayer(featureSource, style, "Test"); //$NON-NLS-1$
        map = new DefaultMapContext(layers);
        

    }

    /**
     * @see net.refractions.udig.project.internal.render.impl.RendererImpl#dispose()
     */
    public void dispose() {
        if( lite!=null )
            lite.stopRendering();
    }

    /**
     * @see net.refractions.udig.project.internal.render.impl.RendererImpl#render(com.vividsolutions.jts.geom.Envelope, org.eclipse.core.runtime.IProgressMonitor)
     */
    public void render( Envelope bounds, IProgressMonitor monitor ) throws RenderException {
        render(getContext().getImage().createGraphics(), bounds, monitor);
    }
    
    /**
     * TODO summary sentence for render ...
     * 
     * @param graphics
     * @param bounds
     * @param monitor
     * @throws RenderException
     */
    private void render(Graphics2D graphics, Envelope bounds, IProgressMonitor monitor) throws RenderException{
        try {
        prepareDraw(monitor);
        
//        setFeatureLoading(monitor);
        
        Envelope validBounds=validateBounds(bounds, monitor);
        listener.init(monitor);        
        setQueries();
        Point min=getContext().worldToPixel(new Coordinate(validBounds.getMinX(), validBounds.getMinY()));
        Point max=getContext().worldToPixel(new Coordinate(validBounds.getMaxX(), validBounds.getMaxY()));
        int width=Math.abs(max.x-min.x);
        int height=Math.abs(max.y-min.y);
        if( height==0 || width==0)
            return;
        Rectangle paintArea = new Rectangle(Math.min(min.x, max.x),Math.min(min.y, max.y), width, height);

        map.setAreaOfInterest(validBounds, getContext().getViewportModel().getCRS());
        getContext().getLayer().setStatus( ILayer.WORKING );

        graphics.setClip(paintArea);
        
        getRenderer().paint(graphics, paintArea, validBounds);
        graphics.dispose();
        if ( listener.exception!=null ){
            RenderException e2=new RenderException("Exception(s) occured during rendering: "+listener.exception.getLocalizedMessage());
            e2.initCause(listener.exception);
            throw e2;
        }
        if ( !listener.featureRendered ){
          getContext().setStatus(ILayer.WARNING);
          getContext().setStatusMessage("No features rendered");            
        }
        } catch (Exception e1) {
            RenderException e2=new RenderException("Exception(s) occured during rendering: "+e1.getLocalizedMessage());
            e2.initCause(e1);
            throw e2;
        }
    }
    

    private LiteRenderer2 getRenderer() {
    	lite = new LiteRenderer2(map);
    lite.setOptimizedDataLoadingEnabled(true);
    lite.removeRenderListener(LiteRenderer2.DEFAULT_LISTENER);
    lite.addRenderListener(listener);
    lite.setConcatTransforms(false);
    lite.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		return lite;
	}

	private static final Envelope LATLONG_BOUNDS=new Envelope( -179.999,179.999,-89.99,89.99 );
    
    private Envelope validateBounds( Envelope bounds, IProgressMonitor monitor ) throws IOException {
        Envelope validBounds=bounds;
        if( validBounds==null ){
            validBounds=getContext().getViewportModel().getBounds();
        }
        CoordinateReferenceSystem crs=getContext().getViewportModel().getCRS();
        try {
            if( getContext().getMap().getBounds(monitor).getCRS()==null
                    || !getContext().getMap().getBounds(monitor).getCRS().equals(crs) )
                return validBounds;
            //Temporary measure until valid area works correctly.
        } catch (IOException e) {
            return validBounds;
        }
        if( crs.equals(ViewportModel.DEFAULT_CRS) ){
            if( LATLONG_BOUNDS.contains(validBounds) )
                return validBounds;
            return new Envelope( 
                    validBounds.getMinX()<LATLONG_BOUNDS.getMinX()?LATLONG_BOUNDS.getMinX():validBounds.getMinX(),
                            validBounds.getMaxX()>LATLONG_BOUNDS.getMaxX()?LATLONG_BOUNDS.getMaxX():validBounds.getMaxX(),
                                    validBounds.getMinY()<LATLONG_BOUNDS.getMinY()?LATLONG_BOUNDS.getMinY():validBounds.getMinY(),
                                            validBounds.getMaxY()>LATLONG_BOUNDS.getMaxY()?LATLONG_BOUNDS.getMaxY():validBounds.getMaxY());
        }
        return validBounds;
    }
    

    private class BasicRenderListener implements RenderListener{

        IProgressMonitor monitor;
        Exception exception;
        int exceptionCount=0;
        boolean featureRendered=false;
        
        long lastUpdate;
        private static final int UPDATE_INTERVAL=3000;
        /**
         * @see org.geotools.renderer.lite.RenderListener#featureRenderer(org.geotools.feature.Feature)
         */
        public void featureRenderer( Feature feature ) {
            if( !featureRendered )
                featureRendered=true;
            if( monitor.isCanceled() )
                lite.stopRendering();
            long current=System.currentTimeMillis();
            if( current-lastUpdate > UPDATE_INTERVAL){
            	lastUpdate=current;
            	setState(RENDERING);
            }
        }

        /**
         * @see org.geotools.renderer.lite.RenderListener#errorOccurred(java.lang.Exception)
         */
        public void errorOccurred( Exception e ) {
            if( exceptionCount>50000)
                lite.stopRendering();
            exception=e;
            exceptionCount++;
        }

        /**
         * Initialize listener
         * 
         * @param monitor
         */
        public void init(IProgressMonitor monitor){
        	lastUpdate=System.currentTimeMillis();
            this.monitor=monitor;
            exception=null;
            exceptionCount=0;
            featureRendered=false;
        }
        
    }
}