/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.render.internal.gridcoverage.basic;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.io.IOException;

import net.refractions.udig.project.internal.StyleBlackboard;
import net.refractions.udig.project.internal.render.impl.RendererImpl;
import net.refractions.udig.project.internal.render.impl.Styling;

import org.eclipse.core.runtime.IProgressMonitor;
import org.geotools.data.DataUtilities;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureType;
import org.geotools.map.DefaultMapContext;
import org.geotools.renderer.lite.LiteRenderer2;
import org.geotools.styling.Style;
import org.opengis.coverage.grid.GridCoverage;

import com.vividsolutions.jts.geom.Envelope;

/**
 * DOCUMENT ME!
 * 
 * @author Jesse Eichar
 * @version $Revision: 1.9 $
 */
public class BasicGridCoverageRenderer extends RendererImpl {

    private LiteRenderer2 lite;
    private DefaultMapContext mapContext;
    private int steps = 5;
    private static final int MAX_ATTEMPTS = 5;

    public void render( Graphics2D graphics, IProgressMonitor monitor) {
        try {
            prepareRender();
        } catch (IOException e1) {
            RendererPlugin.log(e1);
        }
        setStyle();
        mapContext.setAreaOfInterest(getContext().getViewportModel().getBounds(), getContext().getViewportModel().getCRS());
        Envelope bbox = getContext().getViewportModel().getBounds();
        Rectangle displayArea = new Rectangle(getContext().getMapDisplay().getWidth(), getContext()
                .getMapDisplay().getHeight());
        AffineTransform worldToScreen = lite.worldToScreenTransform(bbox, displayArea);

        lite.paint(graphics, displayArea, worldToScreen);
//        int xTileDisplay = getContext().getMapDisplay().getWidth() / steps;
//        int lastXTileDisplay = getContext().getMapDisplay().getWidth() - xTileDisplay * (steps - 1);
//        int yTileDisplay = getContext().getMapDisplay().getHeight() / steps;
//        int lastYTileDisplay = getContext().getMapDisplay().getHeight() - yTileDisplay * (steps - 1);
//
//        int attempts = 0;
//
//        for( int j = 0; j < steps; j++ ) {
//            int y = j * yTileDisplay;
//            int height;
//            if (j < steps - 1)
//                height = yTileDisplay;
//            else
//                height = lastYTileDisplay;
//            for( int i = 0; i < steps; i++ ) {
//                if (getState() == NEVER)
//                    return;
//                int x = i * xTileDisplay;
//                int width;
//                if (i < steps - 1)
//                    width = xTileDisplay;
//                else
//                    width = lastXTileDisplay;
//
//                try {
//                    Rectangle paintArea = new Rectangle(x, y, width, height);
//                    graphics.setClip(x, y, width, height);
//                    lite.paint(graphics, paintArea, worldToScreen);
//                    setState(RENDERING);
//                } catch (Throwable e) {
//                    if (attempts == MAX_ATTEMPTS) {
//                        Display.getDefault().asyncExec(new Runnable(){
//
//                            public void run() {
//                                MessageDialog
//                                        .openError(
//                                                Display.getDefault().getActiveShell(),
//                                                Policy.bind("BasicGridCoverageRenderer.error.title"), //$NON-NLS-1$
//                                                Policy.bind("BasicGridCoverageRenderer.error.message") //$NON-NLS-1$
//                                                        );
//                            }
//                        });
//                        return;
//                    }
//                    i--;
//                    attempts++;
//                    // TODO
//                    if (e instanceof Exception)
//                        RendererPlugin.log((Exception) e);
//                }
//            }
//        }
    }

    /**
     * TODO summary sentence for setStyle ...
     */
    private void setStyle() {
        StyleBlackboard styleBlackboard = 
            (StyleBlackboard) getContext().getLayer().getStyleBlackboard();
    	Style style = (Style) styleBlackboard.lookup(Style.class);
    	if (style != null) {
            mapContext.getLayer(0).setStyle(style);
        }
    }
    /**
     * TODO summary sentence for prepareRender ...
     */
    private void prepareRender() throws IOException {
        if (lite == null) {
            GridCoverage coverage = getContext().getGeoResource()
                    .resolve(GridCoverage.class, null);

     
            /*
            FeatureCollection collection = FeatureCollections.newCollection();
            collection.add(createGCFeatureWrapper(coverage));
            */
            mapContext = new DefaultMapContext();
            mapContext.addLayer(coverage, Styling.createRasterStyle(TYPENAME));
            lite = new LiteRenderer2(mapContext);
        }
    }
    private static final String TYPENAME = "GridCoverage"; //$NON-NLS-1$

    public void stopRendering() {
        setState(STATE_EDEFAULT);
    }

    public void dispose() {
        //TODO
    }
    /*
    public InfoList getInfo( Point screenLocation ) throws IOException {
        InfoList infos = new InfoList(screenLocation);
        infos.add(LayerPointInfo.NOT_FOUND);
        return infos;
    }
    */
    private Feature createGCFeatureWrapper( GridCoverage gc ) {
        try {
            FeatureType type = DataUtilities.createType(TYPENAME,
                    "grid:org.geotools.gc.GridCoverage"); //$NON-NLS-1$
            return type.create(new Object[]{gc});
        } catch (Exception e) {
            return null;
        }
    }

    public void render( Envelope bounds, IProgressMonitor monitor ) {
        render(getContext().getImage().createGraphics(), monitor);
    }


}
