/*
 *    uDig - User Friendly Desktop Internet GIS client
 *    http://udig.refractions.net
 *    (C) 2004, Refractions Research Inc.
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation;
 *    version 2.1 of the License.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 */
package net.refractions.udig.render.internal.wms.basic;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.NoRouteToHostException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.imageio.ImageIO;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.internal.render.impl.RendererImpl;
import net.refractions.udig.project.internal.render.impl.Styling;
import net.refractions.udig.project.render.ICompositeRenderContext;
import net.refractions.udig.project.render.IMultiLayerRenderer;
import net.refractions.udig.project.render.IRenderContext;
import net.refractions.udig.project.render.RenderException;
import net.refractions.udig.render.wms.basic.WMSPlugin;
import net.refractions.udig.render.wms.basic.preferences.PreferenceConstants;
import net.refractions.udig.style.sld.SLDContent;
import net.refractions.udig.ui.graphics.SLDs;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.preference.IPreferenceStore;
import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.coverage.grid.GridCoverageFactory;
import org.geotools.coverage.grid.GridGeometry2D;
import org.geotools.coverage.processing.GridCoverageProcessor2D;
import org.geotools.data.ows.Layer;
import org.geotools.data.wms.WebMapServer;
import org.geotools.data.wms.request.GetMapRequest;
import org.geotools.data.wms.response.GetMapResponse;
import org.geotools.factory.Hints;
import org.geotools.feature.AttributeType;
import org.geotools.feature.AttributeTypeFactory;
import org.geotools.feature.Feature;
import org.geotools.feature.FeatureCollection;
import org.geotools.feature.FeatureCollections;
import org.geotools.feature.FeatureType;
import org.geotools.feature.FeatureTypeFactory;
import org.geotools.feature.IllegalAttributeException;
import org.geotools.feature.SchemaException;
import org.geotools.geometry.GeneralEnvelope;
import org.geotools.geometry.JTS;
import org.geotools.geometry.JTS.ReferencedEnvelope;
import org.geotools.map.DefaultMapContext;
import org.geotools.map.DefaultMapLayer;
import org.geotools.map.MapContext;
import org.geotools.ows.ServiceException;
import org.geotools.referencing.CRS;
import org.geotools.renderer.lite.LiteRenderer2;
import org.geotools.styling.Style;
import org.opengis.coverage.grid.GridCoverage;
import org.opengis.metadata.Identifier;
import org.opengis.referencing.FactoryException;
import org.opengis.referencing.NoSuchAuthorityCodeException;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.TransformException;
import org.opengis.spatialschema.geometry.MismatchedDimensionException;
import org.xml.sax.SAXException;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.CoordinateSequenceFactory;
import com.vividsolutions.jts.geom.DefaultCoordinateSequenceFactory;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LinearRing;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.geom.PrecisionModel;

public class BasicWMSRenderer2 extends RendererImpl implements IMultiLayerRenderer {

    /*
     * Cached objects
     */
    private Point offset;
    private boolean clientSideReprojection; //indicates that reprojection is required
    
    private IPreferenceStore store = WMSPlugin.getDefault().getPreferenceStore();
   
    public BasicWMSRenderer2() {
        super();
    }

    @Override
    public void render( Graphics2D destination, IProgressMonitor monitor ) throws RenderException {
        
        clientSideReprojection = false;
        
        setLayerStatus( ILayer.WAIT );
        try {
                        
//CoordinateReferenceSystem epsg4269 = CRS.decode("EPSG:4269");
//CoordinateReferenceSystem epsg42101 = CRS.decode("EPSG:42101");
//CoordinateReferenceSystem epsg42102 = CRS.decode("EPSG:42102");
//
//try {
//    Envelope llbbox = JTS.transform(getViewportBBox(), CRS.transform(getViewportCRS(), epsg4269));
//    System.out.println("BBOX 4326: "+llbbox);
//    Envelope bbox42101 = JTS.transform(llbbox, CRS.transform(epsg4269, epsg42101));
//    System.out.println("BBOX 42101: "+bbox42101);
//    
//    Envelope bbox42102 = JTS.transform(bbox42101, CRS.transform(epsg42101, epsg42102));
//    System.out.println("BBOX 42102: "+bbox42102);
//    
//} catch (TransformException e) {
//    e.printStackTrace();
//} catch (FactoryException e) {
//    e.printStackTrace();
//}
            
            if (!supportsCRS(getViewportCRS())) {
                clientSideReprojection = true;
            }
            
            if (outsideBounds()) {
                setLayerStatus(ILayer.WARNING);
                WMSPlugin.log("Bounds of the data are outside the bounds of the viewscreen.");
                return;
            }
            
            GetMapRequest request = getWMS().createGetMapRequest();
            
            request.setExceptions(GetMapRequest.EXCEPTION_XML);
            request.setTransparent(true);

            List formats = Arrays.asList(getWMS().getCapabilities().getRequest().getGetMap().getFormatStrings());
            String str;
            if(store.getBoolean(PreferenceConstants.P_USE_DEFAULT_ORDER)){
               str = store.getDefaultString(PreferenceConstants.P_IMAGE_TYPE_ORDER);
            }else {
               str = store.getString(PreferenceConstants.P_IMAGE_TYPE_ORDER);
            }
            String[] preferredFormats = str.split(",");
            // Select one of the available formats from the WMS server
            // the order of preferred formats is set in the preferences
            for(String format : preferredFormats){
               if(formats.contains(format)) { 
                  request.setProperty(GetMapRequest.FORMAT, format);
                  break;
               }
            }
//            if (formats.contains("image/png")) { //$NON-NLS-1$
//                request.setProperty(GetMapRequest.FORMAT, "image/png"); //$NON-NLS-1$
//            } else if (formats.contains("image/gif")) { //$NON-NLS-1$
//                request.setProperty(GetMapRequest.FORMAT, "image/gif"); //$NON-NLS-1$
//            } else if (formats.contains("image/jpeg")) { //$NON-NLS-1$
//                request.setProperty(GetMapRequest.FORMAT, "image/jpeg"); //$NON-NLS-1$
//            } else if (formats.contains("image/bmp")) { //$NON-NLS-1$
//                request.setProperty(GetMapRequest.FORMAT, "image/bmp"); //$NON-NLS-1$
//            }
            
            for(int i = getLayers().size()-1; i >= 0; i--) {
                ILayer ilayer = getLayers().get(i);
                Layer layer = ilayer.getResource(org.geotools.data.ows.Layer.class, null);
                request.addLayer(layer);
            }

            Dimension dimension = calculateImageDimensions();
            request.setDimensions(dimension.width+"", dimension.height+""); //$NON-NLS-1$ //$NON-NLS-2$
            
            ReferencedEnvelope requestBBox = calculateRequestBoundingBox(
                    clientSideReprojection,
                    true,
                    getViewportBBox(),
                    getViewportCRS(),
                    getWMSLayers(),
                    getWMS());
            request.setBBox(requestBBox.getMinX()
                    + "," + requestBBox.getMinY()  //$NON-NLS-1$
                    + "," + requestBBox.getMaxX()  //$NON-NLS-1$
                    + "," + requestBBox.getMaxY());  //$NON-NLS-1$
            
            request.setSRS(requestBBox.getCRS().getIdentifiers().iterator().next().toString());
            
            request.setProperty("DACS_ACS", "-check_fail%20-format%20XMLSCHEMA");
            
            setLayerStatus( ILayer.WORKING );
            GetMapResponse response = getWMS().issueRequest(request);
            
            BufferedImage image = null;
            
            
            //TODO Make Dacshandler global
//            DACSHandler handler = new DACSHandler();
//            
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            int b;
            while ((b = response.getInputStream().read()) != -1) {
            	baos.write(b);
            }
            
            ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
            System.out.println(bais.markSupported());
//            
//            boolean result2 = handler.validate(bais);
//            bais.reset();
            //TODO change to loop to allow multiple requests
            
            image = ImageIO.read( bais );
            
            if (image == null) {
                
//                if (result == false) {
//                	throw new RenderException("Unable to validate");
//                }
                
                Exception e = new RuntimeException("Unable to decode image returned from the server.");
                throw wrapException(e);
            }
                        
            float alpha = 1.0f;
            
            Style style = (Style) getLayers().get(0).getStyleBlackboard().get(SLDContent.ID);
            double result = SLDs.rasterOpacity(SLDs.rasterSymbolizer(style));
                        
            if (!Double.isNaN(result)) {
                alpha = (float) result;
            }
            
//                destination.setComposite(AlphaComposite.getInstance(
//                        AlphaComposite.SRC_OVER, alpha));
//                destination.drawRenderedImage(image, getAffineTransform());
            
            Envelope env = requestBBox;
            GeneralEnvelope gtEnv = new GeneralEnvelope(new double[] { env.getMinX(), env.getMinY() } ,
                    new double[] { env.getMaxX(), env.getMaxY() } ); 

//System.out.println(image.getColorModel().getNumColorComponents());
          
            GridCoverageFactory factory = new GridCoverageFactory();
            GridCoverage2D gc = (GridCoverage2D) factory.create(
                    "GridCoverage", 
                    image, 
                    CRS.decode(findRequestCRS(getWMSLayers())), 
                    gtEnv,
                    null,
                    null,
                    null);

            GridCoverage2D projected = gc;
            if (clientSideReprojection) {
                
                projected = projectTo(gc, getViewportCRS(), null);
//System.out.println(projected.getCoordinateReferenceSystem().toString());
            }
            
            Rectangle paintArea = new Rectangle(getOffset().x, getOffset().y, dimension.width, dimension.height);
            
            MapContext mapContext = new DefaultMapContext();
            
            Feature feature = wrapGcInFeature(projected == null ? gc : projected);
            
            FeatureCollection fc = FeatureCollections.newCollection();
            
            fc.add(feature);
            
            DefaultMapLayer mapLayer = new DefaultMapLayer(fc, style == null ? Styling.createRasterStyle("GridCoverage") : style);
                
            mapContext.addLayer(mapLayer);
            mapContext.setAreaOfInterest(getViewportBBox(), getViewportCRS());
            LiteRenderer2 lite = new LiteRenderer2(mapContext);
                        
            Envelope projEnv = new Envelope(
                    projected.getEnvelope().getMinimum(0),
                    projected.getEnvelope().getMaximum(0),
                    projected.getEnvelope().getMinimum(1),
                    projected.getEnvelope().getMaximum(1));
            
            Envelope imageAreaBBox = getViewportBBox();
            if (getViewportBBox().contains(projEnv)) {
                imageAreaBBox = projEnv;
            }
            lite.setConcatTransforms(true);
            lite.paint(destination, paintArea, imageAreaBBox);
                        
        } catch (UnknownHostException e) {
            throw wrapException(e);
        } catch (FileNotFoundException e) {
            throw wrapException(e);
        } catch (NoRouteToHostException e) {
            throw wrapException(e);
        } catch (IOException e) {
            throw wrapException(e);
        } catch (ServiceException e) {
            throw wrapException(e);
        } catch (SAXException e) {
            throw wrapException(e);
        } catch (MismatchedDimensionException e) {
            throw wrapException(e);// TODO Catch e
        } catch (NoSuchAuthorityCodeException e) {
            throw wrapException(e);
        } catch (IllegalAttributeException e) {
            throw wrapException(e);
        } catch (SchemaException e) {
            throw wrapException(e);
        }
        setLayerStatus(ILayer.DONE);
    }

    @Override
    public void render( Envelope bounds, IProgressMonitor monitor ) throws RenderException {
        Graphics2D graphics = (Graphics2D) getContext().getImage().getGraphics();                
        render(graphics, monitor); 
    }

    public String getEPSGCode() {
        return getViewportCRS().getIdentifiers().iterator().next().toString();
    }
    
    public AffineTransform getAffineTransform() {
        AffineTransform affineTransform = new AffineTransform();
        
        affineTransform = new AffineTransform();
        affineTransform.translate(getOffset().x, getOffset().y);
        
        return affineTransform;
    }
    
    public Point getOffset() {
        return this.offset;
    }
    
    public Dimension calculateImageDimensions() throws IOException {
        int width = getContext().getMapDisplay().getDisplaySize().width;
        int height = getContext().getMapDisplay().getDisplaySize().height;
        
        Envelope layersBBox = calculateRequestBoundingBox(
                clientSideReprojection, 
                false,
                getViewportBBox(),
                getViewportCRS(),
                getWMSLayers(),
                getWMS()
                );
        
        Point min = getContext().worldToPixel(new Coordinate(layersBBox.getMinX(), layersBBox.getMinY()));
        Point max = getContext().worldToPixel(new Coordinate(layersBBox.getMaxX(), layersBBox.getMaxY()));
        width  = Math.max(max.x, min.x) - Math.min(max.x, min.x);
        height = Math.max(max.y, min.y) - Math.min(max.y, min.y);
                
        this.offset = new Point(Math.min(min.x, max.x), max.y);

        return new Dimension(width, height);
    }
    
    public  ReferencedEnvelope calculateRequestBoundingBox(
            boolean clientSideReprojection,
            boolean allowReprojection,
            Envelope viewportBBox,
            CoordinateReferenceSystem viewportCRS,
            List<Layer> wmsLayers, 
            WebMapServer wms) throws IOException {
        
        if (viewportBBox == null) {
            viewportBBox = new Envelope(-180, 180, -90, 90);
        }
        
        Envelope layersBBox = getLayersBoundingBox(viewportCRS, wmsLayers, wms);
        
        double minx,miny,maxx,maxy;
        minx = layersBBox.getMinX();
        maxx = layersBBox.getMaxX();
        miny = layersBBox.getMinY();
        maxy = layersBBox.getMaxY();
        
        if (!viewportBBox.contains(layersBBox.getMinX(), layersBBox.getMinY())) {
            minx = viewportBBox.getMinX();
            miny = viewportBBox.getMinY();
        }
        
        if (!viewportBBox.contains(layersBBox.getMinX(), layersBBox.getMaxY())) {
            minx = viewportBBox.getMinX();
            maxy = viewportBBox.getMaxY();
        }
        
        if (!viewportBBox.contains(layersBBox.getMaxX(), layersBBox.getMinY())) {
            maxx = viewportBBox.getMaxX();
            miny = viewportBBox.getMinY();
        }
        
        if (!viewportBBox.contains(layersBBox.getMaxX(), layersBBox.getMaxY())) {
            maxx = viewportBBox.getMaxX();
            maxy = viewportBBox.getMaxY();
        }
        
        ReferencedEnvelope clippedBBox = new JTS.ReferencedEnvelope(new Envelope(minx, maxx, miny, maxy), viewportCRS);
        
        if (clientSideReprojection && allowReprojection) {
            //Convert the clipped bounding box to the request CRS. This is the 
            //BBox to be used in the request.
            
            try {
                CoordinateReferenceSystem crs = CRS.decode(findRequestCRS(wmsLayers));
                
                clippedBBox = new ReferencedEnvelope(JTS.transform(clippedBBox, CRS.transform(viewportCRS, crs, true), 4), crs);
//                
//                minx = clippedBBox.getMinX();
//                miny = clippedBBox.getMinY();
//                maxx = clippedBBox.getMaxX();
//                maxy = clippedBBox.getMaxY();
//                
//                DirectPosition newLower = transform.transform(new DirectPosition2D(minx, miny),null);
//                DirectPosition newUpper = transform.transform(new DirectPosition2D(maxx, maxy),null);
//                
//                clippedBBox = new Envelope(
//                        newLower.getOrdinate(0),
//                        newUpper.getOrdinate(0),
//                        newLower.getOrdinate(1),
//                        newUpper.getOrdinate(1));
                
            } catch (NoSuchAuthorityCodeException e) {
                WMSPlugin.log(e.getLocalizedMessage(), e);
                return null;
            } catch (IOException e) {
                WMSPlugin.log(e.getLocalizedMessage(), e);
                return null;
            } catch (FactoryException e) {
                WMSPlugin.log(e.getLocalizedMessage(), e);
                return null;
            } catch (MismatchedDimensionException e) {
                WMSPlugin.log(e.getLocalizedMessage(), e);
                return null;
            } catch (TransformException e) {
                e.printStackTrace();
                WMSPlugin.log(e.getLocalizedMessage(), e);
                return null;
            }                        
        } 
        
        return clippedBBox;
    }

    public Envelope getLayersBoundingBox(CoordinateReferenceSystem crs,
            List<Layer> layers, 
            WebMapServer wms) throws IOException {
        Envelope envelope = null;
                
        for (Layer layer : layers) {
        	String name2=layer.getName() ;
        	
        	//XXX This is a disaster waiting to happen.  If the crs does not have any identifiers
        	// then BOOOM big exception...  We need a better way of finding the EPSG code.
        	String crsName = crs.getIdentifiers().iterator().next().toString();
            System.out.println(name2+ crsName);
            GeneralEnvelope temp = wms.getEnvelope(
                    layer, 
                    crs);
            
            if (temp != null) {
                Envelope jtsTemp = new Envelope(
                        temp.getMinimum(0), 
                        temp.getMaximum(0),
                        temp.getMinimum(1), 
                        temp.getMaximum(1));
                if (envelope == null) {
                    envelope = jtsTemp; 
                } else {
                    envelope.expandToInclude(jtsTemp);
                }
            } else {
                return null;
            }
        }
        
        return envelope;
    }
//        
//        if (clientSideReprojection) {
//            //1. Find a CRS that all the layers can agree on. (If we can't, the 
//            //renderer is in an illegal state.) This will be the requestCRS
//            
//            String requestCRS = findRequestCRS();
//            
//            //2. Calculate the bounding box of all the layers in the requestCRS
//            
//            CoordinateReferenceSystem crs = null;
//            try {
//                crs = CRS.decode(requestCRS);
//            } catch (NoSuchAuthorityCodeException e) {
//                WMSPlugin.log("ERROR: findRequestCRS returned an EPSG code that cannot be used by CRS class.", e);
//                return null;
//            }
//            
//            envelope = null;
//            for (ILayer ilayer : getLayers()) {
//                Layer layer = ilayer.getResource(Layer.class, null);
//                GeneralEnvelope temp = getWMS().getEnvelope(layer, crs);
//                
//                if (temp != null) {
//                    Envelope jtsTemp = new Envelope(
//                            temp.getMinimum(0), 
//                            temp.getMaximum(0),
//                            temp.getMinimum(1), 
//                            temp.getMaximum(1));
//                    if (envelope == null) {
//                        envelope = jtsTemp; 
//                    } else {                Layer layer = ilayer.getResource(Layer.class, null);
//                        envelope.expandToInclude(jtsTemp);
//                    }
//                } else {
//                    WMSPlugin.log("WARNING: Unable to reproject bounding box properly.");
//                    return null;
//                }
//            }
//            
//            //3. Convert the request bounding box to the viewport CRS (and return it)
//            
//            MathTransform transform = null;
//            try {
//                transform = CRS.transform(crs, getContext().getViewportModel().getCRS());
//            } catch (FactoryException e) {
//                WMSPlugin.log("WARNING: Unable to reproject bounding box properly.", e);
//                return null;
//            }
//
//            double minx = envelope.getMinX();
//            double miny = envelope.getMinY();
//            double maxx = envelope.getMaxX();
//            double maxy = envelope.getMaxY();
//            
//            DirectPosition newLower = null;
//            DirectPosition newUpper = null;
//            try {
//                newLower = transform.transform(new DirectPosition2D(minx, miny),null);
//                newUpper = transform.transform(new DirectPosition2D(maxx, maxy),null);
//            } catch (MismatchedDimensionException e) {
//                WMSPlugin.log("WARNING: Unable to reproject bounding box properly.", e);
//                return null;
//            } catch (TransformException e) {
//                WMSPlugin.log("WARNING: Unable to reproject bounding box properly.", e);
//                return null;
//            }
//            
//            envelope = new Envelope(
//                    newLower.getOrdinate(0),
//                    newUpper.getOrdinate(0),
//                    newLower.getOrdinate(1),
//                    newUpper.getOrdinate(1));
//        }
//        
//        return envelope;
//    }
    
    public boolean supportsCRS(CoordinateReferenceSystem crs) throws IOException {
        boolean result = false;
        
        EPSG_CODE: for (Identifier id : crs.getIdentifiers() ) {
            String epsgCode = id.toString();
            for (ILayer ilayer : getLayers() ) {
                Layer layer = ilayer.getResource(Layer.class, null);
                
                if (!layer.getSrs().contains(epsgCode)) {
                    continue EPSG_CODE;
                }
            }
            return true;            
        }
        
        return result;
    }
    
    public String findRequestCRS(List<Layer> layers) throws IOException {
        
        String requestCRS = null;
        
        if (layers == null || layers.isEmpty()) {
            return null;
        }
        
        boolean foundLatLon = true;
        for ( Layer layer : layers ) { 
            if (!layer.getSrs().contains("EPSG:4326")) {
                foundLatLon = false;
                break;
            }
        }
        
        if (foundLatLon) {
            return "EPSG:4326";
        }
        
        foundLatLon = true;
        for ( Layer layer : layers ) { 
            if (!layer.getSrs().contains("EPSG:4269")) {
                foundLatLon = false;
                break;
            }
        }
        
        if (foundLatLon) {
            return "EPSG:4269";
        }
        
        Layer firstLayer = layers.get(0);
        for ( Object object : firstLayer.getSrs() ) {
            String epsgCode = (String) object;
            
            boolean canUseThisEPSG = true;
            
            try {
                //Check to see if *we* can actually use this code first.
                CRS.decode(epsgCode);
            } catch (NoSuchAuthorityCodeException e) {
                canUseThisEPSG = false;
                continue;
            }
            
            for (Layer layer : layers) {
               
                if (!layer.getSrs().contains(epsgCode)) {
                    canUseThisEPSG = false;
                }
            }
            
            if (canUseThisEPSG) {
                requestCRS = epsgCode;
                return requestCRS;
            }            
        }
        
        if (requestCRS == null) {
            //Hmm. Our layers have no SRS in common - we are in an illegal state
            WMSPlugin.log("ERROR: Illegal State: Basic WMS Renderer contains layers with no common CRS. Unable to perform request.");
            return null;
        }
        return requestCRS;
    }
    
    /**
     * @return true if the layer's extent is outside the viewport's extent
     * @throws IOException 
     */
    public boolean outsideBounds() throws IOException {
        return false;
//        return (!getViewportBBox().intersects(getLayersBoundingBox(getViewportCRS())) 
//             && !getViewportBBox().contains(getLayersBoundingBox(getViewportCRS())));
    }
    
    public List<ILayer> getLayers() {
        List<ILayer> layers = new ArrayList<ILayer>();
        
        ICompositeRenderContext context1= getContext();
        if (context1 == null) {
        	WMSPlugin.log("WARNING: WMS Renderer's getContext() is null");
        	return null;
        }
        if (context1.getContexts() == null) {
        	WMSPlugin.log("WARNING: WMS Renderer's context1.getContexts() is null");
        	return null;
        }
        IRenderContext[] contexts=context1.getContexts().toArray(new IRenderContext[context1.getContexts().size()]);
        Arrays.sort(contexts);
        for( IRenderContext renderContext: contexts) {
            if( renderContext.getLayer().isVisible() ) {
                layers.add(renderContext.getLayer());
            }
        }
        
        if (layers.isEmpty()) {
            WMSPlugin.log("WARNING: WMS Renderer contains no layers.");
        }
        
        return layers;
    }
    
    public List<Layer> getWMSLayers() throws IOException {
        List<Layer> layers = new ArrayList<Layer>();
        
        for (ILayer iLayer : getLayers()) {
            Layer layer = iLayer.getResource(Layer.class ,null);
            layers.add(layer);
        }
        
        return layers;
    }
    
    public WebMapServer getWMS() throws IOException {
        return getContext().getLayer().getResource(WebMapServer.class, null);
    }

    @Override
    public void dispose() {
    }

    Job refreshJob=new Job("RefreshJob"){
        protected IStatus run( IProgressMonitor monitor ) {
            getContext().clearImage();            
            try {
                render(getContext().getImage().createGraphics(), monitor);
            } catch (Throwable e) {
                WMSPlugin.log( e.getLocalizedMessage(), e);
            }
            return Status.OK_STATUS;
        }
        
    };
    public void refreshImage() throws RenderException {
        if( needRefresh() ){            
            refreshJob.schedule();
            try {
                refreshJob.join();
            } catch (InterruptedException e) {
                // TODO Catch e
            }
        }
    }

    protected boolean needRefresh() {
        return false; // TODO 
    }
    
    public RenderException wrapException(Throwable t) {
        setLayerStatus( ILayer.ERROR );
        RenderException renderException = new RenderException(t.getLocalizedMessage());
        renderException.initCause(t);
        return renderException;
    }
    
    public ICompositeRenderContext getContext() {
        return (ICompositeRenderContext) super.getContext();
    }
    
    protected void setLayerStatus( int layerStatus ){
        for( ILayer layer : getLayers() ){
            layer.setStatus( layerStatus );    
        }
    }  
    
    public CoordinateReferenceSystem getViewportCRS () {
        return getContext().getViewportModel().getCRS(); 
    }
    
    public Envelope getViewportBBox () {
        return getContext().getViewportModel().getBounds();
    }
    
    private Feature wrapGcInFeature(GridCoverage gridCoverage)
    throws IllegalAttributeException, SchemaException {
        // create surrounding polygon
        PrecisionModel pm = new PrecisionModel();
        CoordinateSequenceFactory csf = DefaultCoordinateSequenceFactory.instance();
        GeometryFactory gf = new GeometryFactory(pm, 0);
        Coordinate[] coord = new Coordinate[5];
        Rectangle2D rect = ((GridCoverage2D) gridCoverage).getEnvelope2D().getBounds2D();
        coord[0] = new Coordinate(rect.getMinX(), rect.getMinY());
        coord[1] = new Coordinate(rect.getMaxX(), rect.getMinY());
        coord[2] = new Coordinate(rect.getMaxX(), rect.getMaxY());
        coord[3] = new Coordinate(rect.getMinX(), rect.getMaxY());
        coord[4] = new Coordinate(rect.getMinX(), rect.getMinY());
        
        LinearRing ring = new LinearRing(csf.create(coord), gf);
        Polygon bounds = new Polygon(ring, null, gf);
        
        // create the feature type
        AttributeType geom = AttributeTypeFactory.newAttributeType("geom", Polygon.class);
        AttributeType grid = AttributeTypeFactory.newAttributeType("grid", GridCoverage.class);
        
        FeatureType schema = null;
        AttributeType[] attTypes = {geom, grid};
        
        schema = FeatureTypeFactory.newFeatureType(attTypes, "GridCoverage");
        
        // create the feature
        Feature feature = schema.create(new Object[] {bounds, gridCoverage});
        
        return feature;
    }
    
    //Adapted from Resample2DTest in GeoTools/main
    private GridCoverage2D projectTo(
            final GridCoverage2D coverage,
            final CoordinateReferenceSystem crs,
            final GridGeometry2D       geometry)
    {
        RenderingHints hints = new RenderingHints(Hints.LENIENT_DATUM_SHIFT, Boolean.TRUE);
        final GridCoverageProcessor2D processor = new GridCoverageProcessor2D(GridCoverageProcessor2D.getDefault(), hints);
        final String arg1; final Object value1;
        final String arg2; final Object value2;
        if (crs != null) {
            arg1="CoordinateReferenceSystem"; value1=crs;
            if (geometry != null) {
                arg2="GridGeometry"; value2=geometry;
            } else {
//                arg2="InterpolationType"; value2="NearestNeighbor";
            }
        } else {
            arg1="GridGeometry";      value1=geometry;
//            arg2="InterpolationType"; value2="NearestNeighbor";
        }
        GridCoverage2D projected = coverage.geophysics(true);
        projected = (GridCoverage2D) processor.doOperation("Resample", projected, arg1, value1);

        projected = projected.geophysics(false);

//System.out.println(projected.getEnvelope());
        return projected;
    }
}
