/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.tools.edit;

import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.command.MapCommand;
import net.refractions.udig.project.ui.internal.commands.draw.DrawFeatureCommand;
import net.refractions.udig.project.ui.internal.commands.draw.DrawShapeCommand;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.tool.AbstractModalTool;
import net.refractions.udig.project.ui.tool.ModalTool;

import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;
import org.geotools.data.FeatureReader;
import org.geotools.data.FeatureResults;
import org.geotools.feature.Feature;
import org.geotools.geometry.JTS.ReferencedEnvelope;

import com.vividsolutions.jts.geom.Envelope;

/**
 * Deletes a feature from the currently selected layer or the top layer.
 * 
 * @author Jesse
 * @since 0.8.1
 */
public class DeleteTool extends AbstractModalTool implements ModalTool {

    /**
     * Construct <code>DeleteTool</code>.
     *
     */
    public DeleteTool() {
        super(MOUSE|MOTION);
    }

    
    @Override
    public void setActive(final boolean active) {
    	super.setActive(active);
    	setStatusBarMessage(active);
    }

	private void setStatusBarMessage(final boolean active) {
		getContext().updateUI(new Runnable() {
			public void run() {
				IStatusLineManager bar = getContext().getStatusBar();
				if( bar!=null ){
					if( active ){
						if( getContext().getMapLayers().size()>0 )
							bar.setMessage(Policy.bind("DeleteTool.status")); //$NON-NLS-1$
					}else
						bar.setMessage(""); //$NON-NLS-1$
				}
			}
		});
	}
	
    
    @Override
    public void mousePressed( MapMouseEvent e ) {
        draw.setValid( true ); // make sure context.getViewportPane().repaint() knows about us        
        context.sendASyncCommand( draw ); // should of isValided us       
        feedback( e );
                
    }
    @Override
    public void mouseDragged( MapMouseEvent e ) {
        feedback( e );
        
    }
    
    DrawShapeCommand draw = new DrawShapeCommand();
    
    /**
     * Provides user feedback
     * @param e 
     */
    public void feedback( MapMouseEvent e ) {        
        ReferencedEnvelope box = context.getBoundingBox( e.getPoint(), 6 );
        draw.setShape( context.toShape( box ) );        
        context.getViewportPane().repaint();
        
        super.mouseDragged(e);
    }
    
	
    public void mouseReleased( MapMouseEvent e ) {
    	if( getContext().getMapLayers().size()==0 )
    		return;
        FeatureReader reader = null;
        try {

            ILayer layer = getContext().getEditManager().getSelectedLayer();
            if (layer == null)
                layer = getContext().getMapLayers().get(0);

            if (layer == null)
                throw new Exception("No layers in map"); //$NON-NLS-1$

            Envelope env = getContext().getBoundingBox(e.getPoint(), 6);
            FeatureResults results = getContext().getFeaturesInBbox(
                    layer,
                    env);

            reader = results.reader();
            
            final boolean found=!reader.hasNext() ;
        	getContext().updateUI(new Runnable() {
				public void run() {
	            	IStatusLineManager bar = getContext().getStatusBar();
	            	if (bar!=null){
	            		if( found )
	            			bar.setErrorMessage( Policy.bind("DeleteTool.warning")); //$NON-NLS-1$
	            		else
	            			bar.setErrorMessage(null);
	            	}
				}
			});

            if (found)
                return;

            Feature feature=reader.next();
            DrawFeatureCommand command = getContext().getDrawFactory().createDrawFeatureCommand(
                    feature, layer);
            getContext().sendASyncCommand(command);
            getContext().sendASyncCommand(getContext().getEditFactory().createSetEditFeatureCommand(feature, layer));
            final ILayer editLayer = layer;
            final Feature editFeature = feature;
            PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable(){
                public void run() {
                    boolean result = MessageDialog.openConfirm(PlatformUI.getWorkbench()
                            .getDisplay().getActiveShell(), Policy
                            .bind("DeleteTool.confirmation.title"), //$NON-NLS-1$
                            Policy.bind("DeleteTool.confirmation.text")); //$NON-NLS-1$

                    if (result) {
                        MapCommand deleteFeatureCommand = getContext().getEditFactory()
                                .createDeleteFeature(editFeature, editLayer);
                        getContext().sendASyncCommand(deleteFeatureCommand);
                        try {
                        	IStatusLineManager bar = getContext().getStatusBar();
                        	if( bar!=null )
                        		bar.setErrorMessage(null);
//                            getContext().getRenderManager().refresh(editLayer, JTS.transform(editFeature.getBounds(), editLayer.layerToMapTransform()));
                        } catch (Exception e1) {
                            EditPlugin.log( "", e1);
                        }
                    }
                }
            });
            command.setValid(false);
            getContext().getViewportPane().repaint();
        } catch (Exception e1) {
        	EditPlugin.log( "", e1);
        } finally {
            try {
                if (reader != null)
                    reader.close();
            } catch (Exception e2) {
            	EditPlugin.log( "", e2);
            }
                draw.setValid( false ); // get us off the draw stack for context.getViewportPane().repaint();
                getContext().getViewportPane().repaint();
        }
    }

}
