/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.tools.edit;

import java.awt.Point;
import java.io.IOException;

import net.refractions.udig.core.internal.ExtensionPointList;
import net.refractions.udig.core.internal.FeatureUtils;
import net.refractions.udig.project.ILayer;
import net.refractions.udig.project.ui.internal.commands.draw.DrawEditFeatureCommand;
import net.refractions.udig.project.ui.internal.tool.display.ToolProxy;
import net.refractions.udig.project.ui.render.displayAdapter.MapMouseEvent;
import net.refractions.udig.project.ui.tool.IToolContext;
import net.refractions.udig.project.ui.tool.Tool;
import net.refractions.udig.tools.edit.Policy;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.geotools.data.FeatureStore;
import org.geotools.feature.Feature;
import org.geotools.geometry.JTS;
import org.opengis.referencing.crs.CoordinateReferenceSystem;
import org.opengis.referencing.operation.MathTransform;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.LinearRing;
import com.vividsolutions.jts.geom.MultiLineString;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Polygon;

/**
 * A Tool for manipulating geomtry vertices. This tool adds, moves and deletes vertices.
 * 
 * @author jeichar
 * @since 0.3
 */
public class FeatureEditTool extends AbstractEditTool {
	private int mode = -2;
	private Coordinate lastCoordinate=null;
    private ToolProxy.CursorLoader editCursor;
    private Cursor selectionCursor;
    
    
    /**
     * Creates a new instance of AddVertex
     */
    public FeatureEditTool() {
        ExtensionPointList list = new ExtensionPointList(Tool.EXTENSION_ID);
        for( IConfigurationElement element : list ) {
            if (element.getAttribute("id").equals("net.refractions.udig.tools.featureEdit")) { //$NON-NLS-1$ //$NON-NLS-2$
                IConfigurationElement[] cursorElement = element.getChildren("cursor"); //$NON-NLS-1$
                this.editCursor = new ToolProxy.CursorLoader(cursorElement[0]);
                break;
            }
        }
        ImageDescriptor desc=AbstractUIPlugin.imageDescriptorFromPlugin("net.refractions.udig.tool.default", //$NON-NLS-1$
                "/icons/pointers/point_select_source.png"); //$NON-NLS-1$
        Image image=desc.createImage();
        selectionCursor=new Cursor(Display.getDefault(), image.getImageData(), 5, 5);
        image.dispose();
    }
    
    @Override
    public void dispose() {
        super.dispose();
        editCursor.dispose();
        selectionCursor.dispose();
    }

    /**
     * @see net.refractions.udig.project.ui.tool.AbstractModalTool#setActive(boolean)
     */
    public void setActive( boolean active ) {
        super.setActive(active);
        if( active )
        	refresh();
        else{
			if( featureDrawCommand!=null )
				featureDrawCommand.setValid(false);
			getContext().getViewportPane().repaint();        	
        }
		if( !active && editing ){
			this.noContextMouseDoubleClicked(null);
		}
		if( active ){
			if( getContext().getEditManager().getEditFeature()!=null )
				setStatusBarMessage(Policy.bind("FeatureEditTool.moveMouse")); //$NON-NLS-1$
			else
				setStatusBarMessage(Policy.bind("FeatureEditTool.newFeature")); //$NON-NLS-1$
		}
		else
			setStatusBarMessage(""); //$NON-NLS-1$

    	editing=false;
    }

	private void setStatusBarMessage(final String message ) {
		if( getContext().getMapLayers().size()==0 )
			return;
		getContext().updateUI(new Runnable() {
			public void run() {
				IStatusLineManager bar = getContext().getStatusBar();
				if( bar!=null ){
					bar.setMessage(message);
				}
			}
		});
	}

    @Override
    public void setContext( IToolContext context ) {
        super.setContext(context);
    }
    
    DrawEditFeatureCommand featureDrawCommand;
    
    private void refresh() {
         if( featureDrawCommand==null ){
	         featureDrawCommand = getContext().getDrawFactory().createEditFeaturesCommand(
	        		 getContext().getViewportModel());
	         featureDrawCommand.setDrawKnots(true);
	         getContext().sendASyncCommand(featureDrawCommand);
         }
         if( !featureDrawCommand.isValid() ){
	         featureDrawCommand.setValid(true);
	         getContext().sendASyncCommand(featureDrawCommand);
         }
         
    }

	
    public void noContextMouseMoved( MapMouseEvent e ) {
        if( e.isControlDown() ){
            setCursor(selectionCursor);
            return;
        }

        if (getEditFeature() == null) {
            mode = -2;
            setCursor(editCursor.getCursor());
            return;
        }

        setMode(e);
    	refresh();
    }
//    private Feature getEditFeature() {
//        return getContext().getEditManager().getEditFeature();
//    }

    @Override
    public void noContextMouseDoubleClicked( MapMouseEvent e ) {
        getContext().sendASyncCommand(getContext().getEditFactory().createWriteEditFeatureCommand());
		editing=false;
        lastCoordinate=null;
    }
    public void noContextMouseDragged( MapMouseEvent e ) {
        if (e.button != MapMouseEvent.BUTTON1) {
            return;
        }
        if (getCoords() == null)
            return;
        if (mode >= 0) {
            Coordinate current = getContext().pixelToWorld(e.x, e.y);
            try {
                JTS.transform(current, current, getContext().getMap().getEditManager()
                        .getEditLayer().mapToLayerTransform());
            } catch (Exception e1) {
                EditPlugin.log( "", e1); //$NON-NLS-1$
            }
			
            getCoords()[mode].x = current.x;
            getCoords()[mode].y = current.y;
			
			// Polygons have the first and last point the same so both must be moved.
			if (getGeometry() instanceof Polygon || getGeometry() instanceof MultiPolygon)
				if( mode==0 ){
		            getCoords()[getCoords().length-1].x = current.x;
		            getCoords()[getCoords().length-1].y = current.y;					
				}else if ( mode==getCoords().length-1 ){
		            getCoords()[0].x = current.x;
		            getCoords()[0].y = current.y;
				}
            getContext().getViewportPane().repaint();
        }
    }
//    Geometry lastGeom;
//    Coordinate[] coords;
//    private Coordinate[] getCoords() {
//    	if( lastGeom==getGeometry()){
//    		return coords;
//    	}
//    	lastGeom=getGeometry();
//        if(lastGeom==null )
//            return null;
//        coords=lastGeom.getCoordinates();
//        return coords;
//    }

    public void noContextMouseReleased( MapMouseEvent e ) {
        try {
            if( getContext().getSelectedLayer().getResource(FeatureStore.class, new NullProgressMonitor())==null ){
                IStatusLineManager line = getContext().getStatusBar();
                if( line!=null )
                    line.setMessage(Policy.bind("FeatureEditTool.readOnly")); //$NON-NLS-1$
            }
        } catch (IOException e2) {
            IStatusLineManager line = getContext().getStatusBar();
            if( line!=null )
                line.setMessage(Policy.bind("FeatureEditTool.readOnly")); //$NON-NLS-1$
        }
    	if( getContext().getMapLayers().size()==0 )
    		return;
    		
        if (e.button == MapMouseEvent.BUTTON1  && !e.isControlDown()) {
			editing=true;
            try {
                Coordinate click = getContext().pixelToWorld(e.x, e.y);
                Coordinate transformedClick = getTransformedClick(click);
                // TODO
                if (mode == OVER_EDGE) {
                    int index = getClosest(transformedClick, getCoords());
                    Coordinate newCoord = closestPoint(getCoords()[index],
                            getCoords()[index == getCoords().length - 1 ? 0 : index + 1], transformedClick);
                    Geometry newgeom = createGeometry(getGeometry(), newCoord, getCoords()[index]);
                    getContext().getEditManager().getEditFeature().setDefaultGeometry(newgeom);
                } else if( mode == NO_INTERSECTION){
					if (getGeometry() != null && lastCoordinate!=null ) {
                        Geometry geometry;
                        if (getGeometry().getClass() == LineString.class
                                || getGeometry().getClass() == MultiLineString.class) {
                            geometry = createGeometry(getGeometry(), transformedClick,
                                    getCoords()[getCoords().length - 1]);
                        } else {
                            geometry = createGeometry(getGeometry(), transformedClick, lastCoordinate);
							lastCoordinate=transformedClick;
                        }
                        getContext().getEditManager().getEditFeature().setDefaultGeometry(geometry);
                    } else if( getGeometry() == null ){
                        ILayer editLayer = findEditLayer();
                        CoordinateReferenceSystem crs = getContext().getViewportModel()
                                .getCRS();
						try{
							transformedClick=JTS.transform(click, new Coordinate(),editLayer.mapToLayerTransform());
						}catch (Exception e1) {
							transformedClick=click;
						}
						lastCoordinate=transformedClick;
                        Feature newFeature = FeatureUtils.createFeature(crs, crs,
                                editLayer.getSchema(), new Coordinate[]{transformedClick},
                                editLayer.getSchema().getDefaultGeometry().getType());
                        if( editLayer.getSchema().getDefaultGeometry().getType() == com.vividsolutions.jts.geom.Point.class ){
                            getContext().sendSyncCommand(
                                    getContext().getEditFactory().createSetEditFeatureCommand(
                                            newFeature, editLayer));
                            getContext().sendASyncCommand(getContext().getEditFactory().createWriteEditFeatureCommand());
                            editing=false;
                            lastCoordinate=null;
                        } else {
                            getContext().sendSyncCommand(
                                getContext().getEditFactory().createSetEditFeatureCommand(
                                        newFeature, editLayer));
                        }
                    }
                }
                setMode(e);
            } catch (Exception exception) {
            	EditPlugin.log( "", exception); //$NON-NLS-1$
            }
        }else if( e.button==MapMouseEvent.BUTTON1 && e.isControlDown() ){
            getContext().sendASyncCommand(
                    getContext().getSelectionFactory().createBBoxSelectionCommand(
                    getContext().getBoundingBox(e.getPoint(), 3)) 
                    );
        }
    }

//    private Geometry getGeometry() {
//        if( getEditFeature()==null )
//            return null;
//        return getEditFeature().getDefaultGeometry();
//    }

    private void setMode( MapMouseEvent e ) {
    	if( getEditFeature()==null){
            mode = NO_INTERSECTION;
            setCursor(editCursor.getCursor());
			setStatusBarMessage(Policy.bind("FeatureEditTool.newFeature")); //$NON-NLS-1$
			return;
    	}
        Point p = e.getPoint();
        Envelope env = getContext().getBoundingBox(p, 5);
        try {
            ILayer layer = getContext().getMap().getEditManager().getEditLayer();
            MathTransform mt = layer.mapToLayerTransform();
            env = JTS.transform(env, mt);
        } catch (Exception e1) {
        	EditPlugin.log( "", e1); //$NON-NLS-1$
        }
        int hit = overVertext(env);
        if (hit != OVER_EDGE) {
            if (mode < 0) {
                mode = hit;
                setCursor(SWT.CURSOR_SIZEALL);
                setStatusBarMessage(Policy.bind("FeatureEditTool.moveVertex")); //$NON-NLS-1$
            }
        } else if (overFeature(env.getMinX(), env.getMinY(), env.getMaxX(), env.getMaxY())) {
            if (mode != OVER_EDGE) {
                mode = OVER_EDGE;
                setCursor(SWT.CURSOR_CROSS);
                setStatusBarMessage(Policy.bind("FeatureEditTool.addVertex")); //$NON-NLS-1$
            }
        } else {
            if (mode != NO_INTERSECTION) {
                mode = NO_INTERSECTION;
                setCursor(editCursor.getCursor());
                if( editing )
                    setStatusBarMessage(Policy.bind("FeatureEditTool.addVertex")); //$NON-NLS-1$
                else
                	setStatusBarMessage(Policy.bind("FeatureEditTool.moveMouse")); //$NON-NLS-1$
            }
        }

        getContext().getViewportPane().repaint();
    }

    private void setCursor( final int cursor_id ) {
        final Display display = PlatformUI.getWorkbench().getDisplay();
        display.asyncExec(new Runnable(){

            public void run() {
                setCursor(display.getSystemCursor(cursor_id));
            }
        });

    }
    void setCursor( final Cursor cursor ) {
        final Display display = PlatformUI.getWorkbench().getDisplay();
        display.asyncExec(new Runnable(){

            public void run() {
                getContext().getViewportPane().setCursor(cursor);
            }
        });
    }

//    private int overVertext( Envelope env ) {
//        if( getCoords()==null )
//            return NO_INTERSECTION;
//        for( int i = 0; i < getCoords().length; i++ ) {
//            Coordinate coord = getCoords()[i];
//            if (env.contains(coord))
//                return i;
//        }
//        return OVER_EDGE;
//    }
//    private boolean overFeature( double xmin, double ymin, double xmax, double ymax ) {
//        Coordinate coord0 = getCoords()[getCoords().length - 1];
//        for( int i = 0; i < getCoords().length; i++ ) {
//            Coordinate coord1 = getCoords()[i];
//            if (overLine(coord0.x, coord0.y, coord1.x, coord1.y, xmin, ymin, xmax, ymax))
//                return true;
//            coord0 = coord1;
//        }
//        return false;
//    }
//    private boolean overLine( double x0, double y0, double x1, double y1, double xmin, double ymin,
//            double xmax, double ymax ) {
//        boolean accept = false, done = false;
//        int outcode0 = compOutCode(x0, y0, xmin, xmax, ymin, ymax);
//        int outcode1 = compOutCode(x1, y1, xmin, xmax, ymin, ymax);
//        do {
//            if (outcode0 == 0 || outcode1 == 0)
//                accept = done = true;
//            else if ((outcode0 & outcode1) != 0)
//                done = true;
//            else {
//                double x, y;
//                int outcodeOut = outcode0 > 0 ? outcode0 : outcode1;
//                if ((outcodeOut & TOP) > 0) {
//                    x = x0 + (x1 - x0) * (ymax - y0) / (y1 - y0);
//                    y = ymax;
//                } else if ((outcodeOut & BOTTOM) > 0) {
//                    x = x0 + (x1 - x0) * (ymin - y0) / (y1 - y0);
//                    y = ymin;
//                } else if ((outcodeOut & RIGHT) > 0) {
//                    y = y0 + (y1 - y0) * (xmax - x0) / (x1 - x0);
//                    x = xmax;
//                } else {
//                    y = y0 + (y1 - y0) * (xmin - x0) / (x1 - x0);
//                    x = xmin;
//                }
//                if (outcodeOut == outcode0) {
//                    x0 = x;
//                    y0 = y;
//                    outcode0 = compOutCode(x0, y0, xmin, xmax, ymin, ymax);
//                } else {
//                    x1 = x;
//                    y1 = y;
//                    outcode1 = compOutCode(x1, y1, xmin, xmax, ymin, ymax);
//                }
//            }
//        } while( done == false );
//        return accept;
//    }

//    int TOP = 0x8, BOTTOM = 0x4, RIGHT = 0x2, LEFT = 0x1;
//    private static final int OVER_EDGE = -1;
//    private static final int NO_INTERSECTION = -2;
//	private boolean editing;

//    private int compOutCode( double x, double y, double xmin, double xmax, double ymin, double ymax ) {
//        int outcode = 0;
//        if (y > ymax)
//            outcode |= TOP;
//        if (y < ymin)
//            outcode |= BOTTOM;
//        if (x > xmax)
//            outcode |= RIGHT;
//        if (x < xmin)
//            outcode |= LEFT;
//        return outcode;
//    }
//    private Coordinate closestPoint( Coordinate end1, Coordinate end2, Coordinate p ) {
//        Coordinate v = new Coordinate();
//        v.x = end2.x - end1.x;
//        v.y = end2.y - end1.y;
//        double t = ((p.x - end1.x) * v.x + (p.y - end1.y) * v.y) / (v.x * v.x + v.y * v.y);
//        if (t > 1 || t < 0 || Double.isInfinite(t) || Double.isNaN(t))
//            return null;
//        Coordinate result = new Coordinate();
//        result.x = end1.x + t * v.x;
//        result.y = end1.y + t * v.y;
//        return result;
//    }

    int getClosest( Coordinate click, Coordinate[] coordinates ) {
        int prev = 0;
        double mindist = Double.MAX_VALUE;
        Coordinate coord1 = coordinates[coordinates.length - 1];
        for( int i = 0; i < coordinates.length; i++ ) {
            Coordinate coord2 = coordinates[i];
            Coordinate point = closestPoint(coord1, coord2, click);
            if (point == null) {
                coord1 = coord2;
                continue;
            }
            double x = click.x - point.x;
            double y = click.y - point.y;
            double dist = Math.sqrt(x * x + y * y);
            if (dist < mindist) {
                mindist = dist;
                prev = i;
            }
            coord1 = coord2;
        }
        if (getGeometry() instanceof LinearRing) {
            return (prev == 0 || prev == coordinates.length) ? coordinates.length - 2 : prev;
        }
        return prev != 0 ? prev - 1 : coordinates.length - 2;
    }

//    private ILayer findEditLayer() {
//        ILayer layer = null;
//        IMap map = getContext().getMap();
//        //The selected layer will become the edit layer! If it's a featureStore and editable of course.
//        ILayer selectedLayer = ((EditManagerImpl)map.getEditManager()).getSelectedLayer();
//        if ( selectedLayer != null && selectedLayer.isType(FeatureStore.class) 
//                && selectedLayer.isApplicable("editing") ) { //$NON-NLS-1$
//            return selectedLayer;
//        }
//        //Otherwise we'll fall back to the current editLayer
//        if (map.getEditManager().getEditLayer() != null)
//            return map.getEditManager().getEditLayer();
//        //If all else fails we'll iterate through the layer and find the first layer eligable for editing.
//        //We should really never get here though.
//        for( Iterator<ILayer> iter = map.getMapLayers().iterator(); iter.hasNext(); ) {
//            layer = iter.next();
//            if (layer.isType(FeatureStore.class) && layer.isApplicable("editing") //$NON-NLS-1$
//                    && layer.isVisible())
//                break;
//        }
//        return layer;
//    }

//    private Coordinate getTransformedClick( Coordinate click ) {
//
//        try {
//            MathTransform transform = getContext().getEditManager().getEditLayer()
//                    .mapToLayerTransform();
//            if (transform == null || transform.isIdentity())
//                return click;
//            return JTS.transform(click, new Coordinate(), transform);
//        } catch (Exception e1) {
//            // CorePlugin.log(ToolsPlugin.getDefault(), e1);
//            return click;
//        }
//    }

}