/*
 * uDig - User Friendly Desktop Internet GIS client http://udig.refractions.net (C) 2004,
 * Refractions Research Inc. This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; version 2.1 of the License. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 */
package net.refractions.udig.ui.palette;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * <p>A colour scheme from a palette.
 * A colour scheme has a collection of AWT Colors
 * and a collection of suitability values for each colour
 * </p>
 * @author ptozer
 *
 */
public class ColourScheme {
	private HashMap<String,Color> allColours;//colour letter, awt Color
	private HashMap<String, ArrayList<String>> allSuitability; //colour letter, arrayList of suitability strings
	private String schemeName;
    private ColourPalette owner;
	
	public ColourScheme ( ColourPalette owner, String name) {
        allColours = new HashMap<String,Color>();
        allSuitability = new HashMap<String, ArrayList<String>>();
        this.owner=owner;
        schemeName = name;
        owner.addColourScheme(schemeName, this);
	}
	
	/**
	 * Adds a colour to the scheme.
	 * @param letter the letter value associated with this particular colour
	 * @param rgb integer array of red, green and blue values for the colour
	 */
	public void addColour(String letter, int [] rgb) {
		allColours.put(letter, new Color(rgb[0], rgb[1], rgb[2]));
	}
	
	/**
	 * Adds suitability to the scheme by colour letter.
	 * @param letter letter the letter value associated with this particular colour
	 * @param suitability an array of String valuesindicating suitability for different projection methods, etc
	 */
	public void addSuitability(String letter, String[] suitability) {
		
		ArrayList<String> array = new ArrayList<String>();
		for(int i = 0; i < suitability.length; i++) {
			array.add((suitability[i]).trim());
		}
		allSuitability.put(letter, array);
	}
	
	public ArrayList<String>  getSuitability(String colourLetter) {
		return allSuitability.get(colourLetter);
	}
	
	public Color getColour(String letter) {
		return allColours.get(letter);
	}

	/**
	 * @return Returns the schemeName.
	 */
	public String getSchemeName() {
		return schemeName;
	}
	/**
	 * @param schemeName The schemeName to set.
	 */
	public void setSchemeName(String schemeName) {
		this.schemeName = schemeName;
	}
	/**
	 * @return Returns the allColours.
	 */
	public HashMap<String, Color> getAllColours() {
		return allColours;
	}
	/**
	 * @return Returns the allSuitability.
	 */
	public HashMap<String, ArrayList<String>> getAllSuitability() {
		return allSuitability;
	}
    /**
     * Equals method for ColourScheme.
     * @param schemeToCompare
     * @return true if the colour schemes match, false otherwise
     */
    public boolean equals(ColourScheme schemeToCompare) {
        String otherSchemeName = schemeToCompare.getSchemeName();
        if(this.schemeName.equalsIgnoreCase(otherSchemeName)){
            //TODO: add iteration through colours for complete sameness(?)
            return true;
        }
        return false;
    }

    public ColourPalette getColourPalette() {
        return owner;
    }
    
}
